@@ -12,21 +12,21 @@
 
 import ast
 
-from neutron.db import api as db_api
+from neutron.common import log
 from neutron.db import common_db_mixin
-from neutron_lib.db import model_base
-from neutron_lib import exceptions as n_exc
-from neutron_lib.plugins import constants as pconst
-from neutron_lib.plugins import directory
-from oslo_log import helpers as log
+from neutron.db import model_base
+from neutron.db import models_v2
+from neutron import manager
+from neutron.openstack.common import uuidutils
+from neutron.plugins.common import constants as pconst
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
-from oslo_utils import uuidutils
 import sqlalchemy as sa
 from sqlalchemy.ext.orderinglist import ordering_list
 from sqlalchemy import orm
 from sqlalchemy.orm import exc
 
+from gbpservice.neutron.db import gbp_quota_db as gquota
 from gbpservice.neutron.extensions import servicechain as schain
 from gbpservice.neutron.services.servicechain.common import exceptions as s_exc
 
@@ -36,17 +36,6 @@ MAX_IPV4_SUBNET_PREFIX_LENGTH = 31
 MAX_IPV6_SUBNET_PREFIX_LENGTH = 127
 
 
-class BaseSCResource(model_base.HasId, model_base.HasProject):
-    name = sa.Column(sa.String(255))
-    description = sa.Column(sa.String(255))
-    status = sa.Column(sa.String(length=16), nullable=True)
-    status_details = sa.Column(sa.String(length=4096), nullable=True)
-
-
-class BaseSharedSCResource(BaseSCResource):
-    shared = sa.Column(sa.Boolean)
-
-
 class SpecNodeAssociation(model_base.BASEV2):
     """Models one to many providing relation between Specs and Nodes."""
     __tablename__ = 'sc_spec_node_associations'
@@ -69,22 +58,29 @@ class InstanceSpecAssociation(model_base.BASEV2):
     position = sa.Column(sa.Integer)
 
 
-class ServiceChainNode(model_base.BASEV2, BaseSharedSCResource):
+class ServiceChainNode(gquota.GBPQuotaBase, model_base.BASEV2, models_v2.HasId,
+                       models_v2.HasTenant):
     """ServiceChain Node"""
     __tablename__ = 'sc_nodes'
-    config = sa.Column(sa.TEXT)
+    name = sa.Column(sa.String(50))
+    description = sa.Column(sa.String(255))
+    config = sa.Column(sa.String(4096))
     specs = orm.relationship(SpecNodeAssociation,
                              backref="nodes",
                              cascade='all, delete, delete-orphan')
+    shared = sa.Column(sa.Boolean)
     service_type = sa.Column(sa.String(50), nullable=True)
     service_profile_id = sa.Column(
         sa.String(36), sa.ForeignKey('service_profiles.id'),
         nullable=True)
 
 
-class ServiceChainInstance(model_base.BASEV2, BaseSCResource):
+class ServiceChainInstance(gquota.GBPQuotaBase, model_base.BASEV2,
+                           models_v2.HasId, models_v2.HasTenant):
     """Service chain instances"""
     __tablename__ = 'sc_instances'
+    name = sa.Column(sa.String(50))
+    description = sa.Column(sa.String(255))
     config_param_values = sa.Column(sa.String(4096))
     specs = orm.relationship(
         InstanceSpecAssociation,
@@ -107,10 +103,13 @@ class ServiceChainInstance(model_base.BASEV2, BaseSCResource):
                               nullable=True)
 
 
-class ServiceChainSpec(model_base.BASEV2, BaseSharedSCResource):
+class ServiceChainSpec(gquota.GBPQuotaBase, model_base.BASEV2, models_v2.HasId,
+                       models_v2.HasTenant):
     """ ServiceChain Spec
     """
     __tablename__ = 'sc_specs'
+    name = sa.Column(sa.String(50))
+    description = sa.Column(sa.String(255))
     nodes = orm.relationship(
         SpecNodeAssociation,
         backref='specs', cascade='all, delete, delete-orphan',
@@ -120,13 +119,18 @@ class ServiceChainSpec(model_base.BASEV2, BaseSharedSCResource):
     instances = orm.relationship(InstanceSpecAssociation,
                                  backref="specs",
                                  cascade='all, delete, delete-orphan')
+    shared = sa.Column(sa.Boolean)
 
 
-class ServiceProfile(model_base.BASEV2, BaseSharedSCResource):
+class ServiceProfile(gquota.GBPQuotaBase, model_base.BASEV2, models_v2.HasId,
+                     models_v2.HasTenant):
     """ Service Profile
     """
     __tablename__ = 'service_profiles'
+    name = sa.Column(sa.String(50))
+    description = sa.Column(sa.String(255))
     vendor = sa.Column(sa.String(50))
+    shared = sa.Column(sa.Boolean)
     # Not using ENUM for less painful upgrades. Validation will happen at the
     # API level
     insertion_mode = sa.Column(sa.String(50))
@@ -135,6 +139,16 @@ class ServiceProfile(model_base.BASEV2, BaseSharedSCResource):
     nodes = orm.relationship(ServiceChainNode, backref="service_profile")
 
 
+gquota.DB_CLASS_TO_RESOURCE_NAMES[ServiceChainNode.__name__] = (
+    'servicechain_node')
+gquota.DB_CLASS_TO_RESOURCE_NAMES[ServiceChainSpec.__name__] = (
+    'servicechain_spec')
+gquota.DB_CLASS_TO_RESOURCE_NAMES[ServiceChainInstance.__name__] = (
+    'servicechain_instance')
+gquota.DB_CLASS_TO_RESOURCE_NAMES[ServiceProfile.__name__] = (
+    'service_profile')
+
+
 class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
                            common_db_mixin.CommonDbMixin):
     """ServiceChain plugin interface implementation using SQLAlchemy models."""
@@ -151,27 +165,13 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
     def _grouppolicy_plugin(self):
         # REVISIT(Magesh): Need initialization method after all
         # plugins are loaded to grab and store plugin.
-        grouppolicy_plugin = directory.get_plugin(pconst.GROUP_POLICY)
+        plugins = manager.NeutronManager.get_service_plugins()
+        grouppolicy_plugin = plugins.get(pconst.GROUP_POLICY)
         if not grouppolicy_plugin:
-            LOG.error("No Grouppolicy service plugin found.")
+            LOG.error(_("No Grouppolicy service plugin found."))
             raise s_exc.ServiceChainDeploymentError()
         return grouppolicy_plugin
 
-    # REVISIT: This is temporary, the correct fix is to use the
-    # project_id in the context. Moreover, patch.py already patches
-    # thi, so it should not be required here.
-    def _get_tenant_id_for_create(self, context, resource):
-        if context.is_admin and 'tenant_id' in resource:
-            tenant_id = resource['tenant_id']
-            return tenant_id
-        elif ('tenant_id' in resource and
-              resource['tenant_id'] != context.tenant_id):
-            reason = _('Cannot create resource for another tenant')
-            raise n_exc.AdminRequired(reason=reason)
-        else:
-            tenant_id = context.tenant_id
-            return tenant_id
-
     def _get_servicechain_node(self, context, node_id):
         try:
             return self._get_by_id(context, ServiceChainNode, node_id)
@@ -198,28 +198,26 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
             raise schain.ServiceProfileNotFound(
                 profile_id=profile_id)
 
-    def _populate_common_fields_in_dict(self, db_ref):
-        res = {'id': db_ref['id'],
-               'tenant_id': db_ref['tenant_id'],
-               'name': db_ref['name'],
-               'description': db_ref['description'],
-               'status': db_ref['status'],
-               'status_details': db_ref['status_details'],
-               'shared': db_ref.get('shared', False)}
-        return res
-
     def _make_sc_node_dict(self, sc_node, fields=None):
-        res = self._populate_common_fields_in_dict(sc_node)
-        res['service_profile_id'] = sc_node['service_profile_id']
-        res['service_type'] = sc_node['service_type']
-        res['config'] = sc_node['config']
+        res = {'id': sc_node['id'],
+               'tenant_id': sc_node['tenant_id'],
+               'name': sc_node['name'],
+               'description': sc_node['description'],
+               'service_profile_id': sc_node['service_profile_id'],
+               'service_type': sc_node['service_type'],
+               'config': sc_node['config'],
+               'shared': sc_node['shared']}
         res['servicechain_specs'] = [sc_spec['servicechain_spec_id']
                                      for sc_spec in sc_node['specs']]
         return self._fields(res, fields)
 
     def _make_sc_spec_dict(self, spec, fields=None):
-        res = self._populate_common_fields_in_dict(spec)
-        res['config_param_names'] = spec.get('config_param_names')
+        res = {'id': spec['id'],
+               'tenant_id': spec['tenant_id'],
+               'name': spec['name'],
+               'description': spec['description'],
+               'config_param_names': spec.get('config_param_names'),
+               'shared': spec['shared']}
         res['nodes'] = [sc_node['node_id'] for sc_node in spec['nodes']]
         res['instances'] = [x['servicechain_instance_id'] for x in
                             spec['instances']]
@@ -234,19 +232,21 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
                'provider_ptg_id': instance['provider_ptg_id'],
                'consumer_ptg_id': instance['consumer_ptg_id'],
                'management_ptg_id': instance['management_ptg_id'],
-               'classifier_id': instance['classifier_id'],
-               'status': instance['status'],
-               'status_details': instance['status_details']}
+               'classifier_id': instance['classifier_id']}
         res['servicechain_specs'] = [sc_spec['servicechain_spec_id']
                                     for sc_spec in instance['specs']]
         return self._fields(res, fields)
 
     def _make_service_profile_dict(self, profile, fields=None):
-        res = self._populate_common_fields_in_dict(profile)
-        res['service_type'] = profile['service_type']
-        res['service_flavor'] = profile['service_flavor']
-        res['vendor'] = profile['vendor']
-        res['insertion_mode'] = profile['insertion_mode']
+        res = {'id': profile['id'],
+               'tenant_id': profile['tenant_id'],
+               'name': profile['name'],
+               'description': profile['description'],
+               'shared': profile['shared'],
+               'service_type': profile['service_type'],
+               'service_flavor': profile['service_flavor'],
+               'vendor': profile['vendor'],
+               'insertion_mode': profile['insertion_mode']}
         res['nodes'] = [node['id'] for node in profile['nodes']]
         return self._fields(res, fields)
 
@@ -255,27 +255,25 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
         if service_type not in schain.sc_supported_type:
             raise schain.ServiceTypeNotSupported(sc_service_type=service_type)
 
-    @log.log_method_call
+    @log.log
     def create_servicechain_node(self, context, servicechain_node):
         node = servicechain_node['servicechain_node']
         tenant_id = self._get_tenant_id_for_create(context, node)
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             node_db = ServiceChainNode(
                 id=uuidutils.generate_uuid(), tenant_id=tenant_id,
                 name=node['name'], description=node['description'],
-                service_profile_id=node.get('service_profile_id'),
-                service_type=node.get('service_type'),
-                config=node['config'], shared=node['shared'],
-                status=node.get('status'),
-                status_details=node.get('status_details'))
+                service_profile_id=node['service_profile_id'],
+                service_type=node['service_type'],
+                config=node['config'], shared=node['shared'])
             context.session.add(node_db)
-            return self._make_sc_node_dict(node_db)
+        return self._make_sc_node_dict(node_db)
 
-    @log.log_method_call
+    @log.log
     def update_servicechain_node(self, context, servicechain_node_id,
                                  servicechain_node, set_params=False):
         node = servicechain_node['servicechain_node']
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             node_db = self._get_servicechain_node(context,
                                                   servicechain_node_id)
             node_db.update(node)
@@ -287,11 +285,11 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
                 self._process_nodes_for_spec(
                     context, spec_db, self._make_sc_spec_dict(spec_db),
                     set_params=set_params)
-            return self._make_sc_node_dict(node_db)
+        return self._make_sc_node_dict(node_db)
 
-    @log.log_method_call
+    @log.log
     def delete_servicechain_node(self, context, servicechain_node_id):
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             node_db = self._get_servicechain_node(context,
                                                   servicechain_node_id)
             if node_db.specs:
@@ -299,13 +297,13 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
                                     node_id=servicechain_node_id)
             context.session.delete(node_db)
 
-    @log.log_method_call
+    @log.log
     def get_servicechain_node(self, context, servicechain_node_id,
                               fields=None):
         node = self._get_servicechain_node(context, servicechain_node_id)
         return self._make_sc_node_dict(node, fields)
 
-    @log.log_method_call
+    @log.log
     def get_servicechain_nodes(self, context, filters=None, fields=None,
                                sorts=None, limit=None, marker=None,
                                page_reverse=False):
@@ -318,7 +316,7 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
                                     marker_obj=marker_obj,
                                     page_reverse=page_reverse)
 
-    @log.log_method_call
+    @log.log
     def get_servicechain_nodes_count(self, context, filters=None):
         return self._get_collection_count(context, ServiceChainNode,
                                           filters=filters)
@@ -421,57 +419,54 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
                     result.append(sci)
             return result
 
-    @log.log_method_call
+    @log.log
     def create_servicechain_spec(self, context, servicechain_spec,
                                  set_params=True):
         spec = servicechain_spec['servicechain_spec']
         tenant_id = self._get_tenant_id_for_create(context, spec)
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             spec_db = ServiceChainSpec(id=uuidutils.generate_uuid(),
                                        tenant_id=tenant_id,
                                        name=spec['name'],
                                        description=spec['description'],
-                                       shared=spec['shared'],
-                                       status=spec.get('status'),
-                                       status_details=
-                                       spec.get('status_details'))
+                                       shared=spec['shared'])
             self._process_nodes_for_spec(context, spec_db, spec,
                                          set_params=set_params)
             context.session.add(spec_db)
-            return self._make_sc_spec_dict(spec_db)
+        return self._make_sc_spec_dict(spec_db)
 
-    @log.log_method_call
+    @log.log
     def update_servicechain_spec(self, context, spec_id,
                                  servicechain_spec, set_params=True):
         spec = servicechain_spec['servicechain_spec']
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             spec_db = self._get_servicechain_spec(context,
                                                   spec_id)
             spec = self._process_nodes_for_spec(context, spec_db, spec,
                                                 set_params=set_params)
             spec_db.update(spec)
-            return self._make_sc_spec_dict(spec_db)
+        return self._make_sc_spec_dict(spec_db)
 
-    @log.log_method_call
+    @log.log
     def delete_servicechain_spec(self, context, spec_id):
         policy_actions = self._grouppolicy_plugin.get_policy_actions(
                                 context, filters={"action_value": [spec_id]})
         if policy_actions:
             raise schain.ServiceChainSpecInUse(spec_id=spec_id)
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             spec_db = self._get_servicechain_spec(context,
                                                   spec_id)
             if spec_db.instances:
                 raise schain.ServiceChainSpecInUse(spec_id=spec_id)
             context.session.delete(spec_db)
 
-    @log.log_method_call
+    @log.log
     def get_servicechain_spec(self, context, spec_id,
                               fields=None):
         spec = self._get_servicechain_spec(context, spec_id)
         return self._make_sc_spec_dict(spec, fields)
 
-    @log.log_method_call
+    @log.log
     def get_servicechain_specs(self, context, filters=None, fields=None,
                                sorts=None, limit=None, marker=None,
                                page_reverse=False):
@@ -484,21 +479,21 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
                                     marker_obj=marker_obj,
                                     page_reverse=page_reverse)
 
-    @log.log_method_call
+    @log.log
     def get_servicechain_specs_count(self, context, filters=None):
         return self._get_collection_count(context, ServiceChainSpec,
                                           filters=filters)
 
-    @log.log_method_call
+    @log.log
     def create_servicechain_instance(self, context, servicechain_instance):
         instance = servicechain_instance['servicechain_instance']
         tenant_id = self._get_tenant_id_for_create(context, instance)
-        with db_api.context_manager.writer.using(context):
-            if not instance.get('management_ptg_id'):
+        with context.session.begin(subtransactions=True):
+            if not instance['management_ptg_id']:
                 management_groups = (
                     self._grouppolicy_plugin.get_policy_target_groups(
                         context, {'service_management': [True],
-                                  'tenant_id': [instance.get('tenant_id')]}))
+                                  'tenant_id': [instance['tenant_id']]}))
                 if not management_groups:
                     # Fall back on shared service management
                     management_groups = (
@@ -511,41 +506,39 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
                 tenant_id=tenant_id, name=instance['name'],
                 description=instance['description'],
                 config_param_values=instance['config_param_values'],
-                provider_ptg_id=instance.get('provider_ptg_id'),
-                consumer_ptg_id=instance.get('consumer_ptg_id'),
-                management_ptg_id=instance.get('management_ptg_id'),
-                classifier_id=instance.get('classifier_id'),
-                status=instance.get('status'),
-                status_details=instance.get('status_details'))
+                provider_ptg_id=instance['provider_ptg_id'],
+                consumer_ptg_id=instance['consumer_ptg_id'],
+                management_ptg_id=instance['management_ptg_id'],
+                classifier_id=instance['classifier_id'])
             self._process_specs_for_instance(context, instance_db, instance)
             context.session.add(instance_db)
-            return self._make_sc_instance_dict(instance_db)
+        return self._make_sc_instance_dict(instance_db)
 
-    @log.log_method_call
+    @log.log
     def update_servicechain_instance(self, context, servicechain_instance_id,
                                      servicechain_instance):
         instance = servicechain_instance['servicechain_instance']
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             instance_db = self._get_servicechain_instance(
                 context, servicechain_instance_id)
             instance = self._process_specs_for_instance(context, instance_db,
                                                         instance)
             instance_db.update(instance)
-            return self._make_sc_instance_dict(instance_db)
+        return self._make_sc_instance_dict(instance_db)
 
-    @log.log_method_call
+    @log.log
     def delete_servicechain_instance(self, context, servicechain_instance_id):
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             instance_db = self._get_servicechain_instance(
                 context, servicechain_instance_id)
             context.session.delete(instance_db)
 
-    @log.log_method_call
+    @log.log
     def get_servicechain_instance(self, context, sc_instance_id, fields=None):
         instance_db = self._get_servicechain_instance(context, sc_instance_id)
         return self._make_sc_instance_dict(instance_db, fields)
 
-    @log.log_method_call
+    @log.log
     def get_servicechain_instances(self, context, filters=None, fields=None,
                                    sorts=None, limit=None, marker=None,
                                    page_reverse=False):
@@ -558,47 +551,45 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
                                     marker_obj=marker_obj,
                                     page_reverse=page_reverse)
 
-    @log.log_method_call
+    @log.log
     def get_servicechain_instances_count(self, context, filters=None):
         return self._get_collection_count(context, ServiceChainInstance,
                                           filters=filters)
 
-    @log.log_method_call
+    @log.log
     def get_service_profiles_count(self, context, filters=None):
         return self._get_collection_count(context, ServiceProfile,
                                           filters=filters)
 
-    @log.log_method_call
+    @log.log
     def create_service_profile(self, context, service_profile):
         profile = service_profile['service_profile']
         tenant_id = self._get_tenant_id_for_create(context, profile)
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             profile_db = ServiceProfile(
                 id=uuidutils.generate_uuid(), tenant_id=tenant_id,
                 name=profile['name'], description=profile['description'],
-                service_type=profile.get('service_type'),
-                insertion_mode=profile.get('insertion_mode'),
-                vendor=profile.get('vendor'),
-                service_flavor=profile.get('service_flavor'),
-                shared=profile.get('shared'),
-                status=profile.get('status'),
-                status_details=profile.get('status_details'))
+                service_type=profile['service_type'],
+                insertion_mode=profile['insertion_mode'],
+                vendor=profile['vendor'],
+                service_flavor=profile['service_flavor'],
+                shared=profile['shared'])
             context.session.add(profile_db)
-            return self._make_service_profile_dict(profile_db)
+        return self._make_service_profile_dict(profile_db)
 
-    @log.log_method_call
+    @log.log
     def update_service_profile(self, context, service_profile_id,
                                service_profile):
         profile = service_profile['service_profile']
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             profile_db = self._get_service_profile(context,
                                                    service_profile_id)
             profile_db.update(profile)
-            return self._make_service_profile_dict(profile_db)
+        return self._make_service_profile_dict(profile_db)
 
-    @log.log_method_call
+    @log.log
     def delete_service_profile(self, context, service_profile_id):
-        with db_api.context_manager.writer.using(context):
+        with context.session.begin(subtransactions=True):
             profile_db = self._get_service_profile(context,
                                                    service_profile_id)
             if profile_db.nodes:
@@ -606,13 +597,13 @@ class ServiceChainDbPlugin(schain.ServiceChainPluginBase,
                     profile_id=service_profile_id)
             context.session.delete(profile_db)
 
-    @log.log_method_call
+    @log.log
     def get_service_profile(self, context, service_profile_id, fields=None):
         profile_db = self._get_service_profile(
             context, service_profile_id)
         return self._make_service_profile_dict(profile_db, fields)
 
-    @log.log_method_call
+    @log.log
     def get_service_profiles(self, context, filters=None, fields=None,
                              sorts=None, limit=None, marker=None,
                              page_reverse=False):
