@@ -12,28 +12,30 @@
 
 import abc
 
-from neutron.api import extensions as neutron_extensions
+from neutron.api import extensions
 from neutron.api.v2 import attributes as attr
 from neutron.api.v2 import resource_helper
-from neutron_lib.api import converters as conv
-from neutron_lib.api import extensions
-from neutron_lib.api import validators as valid
-from neutron_lib import exceptions as nexc
-from neutron_lib.plugins import constants
-from neutron_lib.services import base as service_base
+from neutron.common import exceptions as nexc
+from neutron.common import log
+from neutron.plugins.common import constants
+from neutron import quota
+from neutron.services import service_base
 from oslo_config import cfg
 from oslo_log import log as logging
 import six
 
-from gbpservice.neutron import extensions as gbp_extensions
-import gbpservice.neutron.extensions.group_policy  # noqa
+import gbpservice.neutron.extensions
 from gbpservice.neutron.services.servicechain.common import constants as scc
 
 
 # The code below is a monkey patch of key Neutron's modules. This is needed for
 # the GBP service to be loaded correctly. GBP extensions' path is added
 # to Neutron's so that it's found at extension scanning time.
-neutron_extensions.append_api_extensions_path(gbp_extensions.__path__)
+
+extensions.append_api_extensions_path(gbpservice.neutron.extensions.__path__)
+constants.SERVICECHAIN = "SERVICECHAIN"
+constants.COMMON_PREFIXES["SERVICECHAIN"] = "/servicechain"
+
 LOG = logging.getLogger(__name__)
 
 
@@ -88,7 +90,7 @@ def _validate_str_list(data, valid_values=None):
         return msg
 
     for item in data:
-        msg = valid.validate_string(item)
+        msg = attr._validate_string(item)
         if msg:
             LOG.debug(msg)
             return msg
@@ -99,7 +101,7 @@ def _validate_str_list(data, valid_values=None):
         return msg
 
 
-valid.validators['type:string_list'] = _validate_str_list
+attr.validators['type:string_list'] = _validate_str_list
 
 SERVICECHAIN_NODES = 'servicechain_nodes'
 SERVICECHAIN_SPECS = 'servicechain_specs'
@@ -112,7 +114,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                'validate': {'type:uuid': None}, 'is_visible': True,
                'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:gbp_resource_name': None}, 'default': '',
+                 'validate': {'type:string': None}, 'default': '',
                  'is_visible': True},
         'description': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:string': None},
@@ -120,10 +122,6 @@ RESOURCE_ATTRIBUTE_MAP = {
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:string': None},
                       'required_by_policy': True, 'is_visible': True},
-        'status': {'allow_post': False, 'allow_put': False,
-                   'is_visible': True},
-        'status_details': {'allow_post': False, 'allow_put': False,
-                           'is_visible': True},
         'service_type': {'allow_post': True, 'allow_put': False,
                          'validate': {'type:string_or_none': None},
                          'is_visible': True, 'default': None},
@@ -134,7 +132,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                    'validate': {'type:string': None},
                    'required': True, 'is_visible': True},
         attr.SHARED: {'allow_post': True, 'allow_put': True,
-                      'default': False, 'convert_to': conv.convert_to_boolean,
+                      'default': False, 'convert_to': attr.convert_to_boolean,
                       'is_visible': True, 'required_by_policy': True,
                       'enforce_policy': True},
     },
@@ -143,7 +141,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                'validate': {'type:uuid': None}, 'is_visible': True,
                'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:gbp_resource_name': None},
+                 'validate': {'type:string': None},
                  'default': '', 'is_visible': True},
         'description': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:string': None},
@@ -151,20 +149,16 @@ RESOURCE_ATTRIBUTE_MAP = {
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:string': None},
                       'required_by_policy': True, 'is_visible': True},
-        'status': {'allow_post': False, 'allow_put': False,
-                   'is_visible': True},
-        'status_details': {'allow_post': False, 'allow_put': False,
-                           'is_visible': True},
         'nodes': {'allow_post': True, 'allow_put': True,
                   'validate': {'type:uuid_list': None},
-                  'convert_to': conv.convert_none_to_empty_list,
+                  'convert_to': attr.convert_none_to_empty_list,
                   'default': None, 'is_visible': True,
                   'required': True},
         'config_param_names': {'allow_post': False, 'allow_put': False,
                                'validate': {'type:string_list': None},
                                'default': [], 'is_visible': True},
         attr.SHARED: {'allow_post': True, 'allow_put': True,
-                      'default': False, 'convert_to': conv.convert_to_boolean,
+                      'default': False, 'convert_to': attr.convert_to_boolean,
                       'is_visible': True, 'required_by_policy': True,
                       'enforce_policy': True},
     },
@@ -173,7 +167,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                'validate': {'type:uuid': None}, 'is_visible': True,
                'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:gbp_resource_name': None},
+                 'validate': {'type:string': None},
                  'default': '', 'is_visible': True},
         'description': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:string': None},
@@ -181,23 +175,19 @@ RESOURCE_ATTRIBUTE_MAP = {
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:string': None},
                       'required_by_policy': True, 'is_visible': True},
-        'status': {'allow_post': False, 'allow_put': False,
-                   'is_visible': True},
-        'status_details': {'allow_post': False, 'allow_put': False,
-                           'is_visible': True},
         'servicechain_specs': {'allow_post': True, 'allow_put': True,
                               'validate': {'type:uuid_list': None},
-                              'convert_to': conv.convert_none_to_empty_list,
+                              'convert_to': attr.convert_none_to_empty_list,
                               'default': None, 'is_visible': True,
                               'required': True},
         'provider_ptg_id': {'allow_post': True, 'allow_put': False,
-                            'validate': {'type:uuid_or_none': None},
-                            'is_visible': True, 'default': None,
-                            'required': True},
+                         'validate': {'type:uuid_or_none': None},
+                         'is_visible': True, 'default': None,
+                         'required': True},
         'consumer_ptg_id': {'allow_post': True, 'allow_put': False,
-                            'validate': {'type:string_or_none': None},
-                            'is_visible': True, 'default': None,
-                            'required': True},
+                         'validate': {'type:uuid_or_none': None},
+                         'is_visible': True, 'default': None,
+                         'required': True},
         'management_ptg_id': {'allow_post': True, 'allow_put': False,
                               'validate': {'type:uuid_or_none': None},
                               'is_visible': True, 'default': None,
@@ -215,7 +205,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                'validate': {'type:uuid': None}, 'is_visible': True,
                'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:gbp_resource_name': None},
+                 'validate': {'type:string': None},
                  'default': '', 'is_visible': True},
         'description': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:string': None},
@@ -223,12 +213,8 @@ RESOURCE_ATTRIBUTE_MAP = {
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:string': None},
                       'required_by_policy': True, 'is_visible': True},
-        'status': {'allow_post': False, 'allow_put': False,
-                   'is_visible': True},
-        'status_details': {'allow_post': False, 'allow_put': False,
-                           'is_visible': True},
         attr.SHARED: {'allow_post': True, 'allow_put': True,
-                      'default': False, 'convert_to': conv.convert_to_boolean,
+                      'default': False, 'convert_to': attr.convert_to_boolean,
                       'is_visible': True, 'required_by_policy': True,
                       'enforce_policy': True},
         'vendor': {'allow_post': True, 'allow_put': True,
@@ -285,7 +271,7 @@ class Servicechain(extensions.ExtensionDescriptor):
 
     @classmethod
     def get_namespace(cls):
-        return "https://wiki.openstack.org/wiki/Neutron/sc/v2.0/"
+        return "http://wiki.openstack.org/neutron/sc/v2.0/"
 
     @classmethod
     def get_updated(cls):
@@ -295,11 +281,13 @@ class Servicechain(extensions.ExtensionDescriptor):
     def get_resources(cls):
         plural_mappings = resource_helper.build_plural_mappings(
             {}, RESOURCE_ATTRIBUTE_MAP)
-        gbp_extensions.register_plurals(plural_mappings)
+        attr.PLURALS.update(plural_mappings)
+        for resource_name in ['servicechain_node', 'servicechain_spec',
+                              'servicechain_instance', 'service_profile']:
+            quota.QUOTAS.register_resource_by_name(resource_name)
         return resource_helper.build_resource_info(plural_mappings,
                                                    RESOURCE_ATTRIBUTE_MAP,
-                                                   constants.SERVICECHAIN,
-                                                   register_quota=True)
+                                                   constants.SERVICECHAIN)
 
     @classmethod
     def get_plugin_interface(cls):
@@ -328,130 +316,123 @@ class ServiceChainPluginBase(service_base.ServicePluginBase):
     def get_plugin_description(self):
         return 'Service Chain plugin'
 
-    def update_chains_pt_added(self, context, policy_target, instance_id):
+    def update_chains_pt_added(self, context, policy_target):
         """ Auto scaling function.
 
         Override this method to react to policy target creation.
         """
         pass
 
-    def update_chains_pt_removed(self, context, policy_target, instance_id):
+    def update_chains_pt_removed(self, context, policy_target):
         """ Auto scaling function.
 
         Override this method to react to policy target deletion.
         """
         pass
 
-    def update_chains_consumer_added(self, context, policy_target_group,
-                                     instance_id):
-        """ Auto scaling function.
-
-        Override this method to react to policy target group addition as
-        a consumer of a chain.
-        """
-        pass
-
-    def update_chains_consumer_removed(self, context, policy_target_group,
-                                       instance_id):
-        """ Auto scaling function.
-
-        Override this method to react to policy target group removed as a
-        consumer of a chain
-        """
-        pass
-
-    def policy_target_group_updated(self, context, old_policy_target_group,
-                                    current_policy_target_group,
-                                    instance_id):
-        """ Utility function.
-
-        Override this method to react to policy target group update
-        """
-        pass
-
     @abc.abstractmethod
+    @log.log
     def get_servicechain_nodes(self, context, filters=None, fields=None):
         pass
 
     @abc.abstractmethod
+    @log.log
     def get_servicechain_node(self, context, servicechain_node_id,
                               fields=None):
         pass
 
     @abc.abstractmethod
+    @log.log
     def create_servicechain_node(self, context, servicechain_node):
         pass
 
     @abc.abstractmethod
+    @log.log
     def update_servicechain_node(self, context, servicechain_node_id,
                                  servicechain_node):
         pass
 
     @abc.abstractmethod
+    @log.log
     def delete_servicechain_node(self, context, servicechain_node_id):
         pass
 
     @abc.abstractmethod
+    @log.log
     def get_servicechain_specs(self, context, filters=None, fields=None):
         pass
 
     @abc.abstractmethod
+    @log.log
     def get_servicechain_spec(self, context, servicechain_spec_id,
                               fields=None):
         pass
 
     @abc.abstractmethod
+    @log.log
     def create_servicechain_spec(self, context, servicechain_spec):
         pass
 
     @abc.abstractmethod
+    @log.log
     def update_servicechain_spec(self, context, servicechain_spec_id,
                                  servicechain_spec):
         pass
 
     @abc.abstractmethod
+    @log.log
     def delete_servicechain_spec(self, context, servicechain_spec_id):
         pass
 
     @abc.abstractmethod
+    @log.log
     def get_servicechain_instances(self, context, filters=None, fields=None):
         pass
 
     @abc.abstractmethod
+    @log.log
     def get_servicechain_instance(self, context, servicechain_instance_id,
                                   fields=None):
         pass
 
     @abc.abstractmethod
-    def create_servicechain_instance(self, context, servicechain_instance):
+    @log.log
+    def create_servicechain_instance(self, context, servicechain_instance_id):
         pass
 
     @abc.abstractmethod
+    @log.log
     def update_servicechain_instance(self, context, servicechain_instance_id,
                                      servicechain_instance):
         pass
 
     @abc.abstractmethod
+    @log.log
     def delete_servicechain_instance(self, context, servicechain_instance_id):
         pass
 
     @abc.abstractmethod
+    @log.log
     def create_service_profile(self, context, service_profile):
         pass
 
     @abc.abstractmethod
+    @log.log
     def update_service_profile(self, context, service_profile_id,
                                service_profile):
         pass
 
     @abc.abstractmethod
+    @log.log
     def delete_service_profile(self, context, service_profile_id):
         pass
 
     @abc.abstractmethod
+    @log.log
     def get_service_profile(self, context, service_profile_id, fields=None):
         pass
 
     @abc.abstractmethod
+    @log.log
     def get_service_profiles(self, context, filters=None, fields=None):
         pass
