@@ -12,7 +12,7 @@
 
 """Exceptions used by Group Policy plugin and drivers."""
 
-from neutron_lib import exceptions
+from neutron.common import exceptions
 
 
 class GroupPolicyDriverError(exceptions.NeutronException):
@@ -67,11 +67,6 @@ class L3PolicyUpdateOfL2PolicyNotSupported(GroupPolicyBadRequest):
     message = _("Updating L3 policy of L2 policy is not supported.")
 
 
-class UnsettingInjectDefaultRouteOfL2PolicyNotSupported(GroupPolicyBadRequest):
-    message = _("Unsetting inject_default_route attribute of L2 policy is not "
-                "supported.")
-
-
 class L3PolicyMultipleRoutersNotSupported(GroupPolicyBadRequest):
     message = _("L3 policy does not support multiple routers.")
 
@@ -94,11 +89,6 @@ class InvalidPortForPTG(GroupPolicyBadRequest):
                 "%(policy_target_group_id)s.")
 
 
-class InvalidPortExtraAttributes(GroupPolicyBadRequest):
-    message = _("Port extra attribute %(attribute)s is invalid for the "
-                "following reason: %(reason)s")
-
-
 class InvalidSubnetForPTG(GroupPolicyBadRequest):
     message = _("Subnet %(subnet_id)s does not belong to network "
                 "%(network_id)s associated with L2P %(l2p_id)s for PTG "
@@ -193,8 +183,7 @@ class OnlyOneEPPerTenantAllowed(GroupPolicyBadRequest):
 
 
 class ImplicitSubnetNotSupported(GroupPolicyBadRequest):
-    message = _("This policy driver doesn't support implicit external "
-                "subnet creation.")
+    message = _("RMD doesn't support implicit external subnet creation.")
 
 
 class DefaultL3PolicyAlreadyExists(GroupPolicyBadRequest):
@@ -234,13 +223,9 @@ class MultipleRedirectActionsNotSupportedForPRS(GroupPolicyBadRequest):
 
 
 class InvalidNetworkServiceParameters(GroupPolicyBadRequest):
-    message = _("Resource Mapping Driver currently supports Network Service "
-                "Parameters with the following types and values: "
-                "type: 'ip_single' value: 'self_subnet'; "
-                "type: 'ip_pool' value: 'nat_pool'; "
-                "type: 'ip_single' value: 'nat_pool'; "
-                "type: 'qos_maxrate' value: numerical of intended Kbps; "
-                "type: 'qos_burstrate' value: numerical of intended Kbps.")
+    message = _("Resource Mapping Driver currently supports only one "
+                "parameter of type: ip_single and value: self_subnet and one "
+                "parameter of type ip_single or ip_pool and value nat_pool")
 
 
 class ESSubnetRequiredForNatPool(GroupPolicyBadRequest):
@@ -254,7 +239,7 @@ class InvalidESSubnetCidrForNatPool(GroupPolicyBadRequest):
 
 
 class NSPRequiresES(GroupPolicyBadRequest):
-    message = _("Driver requires an External Segment in "
+    message = _("Resource Mapping Driver requires an External Segment in "
                 "l3policy to associate a NSP with value nat_pool to a PTG")
 
 
@@ -281,71 +266,3 @@ class OverlappingNATPoolInES(GroupPolicyBadRequest):
 class OverlappingSubnetForNATPoolInES(GroupPolicyBadRequest):
     message = _("One or more subnets associated with network %(net_id)s "
                 "partially overlaps with NAT Pool %(np_id)s.")
-
-
-class InvalidProxiedGroupL3P(GroupPolicyBadRequest):
-    message = _("Cannot proxy PTG %(ptg_id)s: it's on a different L3 policy "
-                "%(l3p_id)s")
-
-
-class InvalidProxiedGroupL2P(GroupPolicyBadRequest):
-    message = _("Cannot proxy PTG %(ptg_id)s: it's on the same L2 Policy as "
-                "the proxy group of type L2.")
-
-
-class OnlyOneProxyGatewayAllowed(GroupPolicyBadRequest):
-    message = _("Another proxy gateway PT already exists for group "
-                "%(group_id)s")
-
-
-class OnlyOneGroupDefaultGatewayAllowed(GroupPolicyBadRequest):
-    message = _("Another group default gateway PT already exists for group "
-                "%(group_id)s")
-
-
-class PTGAlreadyProvidingRedirectPRS(GroupPolicyBadRequest):
-    message = _("PTG %(ptg_id)s is already providing a redirect PRS.")
-
-
-class InvalidClusterId(GroupPolicyBadRequest):
-    message = _("In RMD and derived drivers, a PT cluster_id should point to "
-                "an existing PT.")
-
-
-class PolicyTargetInUse(GroupPolicyBadRequest):
-    message = _("Cannot delete a PT in use by a cluster.")
-
-
-class InvalidClusterPtg(GroupPolicyBadRequest):
-    message = _("Inter PTG clustering disallowed.")
-
-
-class NatPoolInUseByPort(exceptions.InUse, GroupPolicyException):
-    message = _("Ports or floating IP addresses are using the subnet "
-                "corresponding to Nat Pool.")
-
-
-class IdenticalExternalRoute(GroupPolicyBadRequest):
-    message = _("External segments %(es1)s and %(es2)s cannot have "
-                "identical external route CIDR %(cidr)s if associated "
-                "with a common L3 policy.")
-
-
-class NoValidAddressScope(GroupPolicyBadRequest):
-    message = _("No address scope was either provided, could be "
-                "determined, or could be created for a l3_policy.")
-
-
-class NoAddressScopeForSubnetpool(GroupPolicyBadRequest):
-    message = _("Subnetpool does not have an associated address scope.")
-
-
-class InconsistentAddressScopeSubnetpool(GroupPolicyBadRequest):
-    message = _("Subnetpool is not associated with the address "
-                "scope for a l3_policy.")
-
-
-class IncorrectSubnetpoolUpdate(GroupPolicyBadRequest):
-    message = _("Subnetpool %(subnetpool_id)s cannot be disassociated "
-                "from L3 Policy %(l3p_id)s since it has allocated subnet(s) "
-                "associated with that L3 Policy")
