@@ -11,14 +11,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
 from oslo_log import log
-from oslo_utils import excutils
 import stevedore
 
-from gbpservice.neutron.services.grouppolicy.common import exceptions as gp_exc
-
 
 LOG = log.getLogger(__name__)
 
@@ -31,14 +27,14 @@ class ExtensionManager(stevedore.named.NamedExtensionManager):
         # the order in which the drivers are called.
         self.ordered_ext_drivers = []
 
-        LOG.info("Configured extension driver names: %s",
+        LOG.info(_("Configured extension driver names: %s"),
                  cfg.CONF.group_policy.extension_drivers)
         super(ExtensionManager, self).__init__(
             'gbpservice.neutron.group_policy.extension_drivers',
             cfg.CONF.group_policy.extension_drivers,
             invoke_on_load=True,
             name_order=True)
-        LOG.info("Loaded extension driver names: %s", self.names())
+        LOG.info(_("Loaded extension driver names: %s"), self.names())
         self._register_drivers()
 
     def _register_drivers(self):
@@ -49,13 +45,13 @@ class ExtensionManager(stevedore.named.NamedExtensionManager):
         """
         for ext in self:
             self.ordered_ext_drivers.append(ext)
-        LOG.info("Registered extension drivers: %s",
+        LOG.info(_("Registered extension drivers: %s"),
                  [driver.name for driver in self.ordered_ext_drivers])
 
     def initialize(self):
         # Initialize each driver in the list.
         for driver in self.ordered_ext_drivers:
-            LOG.info("Initializing extension driver '%s'", driver.name)
+            LOG.info(_("Initializing extension driver '%s'"), driver.name)
             driver.obj.initialize()
 
     def extension_aliases(self):
@@ -63,7 +59,7 @@ class ExtensionManager(stevedore.named.NamedExtensionManager):
         for driver in self.ordered_ext_drivers:
             alias = driver.obj.extension_alias
             exts.append(alias)
-            LOG.info("Got %(alias)s extension from driver '%(drv)s'",
+            LOG.info(_("Got %(alias)s extension from driver '%(drv)s'"),
                      {'alias': alias, 'drv': driver.name})
         return exts
 
@@ -72,19 +68,11 @@ class ExtensionManager(stevedore.named.NamedExtensionManager):
         for driver in self.ordered_ext_drivers:
             try:
                 getattr(driver.obj, method_name)(session, data, result)
-            except (gp_exc.GroupPolicyException, n_exc.NeutronException):
-                with excutils.save_and_reraise_exception():
-                    LOG.exception(
-                        "Extension driver '%(name)s' "
-                        "failed in %(method)s",
-                        {'name': driver.name, 'method': method_name}
-                    )
             except Exception:
-                LOG.exception("Extension driver '%(name)s' "
-                              "failed in %(method)s",
-                              {'name': driver.name, 'method': method_name})
-                # We are replacing a non-GBP/non-Neutron exception here
-                raise gp_exc.GroupPolicyDriverError(method=method_name)
+                LOG.exception(
+                    _("Extension driver '%(name)s' failed in %(method)s"),
+                    {'name': driver.name, 'method': method_name}
+                )
 
     def process_create_policy_target(self, session, data, result):
         """Call all extension drivers during PT creation."""
@@ -116,21 +104,6 @@ class ExtensionManager(stevedore.named.NamedExtensionManager):
         for driver in self.ordered_ext_drivers:
             driver.obj.extend_policy_target_group_dict(session, result)
 
-    def process_create_application_policy_group(self, session, data, result):
-        """Call all extension drivers during PTG creation."""
-        self._call_on_ext_drivers("process_create_application_policy_group",
-                                  session, data, result)
-
-    def process_update_application_policy_group(self, session, data, result):
-        """Call all extension drivers during PTG update."""
-        self._call_on_ext_drivers("process_update_application_policy_group",
-                                  session, data, result)
-
-    def extend_application_policy_group_dict(self, session, result):
-        """Call all extension drivers to extend PTG dictionary."""
-        for driver in self.ordered_ext_drivers:
-            driver.obj.extend_application_policy_group_dict(session, result)
-
     def process_create_l2_policy(self, session, data, result):
         """Call all extension drivers during L2P creation."""
         self._call_on_ext_drivers("process_create_l2_policy",
