@@ -10,18 +10,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutron.db import api as db_api
-from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
-from oslo_db import exception as oslo_db_excp
 from oslo_log import log
-from oslo_policy import policy as oslo_policy
-from oslo_utils import excutils
-from sqlalchemy import exc as sqlalchemy_exc
 import stevedore
 
-from gbpservice.neutron.services.grouppolicy import (
-    group_policy_driver_api as api)
 from gbpservice.neutron.services.grouppolicy.common import exceptions as gp_exc
 
 
@@ -70,14 +62,14 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self.ordered_policy_drivers = []
         self.reverse_ordered_policy_drivers = []
 
-        LOG.info("Configured policy driver names: %s",
+        LOG.info(_("Configured policy driver names: %s"),
                  cfg.CONF.group_policy.policy_drivers)
         super(PolicyDriverManager,
               self).__init__('gbpservice.neutron.group_policy.policy_drivers',
                              cfg.CONF.group_policy.policy_drivers,
                              invoke_on_load=True,
                              name_order=True)
-        LOG.info("Loaded policy driver names: %s", self.names())
+        LOG.info(_("Loaded policy driver names: %s"), self.names())
         self._register_policy_drivers()
 
     def _register_policy_drivers(self):
@@ -91,7 +83,7 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
             self.ordered_policy_drivers.append(ext)
 
         self.reverse_ordered_policy_drivers = self.ordered_policy_drivers[::-1]
-        LOG.info("Registered policy drivers: %s",
+        LOG.info(_("Registered policy drivers: %s"),
                  [driver.name for driver in self.ordered_policy_drivers])
 
     def initialize(self):
@@ -101,12 +93,12 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         # set it to True such that the drivers can override it.
         self.native_bulk_support = False
         for driver in self.ordered_policy_drivers:
-            LOG.info("Initializing policy driver '%s'", driver.name)
+            LOG.info(_("Initializing policy driver '%s'"), driver.name)
             driver.obj.initialize()
             self.native_bulk_support &= getattr(driver.obj,
                                                 'native_bulk_support', True)
 
-    def _call_on_drivers(self, method_name, context=None,
+    def _call_on_drivers(self, method_name, context,
                          continue_on_failure=False):
         """Helper method for calling a method across all policy drivers.
 
@@ -121,69 +113,25 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         drivers = (self.ordered_policy_drivers if not
                    method_name.startswith('delete') else
                    self.reverse_ordered_policy_drivers)
-        if method_name == 'start_rpc_listeners':
-            servers = []
         for driver in drivers:
             try:
-                if method_name == 'start_rpc_listeners':
-                    server = getattr(driver.obj, method_name)()
-                    if server:
-                        servers.extend(server)
-                else:
-                    getattr(driver.obj, method_name)(context)
-            except Exception as e:
-                if db_api.is_retriable(e):
-                    with excutils.save_and_reraise_exception():
-                        LOG.debug(
-                            "Policy driver '%(name)s' failed in"
-                            " %(method)s, operation will be retried",
-                            {'name': driver.name, 'method': method_name}
-                        )
-                elif isinstance(e, gp_exc.GroupPolicyException) or isinstance(
-                    e, n_exc.NeutronException) or isinstance(
-                        e, oslo_policy.PolicyNotAuthorized):
-                    with excutils.save_and_reraise_exception():
-                        LOG.exception(
-                            "Policy driver '%(name)s' failed in"
-                            " %(method)s",
-                            {'name': driver.name, 'method': method_name}
-                        )
-                elif isinstance(e, sqlalchemy_exc.InvalidRequestError):
-                    LOG.exception(
-                        "Policy driver '%(name)s' failed in %(method)s ",
-                        "with sqlalchemy.exc.InvalidRequestError",
-                        {'name': driver.name, 'method': method_name})
-                    raise oslo_db_excp.RetryRequest(e)
-                else:
-                    error = True
-                    # We are eating a non-GBP/non-Neutron exception here
-                    LOG.exception(
-                        "Policy driver '%(name)s' failed in %(method)s",
-                        {'name': driver.name, 'method': method_name})
-                    if not continue_on_failure:
-                        break
+                getattr(driver.obj, method_name)(context)
+            except gp_exc.GroupPolicyException:
+                # This is an exception for the user.
+                raise
+            except Exception:
+                # This is an internal failure.
+                LOG.exception(
+                    _("Policy driver '%(name)s' failed in %(method)s"),
+                    {'name': driver.name, 'method': method_name}
+                )
+                error = True
+                if not continue_on_failure:
+                    break
         if error:
-            raise gp_exc.GroupPolicyDriverError(method=method_name)
-
-        if method_name == 'start_rpc_listeners':
-            return servers
-
-    def ensure_tenant(self, plugin_context, tenant_id):
-        for driver in self.ordered_policy_drivers:
-            if isinstance(driver.obj, api.PolicyDriver):
-                try:
-                    driver.obj.ensure_tenant(plugin_context, tenant_id)
-                except Exception as e:
-                    if db_api.is_retriable(e):
-                        with excutils.save_and_reraise_exception():
-                            LOG.debug("Policy driver '%(driver)s' failed in "
-                                      "ensure_tenant, operation will "
-                                      "be retried", {'driver': driver.name})
-                    else:
-                        LOG.exception("Policy driver '%s' failed in "
-                                      "ensure_tenant", driver.name)
-                        raise gp_exc.GroupPolicyDriverError(
-                                method="ensure_tenant")
+            raise gp_exc.GroupPolicyDriverError(
+                method=method_name
+            )
 
     def create_policy_target_precommit(self, context):
         self._call_on_drivers("create_policy_target_precommit", context)
@@ -204,9 +152,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self._call_on_drivers("delete_policy_target_postcommit", context,
                               continue_on_failure=True)
 
-    def get_policy_target_status(self, context):
-        self._call_on_drivers("get_policy_target_status", context)
-
     def create_policy_target_group_precommit(self, context):
         self._call_on_drivers("create_policy_target_group_precommit", context)
 
@@ -226,36 +171,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self._call_on_drivers("delete_policy_target_group_postcommit", context,
                               continue_on_failure=True)
 
-    def get_policy_target_group_status(self, context):
-        self._call_on_drivers("get_policy_target_group_status", context)
-
-    def create_application_policy_group_precommit(self, context):
-        self._call_on_drivers("create_application_policy_group_precommit",
-                              context)
-
-    def create_application_policy_group_postcommit(self, context):
-        self._call_on_drivers("create_application_policy_group_postcommit",
-                              context)
-
-    def update_application_policy_group_precommit(self, context):
-        self._call_on_drivers("update_application_policy_group_precommit",
-                              context)
-
-    def update_application_policy_group_postcommit(self, context):
-        self._call_on_drivers("update_application_policy_group_postcommit",
-                              context)
-
-    def delete_application_policy_group_precommit(self, context):
-        self._call_on_drivers("delete_application_policy_group_precommit",
-                              context)
-
-    def delete_application_policy_group_postcommit(self, context):
-        self._call_on_drivers("delete_application_policy_group_postcommit",
-                              context, continue_on_failure=True)
-
-    def get_application_policy_group_status(self, context):
-        self._call_on_drivers("get_application_policy_group_status", context)
-
     def create_l2_policy_precommit(self, context):
         self._call_on_drivers("create_l2_policy_precommit", context)
 
@@ -275,9 +190,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self._call_on_drivers("delete_l2_policy_postcommit", context,
                               continue_on_failure=True)
 
-    def get_l2_policy_status(self, context):
-        self._call_on_drivers("get_l2_policy_status", context)
-
     def create_l3_policy_precommit(self, context):
         self._call_on_drivers("create_l3_policy_precommit", context)
 
@@ -297,9 +209,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self._call_on_drivers("delete_l3_policy_postcommit", context,
                               continue_on_failure=True)
 
-    def get_l3_policy_status(self, context):
-        self._call_on_drivers("get_l3_policy_status", context)
-
     def create_network_service_policy_precommit(self, context):
         self._call_on_drivers(
             "create_network_service_policy_precommit", context)
@@ -325,9 +234,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
             "delete_network_service_policy_postcommit", context,
             continue_on_failure=True)
 
-    def get_network_service_policy_status(self, context):
-        self._call_on_drivers("get_network_service_policy_status", context)
-
     def create_policy_classifier_precommit(self, context):
         self._call_on_drivers("create_policy_classifier_precommit", context)
 
@@ -347,9 +253,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self._call_on_drivers("delete_policy_classifier_postcommit", context,
                               continue_on_failure=True)
 
-    def get_policy_classifier_status(self, context):
-        self._call_on_drivers("get_policy_classifier_status", context)
-
     def create_policy_action_precommit(self, context):
         self._call_on_drivers("create_policy_action_precommit", context)
 
@@ -369,9 +272,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self._call_on_drivers("delete_policy_action_postcommit", context,
                               continue_on_failure=True)
 
-    def get_policy_action_status(self, context):
-        self._call_on_drivers("get_policy_action_status", context)
-
     def create_policy_rule_precommit(self, context):
         self._call_on_drivers("create_policy_rule_precommit", context)
 
@@ -391,9 +291,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self._call_on_drivers("delete_policy_rule_postcommit", context,
                               continue_on_failure=True)
 
-    def get_policy_rule_status(self, context):
-        self._call_on_drivers("get_policy_rule_status", context)
-
     def create_policy_rule_set_precommit(self, context):
         self._call_on_drivers("create_policy_rule_set_precommit", context)
 
@@ -413,9 +310,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self._call_on_drivers("delete_policy_rule_set_postcommit", context,
                               continue_on_failure=True)
 
-    def get_policy_rule_set_status(self, context):
-        self._call_on_drivers("get_policy_rule_set_status", context)
-
     def create_external_segment_precommit(self, context):
         self._call_on_drivers("create_external_segment_precommit",
                               context)
@@ -440,9 +334,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self._call_on_drivers("delete_external_segment_postcommit",
                               context, continue_on_failure=True)
 
-    def get_external_segment_status(self, context):
-        self._call_on_drivers("get_external_segment_status", context)
-
     def create_external_policy_precommit(self, context):
         self._call_on_drivers("create_external_policy_precommit",
                               context)
@@ -467,9 +358,6 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
         self._call_on_drivers("delete_external_policy_postcommit",
                               context, continue_on_failure=True)
 
-    def get_external_policy_status(self, context):
-        self._call_on_drivers("get_external_policy_status", context)
-
     def create_nat_pool_precommit(self, context):
         self._call_on_drivers("create_nat_pool_precommit", context)
 
@@ -488,20 +376,3 @@ class PolicyDriverManager(stevedore.named.NamedExtensionManager):
     def delete_nat_pool_postcommit(self, context):
         self._call_on_drivers("delete_nat_pool_postcommit", context,
                               continue_on_failure=True)
-
-    def get_nat_pool_status(self, context):
-        self._call_on_drivers("get_nat_pool_status", context)
-
-    def start_rpc_listeners(self):
-        return self._call_on_drivers("start_rpc_listeners")
-
-    def validate_state(self, repair):
-        result = api.VALIDATION_PASSED
-        for driver in self.ordered_policy_drivers:
-            this_result = driver.obj.validate_state(repair)
-            if this_result == api.VALIDATION_FAILED:
-                result = this_result
-            elif (this_result == api.VALIDATION_REPAIRED and
-                  result != api.VALIDATION_FAILED):
-                result = this_result
-        return result
