@@ -12,14 +12,10 @@
 
 import time
 
-from heatclient import exc as heat_exc
-from neutron.db import api as db_api
-from neutron.db import models_v2 as ndb
-from neutron_lib.db import model_base
-from neutron_lib.plugins import constants as pconst
+from neutron.common import log
+from neutron.db import model_base
+from neutron.plugins.common import constants as pconst
 from oslo_config import cfg
-from oslo_db import exception as db_exc
-from oslo_log import helpers as log
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 import sqlalchemy as sa
@@ -37,9 +33,6 @@ service_chain_opts = [
                default=15,
                help=_("Seconds to wait for pending stack operation "
                       "to complete")),
-    cfg.IntOpt('delete_vip_port_retries',
-               default=10,
-               help=_("Retries to check if LB VIP port is deleted")),
     cfg.StrOpt('heat_uri',
                default='http://localhost:8004/v1',
                help=_("Heat API server address to instantiate services "
@@ -54,7 +47,6 @@ cfg.CONF.register_opts(service_chain_opts, "heat_node_driver")
 EXCLUDE_POOL_MEMBER_TAG = cfg.CONF.heat_node_driver.exclude_pool_member_tag
 STACK_ACTION_WAIT_TIME = cfg.CONF.heat_node_driver.stack_action_wait_time
 STACK_ACTION_RETRY_WAIT = 5  # Retry after every 5 seconds
-DELETE_VIP_PORT_RETRIES = cfg.CONF.heat_node_driver.delete_vip_port_retries
 
 
 class ServiceNodeInstanceStack(model_base.BASEV2):
@@ -114,32 +106,30 @@ class ServiceTypeUpdateNotSupported(exc.NodeCompositionPluginBadRequest):
 
 class HeatNodeDriver(driver_base.NodeDriverBase):
 
+    sc_supported_type = [pconst.LOADBALANCER, pconst.FIREWALL]
     vendor_name = 'heat_based_node_driver'
     initialized = False
-    sc_supported_type = [pconst.LOADBALANCERV2, pconst.FIREWALL]
-    required_heat_resources = {
-        pconst.LOADBALANCERV2: ['OS::Neutron::LBaaS::LoadBalancer',
-                                'OS::Neutron::LBaaS::Listener',
-                                'OS::Neutron::LBaaS::Pool'],
-        pconst.FIREWALL: ['OS::Neutron::Firewall',
-                          'OS::Neutron::FirewallPolicy'],
-    }
+    required_heat_resources = {pconst.LOADBALANCER: [
+                                            'OS::Neutron::LoadBalancer',
+                                            'OS::Neutron::Pool'],
+                               pconst.FIREWALL: [
+                                            'OS::Neutron::Firewall',
+                                            'OS::Neutron::FirewallPolicy']}
 
-    @log.log_method_call
+    @log.log
     def initialize(self, name):
         self.initialized = True
         self._name = name
 
-    @log.log_method_call
+    @log.log
     def get_plumbing_info(self, context):
         pass
 
-    @log.log_method_call
+    @log.log
     def validate_create(self, context):
         if context.current_profile is None:
             raise ServiceProfileRequired()
-        if context.current_profile['vendor'].lower() != (
-            self.vendor_name.lower()):
+        if context.current_profile['vendor'] != self.vendor_name:
             raise NodeVendorMismatch(vendor=self.vendor_name)
         service_type = context.current_profile['service_type']
         if service_type not in self.sc_supported_type:
@@ -148,7 +138,7 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
         self._validate_service_config(context.current_node['config'],
                                       service_type)
 
-    @log.log_method_call
+    @log.log
     def validate_update(self, context):
         if not context.original_node:  # PT create/delete notifications
             return
@@ -189,7 +179,7 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
                 raise HeatResourceMissing(resource=resource_name,
                                           servicetype=service_type)
 
-    @log.log_method_call
+    @log.log
     def create(self, context):
         heatclient = self._get_heat_client(context.plugin_context)
 
@@ -207,7 +197,7 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
             context.plugin_session, context.current_node['id'],
             context.instance['id'], stack['stack']['id'])
 
-    @log.log_method_call
+    @log.log
     def delete(self, context):
         stack_ids = self._get_node_instance_stacks(context.plugin_session,
                                                    context.current_node['id'],
@@ -215,41 +205,15 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
         heatclient = self._get_heat_client(context.plugin_context)
 
         for stack in stack_ids:
-            vip_port_id = None
-            try:
-                rstr = heatclient.client.resources.get(stack_ids[0].stack_id,
-                    'loadbalancer')
-                vip_port_id = rstr.attributes['vip_port_id']
-            except heat_exc.HTTPNotFound:
-                # stack not found, so no need to process any further
-                pass
             heatclient.delete(stack.stack_id)
-            if vip_port_id:
-                for x in range(0, DELETE_VIP_PORT_RETRIES):
-                    # We intentionally get a new session so as to be
-                    # able to read the updated DB
-                    session = db_api.get_reader_session()
-                    vip_port = session.query(ndb.Port).filter_by(
-                        id=vip_port_id).all()
-                    if vip_port:
-                        # heat stack delete is not finished yet, so try again
-                        LOG.debug(("VIP port %s is not yet deleted"), vip_port)
-                        LOG.debug(("Retry attempt; %s"), x + 1)
-                        # Stack delete will at least take some minimal amount
-                        # of time, hence we wait a little bit.
-                        time.sleep(STACK_ACTION_WAIT_TIME)
-                    else:
-                        # we force a retry so that a new session can be
-                        # used that will correctly reflect the VIP port as
-                        # deleted and hence allow the subsequent policy driver
-                        # to delete the VIP subnet
-                        raise db_exc.RetryRequest(Exception)
-
+        for stack in stack_ids:
+            self._wait_for_stack_operation_complete(
+                                heatclient, stack.stack_id, 'delete')
         self._delete_node_instance_stack_in_db(context.plugin_session,
                                                context.current_node['id'],
                                                context.instance['id'])
 
-    @log.log_method_call
+    @log.log
     def update(self, context):
         heatclient = self._get_heat_client(context.plugin_context)
 
@@ -263,37 +227,20 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
                                 heatclient, stack.stack_id, 'update')
             heatclient.update(stack.stack_id, stack_template, stack_params)
 
-    @log.log_method_call
+    @log.log
     def update_policy_target_added(self, context, policy_target):
-        if context.current_profile['service_type'] == pconst.LOADBALANCERV2:
+        if context.current_profile['service_type'] == pconst.LOADBALANCER:
             self.update(context)
 
-    @log.log_method_call
+    @log.log
     def update_policy_target_removed(self, context, policy_target):
-        if context.current_profile['service_type'] == pconst.LOADBALANCERV2:
+        if context.current_profile['service_type'] == pconst.LOADBALANCER:
             self.update(context)
 
-    @log.log_method_call
-    def update_node_consumer_ptg_added(self, context, policy_target_group):
-        pass
-
-    @log.log_method_call
-    def update_node_consumer_ptg_removed(self, context, policy_target_group):
-        pass
-
-    @log.log_method_call
+    @log.log
     def notify_chain_parameters_updated(self, context):
         self.update(context)
 
-    @log.log_method_call
-    def policy_target_group_updated(self, context, old_policy_target_group,
-                                    current_policy_target_group):
-        pass
-
-    def get_status(self, context):
-        # TODO(Sumit): Needs to be implemented
-        return {'status': '', 'status_details': ''}
-
     @property
     def name(self):
         return self._name
@@ -322,29 +269,25 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
         is_template_aws_version = stack_template.get(
                                         'AWSTemplateFormatVersion', False)
 
-        if service_type == pconst.LOADBALANCERV2:
+        if service_type == pconst.LOADBALANCER:
             self._generate_pool_members(context, stack_template,
                                         config_param_values,
                                         provider_ptg,
                                         is_template_aws_version)
-        elif service_type == pconst.FIREWALL:
+        else:
             provider_subnet = context.core_plugin.get_subnet(
                                 context.plugin_context, provider_ptg_subnet_id)
-            consumer_cidrs = []
-            if consumer:
-                if context.is_consumer_external:
-                    # REVISIT(Magesh): Allowing the first destination which is
-                    # 0/0 Validate and skip adding FW rule in case routes is
-                    # not set
-                    es = context.gbp_plugin.get_external_segment(
-                        context.plugin_context,
-                        consumer['external_segments'][0])
-                    consumer_cidrs = [x['destination']
-                                      for x in es['external_routes']]
-                else:
-                    consumer_subnet = context.core_plugin.get_subnet(
-                        context._plugin_context, consumer['subnets'][0])
-                    consumer_cidrs = [consumer_subnet['cidr']]
+            if context.is_consumer_external:
+                # REVISIT(Magesh): Allowing the first destination which is 0/0
+                # Validate and skip adding FW rule in case routes is not set
+                es = context.gbp_plugin.get_external_segment(
+                    context.plugin_context, consumer['external_segments'][0])
+                consumer_cidrs = [x['destination']
+                                  for x in es['external_routes']]
+            else:
+                consumer_subnet = context.core_plugin.get_subnet(
+                    context._plugin_context, consumer['subnets'][0])
+                consumer_cidrs = [consumer_subnet['cidr']]
             provider_cidr = provider_subnet['cidr']
             self._update_template_with_firewall_rules(
                     context, provider_ptg, provider_cidr, consumer_cidrs,
@@ -356,8 +299,6 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
         for parameter in node_params:
             if parameter == "Subnet":
                 stack_params[parameter] = provider_ptg_subnet_id
-            elif parameter == "service_chain_metadata":
-                stack_params[parameter] = sc_instance['id']
             elif parameter in config_param_values:
                 stack_params[parameter] = config_param_values[parameter]
         return (stack_template, stack_params)
@@ -373,15 +314,15 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
                                                 'DELETE_IN_PROGRESS']:
                     return
             except Exception:
-                LOG.exception("Retrieving the stack %(stack)s failed.",
+                LOG.exception(_("Retrieving the stack %(stack)s failed."),
                               {'stack': stack_id})
                 return
             else:
                 time.sleep(STACK_ACTION_RETRY_WAIT)
                 time_waited = time_waited + STACK_ACTION_RETRY_WAIT
                 if time_waited >= STACK_ACTION_WAIT_TIME:
-                    LOG.error("Stack %(action)s not completed within "
-                              "%(wait)s seconds",
+                    LOG.error(_("Stack %(action)s not completed within "
+                                "%(wait)s seconds"),
                               {'action': action,
                                'wait': STACK_ACTION_WAIT_TIME,
                                'stack': stack_id})
@@ -460,7 +401,9 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
                     "destination_port": destination_port,
                     "action": "allow",
                     "destination_ip_address": destination_cidr,
-                    "source_ip_address": source_cidr}}
+                    "source_ip_address": source_cidr
+                }
+                }
 
     def _generate_pool_members(self, context, stack_template,
                                config_param_values, provider_ptg,
@@ -474,7 +417,7 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
         pool_res_name = None
         for resource in stack_template[resources_key]:
             if stack_template[resources_key][resource][type_key] == (
-                    'OS::Neutron::LBaaS::Pool'):
+                                                    'OS::Neutron::Pool'):
                 pool_res_name = resource
                 break
 
@@ -492,13 +435,13 @@ class HeatNodeDriver(driver_base.NodeDriverBase):
         properties_key = ('Properties' if is_template_aws_version
                           else 'properties')
         res_key = 'Ref' if is_template_aws_version else 'get_resource'
-        return {type_key: "OS::Neutron::LBaaS::PoolMember",
+        return {type_key: "OS::Neutron::PoolMember",
                 properties_key: {
                     "address": member_ip,
                     "admin_state_up": True,
-                    "pool": {res_key: pool_res_name},
-                    "protocol_port": {'get_param': 'app_port'},
-                    "subnet": {'get_param': 'Subnet'},
+                    "pool_id": {res_key: pool_res_name},
+                    # FIXME(Magesh): Need to handle port range
+                    "protocol_port": context.classifier["port_range"],
                     "weight": 1}}
 
     def _get_member_ips(self, context, ptg):
