@@ -10,18 +10,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutron.db import api as db_api
-from neutron.quota import resource_registry
-from neutron_lib.plugins import constants as pconst
+from neutron.common import log
 from oslo_config import cfg
-from oslo_log import helpers as log
 from oslo_log import log as logging
 from oslo_utils import excutils
 
 from gbpservice.common import utils
 from gbpservice.neutron.db import servicechain_db
-from gbpservice.neutron.services.grouppolicy.common import constants as gp_cts
-from gbpservice.neutron.services.grouppolicy.common import utils as gutils
 from gbpservice.neutron.services.servicechain.plugins.ncp import (
     context as ctx)
 from gbpservice.neutron.services.servicechain.plugins.ncp import (
@@ -33,12 +28,6 @@ from gbpservice.neutron.services.servicechain.plugins import sharing
 LOG = logging.getLogger(__name__)
 
 PLUMBER_NAMESPACE = 'gbpservice.neutron.servicechain.ncp_plumbers'
-cfg.CONF.import_opt('policy_drivers',
-                    'gbpservice.neutron.services.grouppolicy.config',
-                    group='group_policy')
-STATUS = 'status'
-STATUS_DETAILS = 'status_details'
-STATUS_SET = set([STATUS, STATUS_DETAILS])
 
 
 class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
@@ -48,37 +37,25 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
 
     """
     supported_extension_aliases = ["servicechain"]
-    path_prefix = gp_cts.GBP_PREFIXES[pconst.SERVICECHAIN]
 
-    @resource_registry.tracked_resources(
-        servicechain_node=servicechain_db.ServiceChainNode,
-        servicechain_spec=servicechain_db.ServiceChainSpec,
-        servicechain_instance=servicechain_db.ServiceChainInstance,
-        service_profile=servicechain_db.ServiceProfile)
     def __init__(self):
         self.driver_manager = manager.NodeDriverManager()
         super(NodeCompositionPlugin, self).__init__()
         self.driver_manager.initialize()
-        plumber_klass = cfg.CONF.node_composition_plugin.node_plumber
         self.plumber = utils.load_plugin(
-            PLUMBER_NAMESPACE, plumber_klass)
+            PLUMBER_NAMESPACE, cfg.CONF.node_composition_plugin.node_plumber)
         self.plumber.initialize()
-        LOG.info("Initialized node plumber '%s'", plumber_klass)
 
-    @log.log_method_call
+    @log.log
     def create_servicechain_instance(self, context, servicechain_instance):
         """Instance created.
 
         When a Servicechain Instance is created, all its nodes need to be
         instantiated.
         """
-        instance = self._process_commit_phase(context)
-        if instance:
-            return instance
-
+        session = context.session
         deployers = {}
-        # REVISIT: Consider adding ensure_tenant() call here
-        with db_api.context_manager.writer.using(context):
+        with session.begin(subtransactions=True):
             instance = super(NodeCompositionPlugin,
                              self).create_servicechain_instance(
                                  context, servicechain_instance)
@@ -86,59 +63,21 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
                 raise exc.OneSpecPerInstanceAllowed()
             deployers = self._get_scheduled_drivers(context, instance,
                                                     'deploy')
-        if not gutils.is_precommit_policy_driver_configured():
-            # Actual node deploy
-            try:
-                self._deploy_servicechain_nodes(context, deployers)
-            except Exception:
-                # Some node could not be deployed
-                with excutils.save_and_reraise_exception():
-                    LOG.error("Node deployment failed, "
-                              "deleting servicechain_instance %s",
-                              instance['id'])
-                    self.delete_servicechain_instance(context, instance['id'])
 
-        return instance
-
-    def _process_commit_phase(self, context):
-        if hasattr(context, 'commit_phase'):
-            if not gutils.is_precommit_policy_driver_configured() and (
-                context.commit_phase == gp_cts.PRE_COMMIT):
-                # The following is a bit of a hack to no-op
-                # the call from the postcommit policy driver
-                # during the pre-commit phase.
-                return True
-            if gutils.is_precommit_policy_driver_configured() and (
-                context.commit_phase == gp_cts.POST_COMMIT):
-                instance = self.get_servicechain_instance(
-                    context, context.servicechain_instance['id'])
-                self._call_deploy_sc_node(context, instance)
-                return instance
-
-    def _call_deploy_sc_node(self, context, instance):
         # Actual node deploy
         try:
-            deployers = self._get_scheduled_drivers(
-                context, instance, 'deploy')
             self._deploy_servicechain_nodes(context, deployers)
         except Exception:
             # Some node could not be deployed
             with excutils.save_and_reraise_exception():
-                LOG.error("Node deployment failed, "
-                          "servicechain_instance %s is in ERROR state",
+                LOG.error(_("Node deployment failed, "
+                            "deleting servicechain_instance %s"),
                           instance['id'])
+                self.delete_servicechain_instance(context, instance['id'])
 
-    @log.log_method_call
-    def get_servicechain_instance(self, context,
-                                  servicechain_instance_id, fields=None):
-        """Instance retrieved.
+        return instance
 
-        While get Servicechain Instance details, get all nodes details.
-        """
-        return self._get_resource(context, 'servicechain_instance',
-                                  servicechain_instance_id, fields)
-
-    @log.log_method_call
+    @log.log
     def update_servicechain_instance(self, context, servicechain_instance_id,
                                      servicechain_instance):
         """Instance updated.
@@ -147,14 +86,11 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
         nodes of the previous spec should be destroyed and the newer ones
         created.
         """
-        instance = self._process_commit_phase(context)
-        if instance:
-            return instance
-
+        session = context.session
         deployers = {}
         updaters = {}
         destroyers = {}
-        with db_api.context_manager.writer.using(context):
+        with session.begin(subtransactions=True):
             original_instance = self.get_servicechain_instance(
                 context, servicechain_instance_id)
             updated_instance = super(
@@ -176,43 +112,41 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
             self._destroy_servicechain_nodes(context, destroyers)
             deployers = self._get_scheduled_drivers(
                         context, updated_instance, 'deploy')
-            context.deployers = deployers
-            context.servicechain_instance = updated_instance
-            if not gutils.is_precommit_policy_driver_configured():
-                self._deploy_servicechain_nodes(context, deployers)
+            self._deploy_servicechain_nodes(context, deployers)
         else:
             self._update_servicechain_nodes(context, updaters)
         return updated_instance
 
-    @log.log_method_call
+    @log.log
     def delete_servicechain_instance(self, context, servicechain_instance_id):
         """Instance deleted.
 
         When a Servicechain Instance is deleted, all its nodes need to be
         destroyed.
         """
-        with db_api.context_manager.writer.using(context):
+        session = context.session
+        with session.begin(subtransactions=True):
             instance = self.get_servicechain_instance(context,
                                                       servicechain_instance_id)
             destroyers = self._get_scheduled_drivers(context, instance,
                                                      'destroy')
         self._destroy_servicechain_nodes(context, destroyers)
 
-        with db_api.context_manager.writer.using(context):
+        with session.begin(subtransactions=True):
             super(NodeCompositionPlugin, self).delete_servicechain_instance(
                 context, servicechain_instance_id)
 
-    @log.log_method_call
+    @log.log
     def create_servicechain_node(self, context, servicechain_node):
-        # REVISIT: Consider adding ensure_tenant() call here
-        with db_api.context_manager.writer.using(context):
+        session = context.session
+        with session.begin(subtransactions=True):
             result = super(NodeCompositionPlugin,
                            self).create_servicechain_node(context,
                                                           servicechain_node)
             self._validate_shared_create(context, result, 'servicechain_node')
         return result
 
-    @log.log_method_call
+    @log.log
     def update_servicechain_node(self, context, servicechain_node_id,
                                  servicechain_node):
         """Node Update.
@@ -221,8 +155,9 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
         need to be updated as well. This usually results in a node
         reconfiguration.
         """
+        session = context.session
         updaters = {}
-        with db_api.context_manager.writer.using(context):
+        with session.begin(subtransactions=True):
             original_sc_node = self.get_servicechain_node(
                 context, servicechain_node_id)
             updated_sc_node = super(NodeCompositionPlugin,
@@ -251,31 +186,26 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
             try:
                 update['driver'].update(update['context'])
             except exc.NodeDriverError as ex:
-                LOG.error("Node Update failed, %s",
+                LOG.error(_("Node Update failed, %s"),
                           ex.message)
 
         return updated_sc_node
 
-    @log.log_method_call
-    def get_servicechain_node(self, context, servicechain_node_id,
-                              fields = None):
-        return self._get_resource(context, 'servicechain_node',
-                                  servicechain_node_id, fields)
-
-    @log.log_method_call
+    @log.log
     def create_servicechain_spec(self, context, servicechain_spec):
-        # REVISIT: Consider adding ensure_tenant() call here
-        with db_api.context_manager.writer.using(context):
+        session = context.session
+        with session.begin(subtransactions=True):
             result = super(
                 NodeCompositionPlugin, self).create_servicechain_spec(
                     context, servicechain_spec, set_params=False)
             self._validate_shared_create(context, result, 'servicechain_spec')
         return result
 
-    @log.log_method_call
+    @log.log
     def update_servicechain_spec(self, context, servicechain_spec_id,
                                  servicechain_spec):
-        with db_api.context_manager.writer.using(context):
+        session = context.session
+        with session.begin(subtransactions=True):
             original_sc_spec = self.get_servicechain_spec(
                                          context, servicechain_spec_id)
             updated_sc_spec = super(NodeCompositionPlugin,
@@ -284,34 +214,48 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
                                         servicechain_spec, set_params=False)
             self._validate_shared_update(context, original_sc_spec,
                                          updated_sc_spec, 'servicechain_spec')
-            # The reference plumber does not support modifying or reordering of
-            # nodes in a service chain spec. Disallow update for now
-            if (original_sc_spec['nodes'] != updated_sc_spec['nodes'] and
-                original_sc_spec['instances']):
-                raise exc.InuseSpecNodeUpdateNotAllowed()
+
+        # Remove the deleted node instances and add any new ones added
+        # REVISIT(Magesh): Invoke a validate update in Node plumber
+        # and reject the update in case the plumber does not support node
+        # reordering in the spec. For now reordering is a NOOP
+        if original_sc_spec['nodes'] != updated_sc_spec['nodes']:
+            instances = original_sc_spec['instances']
+            instances = self.get_servicechain_instances(
+                context, filters={'id': instances})
+            for instance in instances:
+                removed_nodes = (set(original_sc_spec['nodes']) -
+                                 set(updated_sc_spec['nodes']))
+                added_nodes = (set(updated_sc_spec['nodes']) -
+                               set(original_sc_spec['nodes']))
+                removed_nodes = self.get_servicechain_nodes(
+                        context, filters={'id': removed_nodes})
+                added_nodes = self.get_servicechain_nodes(
+                        context, filters={'id': added_nodes})
+                destroyers = self._get_scheduled_drivers(
+                        context, instance, 'destroy', nodes=removed_nodes)
+                deployers = self._get_scheduled_drivers(
+                        context, instance, 'deploy', nodes=added_nodes)
+                self._destroy_servicechain_nodes(context, destroyers)
+                self._deploy_servicechain_nodes(context, deployers)
 
         return updated_sc_spec
 
-    @log.log_method_call
-    def get_servicechain_spec(self, context,
-                              servicechain_spec_id, fields = None):
-        return self._get_resource(context, 'servicechain_spec',
-                                  servicechain_spec_id, fields)
-
-    @log.log_method_call
+    @log.log
     def create_service_profile(self, context, service_profile):
-        # REVISIT: Consider adding ensure_tenant() call here
-        with db_api.context_manager.writer.using(context):
+        session = context.session
+        with session.begin(subtransactions=True):
             result = super(
                 NodeCompositionPlugin, self).create_service_profile(
                     context, service_profile)
             self._validate_shared_create(context, result, 'service_profile')
         return result
 
-    @log.log_method_call
+    @log.log
     def update_service_profile(self, context, service_profile_id,
                                service_profile):
-        with db_api.context_manager.writer.using(context):
+        session = context.session
+        with session.begin(subtransactions=True):
             original_profile = self.get_service_profile(
                 context, service_profile_id)
             updated_profile = super(NodeCompositionPlugin,
@@ -322,105 +266,35 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
                                           updated_profile)
         return updated_profile
 
-    @log.log_method_call
-    def get_service_profile(self, context, service_profile_id, fields = None):
-        return self._get_resource(context, 'service_profile',
-                                  service_profile_id, fields)
-
-    def update_chains_pt_added(self, context, policy_target, instance_id):
+    def update_chains_pt_added(self, context, policy_target):
         """ Auto scaling function.
 
         Notify the correct set of node drivers that a new policy target has
         been added to a relevant PTG.
         """
-        self._update_chains_pt_modified(context, policy_target, instance_id,
-                                        'added')
+        self._update_chains_pt_modified(context, policy_target, 'added')
 
-    def update_chains_pt_removed(self, context, policy_target, instance_id):
+    def update_chains_pt_removed(self, context, policy_target):
         """ Auto scaling function.
 
         Notify the correct set of node drivers that a new policy target has
         been removed from a relevant PTG.
         """
-        self._update_chains_pt_modified(context, policy_target, instance_id,
-                                        'removed')
+        self._update_chains_pt_modified(context, policy_target, 'removed')
 
-    def update_chains_consumer_added(self, context, policy_target_group,
-                                     instance_id):
-        """ Auto scaling function.
+    def _update_chains_pt_modified(self, context, policy_target, action):
+        scis = self._get_instances_from_policy_target(context, policy_target)
 
-        Override this method to react to policy target group addition as
-        a consumer of a chain.
-        """
-        self._update_chains_consumer_modified(context, policy_target_group,
-                                              instance_id, 'added')
-
-    def policy_target_group_updated(self, context, old_policy_target_group,
-                                    current_policy_target_group,
-                                    instance_id):
-        """ Utility function.
-        Override this method to react to policy target group update
-        """
-        self._policy_target_group_updated(context,
-                                          old_policy_target_group,
-                                          current_policy_target_group,
-                                          instance_id)
-
-    def update_chains_consumer_removed(self, context, policy_target_group,
-                                       instance_id):
-        """ Auto scaling function.
-
-        Override this method to react to policy target group removed as a
-        consumer of a chain
-        """
-        self._update_chains_consumer_modified(context, policy_target_group,
-                                              instance_id, 'removed')
-
-    def _policy_target_group_updated(self, context, old_policy_target_group,
-                                     current_policy_target_group,
-                                     instance_id):
-        updaters = self._get_scheduled_drivers(
-                context,
-                self.get_servicechain_instance(context, instance_id),
-                'update')
-        for update in updaters.values():
-            try:
-                update['driver'].policy_target_group_updated(
-                        update['context'],
-                        old_policy_target_group,
-                        current_policy_target_group)
-            except exc.NodeDriverError as ex:
-                LOG.error("Node Update on policy target group modification"
-                          " failed, %s", ex.message)
-
-    def _update_chains_pt_modified(self, context, policy_target, instance_id,
-                                   action):
-        updaters = self._get_scheduled_drivers(
-            context, self.get_servicechain_instance(context, instance_id),
-            'update')
-        for update in updaters.values():
-            try:
-                getattr(update['driver'],
-                        'update_policy_target_' + action)(
-                            update['context'], policy_target)
-            except exc.NodeDriverError as ex:
-                LOG.error("Node Update on policy target modification "
-                          "failed, %s", ex.message)
-
-    def _update_chains_consumer_modified(self, context, policy_target_group,
-                                         instance_id, action):
-        updaters = self._get_scheduled_drivers(
-            context, self.get_servicechain_instance(context, instance_id),
-            'update')
-        for update in updaters.values():
-            try:
-                getattr(update['driver'],
-                        'update_node_consumer_ptg_' + action)(
-                            update['context'], policy_target_group)
-            except exc.NodeDriverError as ex:
-                LOG.error(
-                    "Node Update on policy target group modification "
-                    "failed, %s", ex.message)
+        for sci in scis:
+            updaters = self._get_scheduled_drivers(context, sci, 'update')
+            for update in updaters.values():
+                try:
+                    getattr(update['driver'],
+                            'update_policy_target_' + action)(
+                                update['context'], policy_target)
+                except exc.NodeDriverError as ex:
+                    LOG.error(_("Node Update on policy target modification "
+                                "failed, %s"), ex.message)
 
     def notify_chain_parameters_updated(self, context,
                                         servicechain_instance_id):
@@ -439,11 +313,10 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
                 getattr(update['driver'],
                         'notify_chain_parameters_updated')(update['context'])
             except exc.NodeDriverError as ex:
-                LOG.error("Node Update on GBP parameter update "
-                          "failed, %s", ex.message)
+                LOG.error(_("Node Update on GBP parameter update "
+                            "failed, %s"), ex.message)
 
     def _get_instance_nodes(self, context, instance):
-        context = utils.admin_context(context)
         if not instance['servicechain_specs']:
             return []
         specs = self.get_servicechain_spec(
@@ -451,7 +324,6 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
         return self.get_servicechain_nodes(context, {'id': specs['nodes']})
 
     def _get_node_instances(self, context, node):
-        context = utils.admin_context(context)
         specs = self.get_servicechain_specs(
             context, {'id': node['servicechain_specs']})
         result = []
@@ -479,73 +351,6 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
                 node_context)
         return result
 
-    def _get_resource(self, context, resource_name, resource_id, fields=None):
-        deployers = {}
-        with db_api.context_manager.writer.using(context):
-            resource = getattr(super(NodeCompositionPlugin,
-                self), 'get_' + resource_name)(context, resource_id)
-            if resource_name == 'servicechain_instance':
-                if len(resource['servicechain_specs']) > 1:
-                    raise exc.OneSpecPerInstanceAllowed()
-                try:
-                    deployers = self._get_scheduled_drivers(context, resource,
-                                                        'get')
-                except Exception:
-                    LOG.warning("Failed to get node driver")
-
-        # Invoke drivers only if status attributes are requested
-        if not fields or STATUS_SET.intersection(set(fields)):
-            _resource = self._get_resource_status(context, resource_name,
-                                                  deployers)
-            if _resource:
-                updated_status = _resource['status']
-                updated_status_details = _resource['status_details']
-                if resource['status'] != updated_status or (
-                    resource['status_details'] != updated_status_details):
-                    new_status = {resource_name:
-                              {'status': updated_status,
-                               'status_details': updated_status_details}}
-                    session = context.session
-                    with session.begin(subtransactions=True):
-                        getattr(super(NodeCompositionPlugin, self),
-                            'update_' + resource_name)(
-                             context, resource['id'], new_status)
-                    resource['status'] = updated_status
-                    resource['status_details'] = updated_status_details
-        return self._fields(resource, fields)
-
-    def _get_resource_status(self, context, resource_name, deployers=None):
-        """
-        Invoke node drivers only for servicechain_instance.
-        Node driver should implement get_status api to return status
-        and status_details of servicechain_instance
-        """
-        if resource_name == 'servicechain_instance':
-            nodes_status = []
-            result = {'status': 'BUILD',
-                      'status_details': 'node deployment in progress'}
-            if deployers:
-                try:
-                    for deploy in deployers.values():
-                        driver = deploy['driver']
-                        nodes_status.append(driver.get_status(
-                            deploy['context']))
-                    node_status = [node['status'] for node in nodes_status]
-                    if 'ERROR' in node_status:
-                        result['status'] = 'ERROR'
-                        result['status_details'] = 'node deployment failed'
-                    elif node_status.count('ACTIVE') == len(
-                            deployers.values()):
-                        result['status'] = 'ACTIVE'
-                        result['status_details'] = 'node deployment completed'
-                except Exception as exc:
-                    LOG.error("Failed to get servicechain instance status "
-                        "from node driver, Error: %(exc)s", {'exc': exc})
-                    return
-                return result
-        result = {'status': 'ACTIVE', 'status_details': ''}
-        return result
-
     def _deploy_servicechain_nodes(self, context, deployers):
         self.plumber.plug_services(context, deployers.values())
         for deploy in deployers.values():
@@ -565,16 +370,9 @@ class NodeCompositionPlugin(servicechain_db.ServiceChainDbPlugin,
                 try:
                     driver.delete(destroy['context'])
                 except exc.NodeDriverError:
-                    LOG.error("Node destroy failed, for node %s ",
+                    LOG.error(_("Node destroy failed, for node %s "),
                               driver['context'].current_node['id'])
                 except Exception as e:
-                    if db_api.is_retriable(e):
-                        with excutils.save_and_reraise_exception():
-                            LOG.debug(
-                                "Node driver '%(name)s' failed in"
-                                " %(method)s, operation will be retried",
-                                {'name': driver._name, 'method': 'delete'}
-                            )
                     LOG.exception(e)
                 finally:
                     self.driver_manager.clear_node_owner(destroy['context'])
