@@ -14,48 +14,37 @@
 import webob.exc
 
 from neutron.tests.unit.extensions import test_l3
-from neutron_lib import context as nctx
+from neutron.tests.unit import testlib_api
 
 from gbpservice.neutron.db.grouppolicy import group_policy_mapping_db as gpmdb
-from gbpservice.neutron.services.grouppolicy.common import exceptions as gpexc
 from gbpservice.neutron.tests.unit.db.grouppolicy import (
     test_group_policy_db as tgpdb)
 
 
 class GroupPolicyMappingDBTestPlugin(gpmdb.GroupPolicyMappingDbPlugin):
 
-    supported_extension_aliases = ['group-policy', 'group-policy-mapping'] + (
-        tgpdb.UNSUPPORTED_REQUIRED_EXTS)
-    path_prefix = "/grouppolicy"
+    supported_extension_aliases = ['group-policy', 'group-policy-mapping']
 
 
 DB_GP_PLUGIN_KLASS = (GroupPolicyMappingDBTestPlugin.__module__ + '.' +
                       GroupPolicyMappingDBTestPlugin.__name__)
 
 SC_PLUGIN_KLASS = (
-    "gbpservice.neutron.services.servicechain.plugins.ncp.plugin."
-    "NodeCompositionPlugin")
+    "gbpservice.neutron.services.servicechain.plugins.msc.plugin."
+    "ServiceChainPlugin")
 
 
 class GroupPolicyMappingDbTestCase(tgpdb.GroupPolicyDbTestCase,
                                    test_l3.L3NatTestCaseMixin):
 
-    def setUp(self, core_plugin=None, l3_plugin=None, gp_plugin=None,
-              service_plugins=None, sc_plugin=None, qos_plugin=None,
-              trunk_plugin=None):
+    def setUp(self, core_plugin=None, gp_plugin=None, service_plugins=None):
+        testlib_api.SqlTestCase._TABLES_ESTABLISHED = False
         if not gp_plugin:
             gp_plugin = DB_GP_PLUGIN_KLASS
         if not service_plugins:
-            service_plugins = {
-                'gp_plugin_name': gp_plugin,
-                'flavors_plugin_name': 'neutron.services.flavors.'
-                                       'flavors_plugin.FlavorsPlugin',
-                'servicechain_plugin': sc_plugin or SC_PLUGIN_KLASS}
-        service_plugins['l3_plugin_name'] = l3_plugin or "router"
-        if qos_plugin:
-            service_plugins['qos_plugin_name'] = qos_plugin
-        if trunk_plugin:
-            service_plugins['trunk_plugin_name'] = trunk_plugin
+            service_plugins = {'l3_plugin_name': "router",
+                               'gp_plugin_name': gp_plugin,
+                               'servicechain_plugin': SC_PLUGIN_KLASS}
         super(GroupPolicyMappingDbTestCase, self).setUp(
             core_plugin=core_plugin, gp_plugin=gp_plugin,
             service_plugins=service_plugins
@@ -80,7 +69,7 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
             self.assertEqual(port_id, res['policy_target']['port_id'])
             req = self.new_delete_request('policy_targets', pt_id)
             res = req.get_response(self.ext_api)
-            self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+            self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
 
     def test_create_delete_policy_target_group_with_subnets(self):
         with self.subnet(cidr='10.10.1.0/24') as subnet1:
@@ -98,7 +87,7 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
                                  sorted(res['policy_target_group']['subnets']))
                 req = self.new_delete_request('policy_target_groups', ptg_id)
                 res = req.get_response(self.ext_api)
-                self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+                self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
 
     def test_update_policy_target_group_subnets(self):
         with self.subnet(cidr='10.10.1.0/24') as subnet1:
@@ -134,8 +123,8 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
                     req = self.new_delete_request('policy_target_groups',
                                                   ptg_id)
                     res = req.get_response(self.ext_api)
-                    self.assertEqual(webob.exc.HTTPNoContent.code,
-                                     res.status_int)
+                    self.assertEqual(
+                        res.status_int, webob.exc.HTTPNoContent.code)
 
     def test_create_delete_l2_policy_with_network(self):
         with self.network() as network:
@@ -148,7 +137,7 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
             self.assertEqual(network_id, res['l2_policy']['network_id'])
             req = self.new_delete_request('l2_policies', l2p_id)
             res = req.get_response(self.ext_api)
-            self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+            self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
 
     def test_create_delete_l3_policy_with_routers(self):
         with self.router() as router1:
@@ -156,14 +145,6 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
                 routers = [router1['router']['id'], router2['router']['id']]
                 l3p = self.create_l3_policy(routers=routers)
                 l3p_id = l3p['l3_policy']['id']
-                test_l3p_id = self._gbp_plugin.get_l3p_id_from_router_id(
-                    nctx.get_admin_context(),
-                    router1['router']['id'])
-                self.assertEqual(l3p_id, test_l3p_id)
-                test_l3p_id = self._gbp_plugin.get_l3p_id_from_router_id(
-                    nctx.get_admin_context(),
-                    router2['router']['id'])
-                self.assertEqual(l3p_id, test_l3p_id)
                 self.assertEqual(sorted(routers),
                                  sorted(l3p['l3_policy']['routers']))
                 req = self.new_show_request('l3_policies', l3p_id,
@@ -174,7 +155,7 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
                                  sorted(res['l3_policy']['routers']))
                 req = self.new_delete_request('l3_policies', l3p_id)
                 res = req.get_response(self.ext_api)
-                self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+                self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
 
     def test_update_l3_policy_routers(self):
         with self.router() as router1:
@@ -205,8 +186,8 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
                     # the resource(s) that are created.
                     req = self.new_delete_request('l3_policies', l3p_id)
                     res = req.get_response(self.ext_api)
-                    self.assertEqual(webob.exc.HTTPNoContent.code,
-                                     res.status_int)
+                    self.assertEqual(res.status_int,
+                                     webob.exc.HTTPNoContent.code)
 
     def test_create_delete_es_with_subnet(self):
         with self.subnet(cidr='10.10.1.0/24') as subnet:
@@ -221,7 +202,7 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
             self.assertEqual(subnet_id, res['external_segment']['subnet_id'])
             req = self.new_delete_request('external_segments', es_id)
             res = req.get_response(self.ext_api)
-            self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+            self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
 
     def test_list_policy_targets(self):
         with self.port() as port1:
@@ -229,7 +210,6 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
                 ports = [port1['port']['id'], port2['port']['id']]
                 pts = [self.create_policy_target(port_id=ports[0]),
                        self.create_policy_target(port_id=ports[1])]
-                self._test_list_resources('policy_target', pts)
                 self._test_list_resources('policy_target', [pts[0]],
                                           query_params='port_id=' + ports[0])
 
@@ -241,10 +221,8 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
                 l2_policies = [self.create_l2_policy(network_id=networks[0]),
                                self.create_l2_policy(network_id=networks[1])]
                 self._test_list_resources(
-                    'l2_policy', l2_policies)
-                self._test_list_resources(
-                    'l2_policy', [l2_policies[0]],
-                    query_params='network_id=' + networks[0])
+                                'l2_policy', [l2_policies[0]],
+                                query_params='network_id=' + networks[0])
 
     def test_list_es(self):
         with self.subnet(cidr='10.10.1.0/24') as subnet1:
@@ -254,29 +232,5 @@ class TestMappedGroupResourceAttrs(GroupPolicyMappingDbTestCase):
                             self.create_external_segment(subnet_id=subnets[0]),
                             self.create_external_segment(subnet_id=subnets[1])]
                 self._test_list_resources(
-                    'external_segment', external_segments)
-                self._test_list_resources(
-                    'external_segment', [external_segments[0]],
-                    query_params='subnet_id=' + subnets[0])
-
-    def test_pt_port_extra_attributes_fail(self):
-        ptg = self.create_policy_target_group()['policy_target_group']
-        ctx = nctx.get_admin_context()
-        self.assertRaises(gpexc.InvalidPortExtraAttributes,
-                          self._gbp_plugin.create_policy_target,
-                          ctx, {'policy_target': {
-                                    'description': '', 'name': '',
-                                    'port_id': None, 'cluster_id': '',
-                                    'policy_target_group_id': ptg['id'],
-                                    'port_attributes': {'network_id': ''},
-                                    'tenant_id': self._tenant_id}})
-
-        self.assertRaises(gpexc.InvalidPortExtraAttributes,
-                          self._gbp_plugin.create_policy_target,
-                          ctx, {'policy_target': {
-                                    'description': '', 'name': '',
-                                    'port_id': None, 'cluster_id': '',
-                                    'policy_target_group_id': ptg['id'],
-                                    'port_attributes': {
-                                        'allowed_address_pairs': ''},
-                                    'tenant_id': self._tenant_id}})
+                            'external_segment', [external_segments[0]],
+                            query_params='subnet_id=' + subnets[0])
