@@ -11,25 +11,84 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import six
 import webob.exc
 
-from neutron_lib import context
-from neutron_lib.plugins import constants
+from neutron.api import extensions
+from neutron import context
+from neutron.db import api as db_api
+from neutron.db import model_base
+from neutron.openstack.common import uuidutils
+from neutron.plugins.common import constants
+from neutron.tests.unit.api import test_extensions
+from neutron.tests.unit.db import test_db_base_plugin_v2
 from oslo_config import cfg
-from oslo_utils import uuidutils
+from oslo_utils import importutils
 
 from gbpservice.neutron.db import servicechain_db as svcchain_db
+from gbpservice.neutron.extensions import group_policy as gpolicy
 from gbpservice.neutron.extensions import servicechain as service_chain
 from gbpservice.neutron.tests.unit import common as cm
 from gbpservice.neutron.tests.unit.db.grouppolicy import test_group_policy_db
 
 JSON_FORMAT = 'json'
-GP_PLUGIN_KLASS = (
-    "gbpservice.neutron.services.grouppolicy.plugin.GroupPolicyPlugin")
 
 
-class ServiceChainDBTestBase(test_group_policy_db.GroupPolicyDBTestBase):
+class ServiceChainDBTestBase(test_group_policy_db.ApiManagerMixin):
+    resource_prefix_map = dict(
+        (k, constants.COMMON_PREFIXES[constants.SERVICECHAIN])
+        for k in service_chain.RESOURCE_ATTRIBUTE_MAP.keys())
+    resource_prefix_map.update(dict(
+        (k, constants.COMMON_PREFIXES[constants.GROUP_POLICY])
+        for k in gpolicy.RESOURCE_ATTRIBUTE_MAP.keys()
+    ))
+
+    fmt = JSON_FORMAT
+
+    def __getattr__(self, item):
+        # Verify is an update of a proper GBP object
+
+        def _is_sc_resource(plural):
+            return plural in service_chain.RESOURCE_ATTRIBUTE_MAP
+
+        def _is_gbp_resource(plural):
+            return plural in gpolicy.RESOURCE_ATTRIBUTE_MAP
+
+        def _is_valid_resource(plural):
+            return _is_gbp_resource(plural) or _is_sc_resource(plural)
+        # Update Method
+        if item.startswith('update_'):
+            resource = item[len('update_'):]
+            plural = cm.get_resource_plural(resource)
+            if _is_valid_resource(plural):
+                def update_wrapper(id, **kwargs):
+                    return self._update_resource(id, resource, **kwargs)
+                return update_wrapper
+        # Show Method
+        if item.startswith('show_'):
+            resource = item[len('show_'):]
+            plural = cm.get_resource_plural(resource)
+            if _is_valid_resource(plural):
+                def show_wrapper(id, **kwargs):
+                    return self._show_resource(id, plural, **kwargs)
+                return show_wrapper
+        # Create Method
+        if item.startswith('create_'):
+            resource = item[len('create_'):]
+            plural = cm.get_resource_plural(resource)
+            if _is_valid_resource(plural):
+                def create_wrapper(**kwargs):
+                    return self._create_resource(resource, **kwargs)
+                return create_wrapper
+        # Delete Method
+        if item.startswith('delete_'):
+            resource = item[len('delete_'):]
+            plural = cm.get_resource_plural(resource)
+            if _is_valid_resource(plural):
+                def delete_wrapper(id, **kwargs):
+                    return self._delete_resource(id, plural, **kwargs)
+                return delete_wrapper
+
+        raise AttributeError
 
     def _get_resource_plural(self, resource):
         if resource.endswith('y'):
@@ -58,7 +117,7 @@ class ServiceChainDBTestBase(test_group_policy_db.GroupPolicyDBTestBase):
                          sorted([i[resource]['id'] for i in items]))
 
     def _create_profiled_servicechain_node(
-            self, service_type=constants.LOADBALANCERV2, shared_profile=False,
+            self, service_type=constants.LOADBALANCER, shared_profile=False,
             profile_tenant_id=None, **kwargs):
         prof = self.create_service_profile(
             service_type=service_type,
@@ -70,24 +129,39 @@ class ServiceChainDBTestBase(test_group_policy_db.GroupPolicyDBTestBase):
 
 class ServiceChainDBTestPlugin(svcchain_db.ServiceChainDbPlugin):
 
-    supported_extension_aliases = ['servicechain'] + (
-        test_group_policy_db.UNSUPPORTED_REQUIRED_EXTS)
-    path_prefix = "/servicechain"
+        supported_extension_aliases = ['servicechain']
+
 
 DB_GP_PLUGIN_KLASS = (ServiceChainDBTestPlugin.__module__ + '.' +
                       ServiceChainDBTestPlugin.__name__)
 
+GP_PLUGIN_KLASS = (
+    "gbpservice.neutron.services.grouppolicy.plugin.GroupPolicyPlugin")
 
-class ServiceChainDbTestCase(test_group_policy_db.GroupPolicyDbTestCase):
+
+class ServiceChainDbTestCase(ServiceChainDBTestBase,
+                             test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
 
     def setUp(self, core_plugin=None, sc_plugin=None, service_plugins=None,
               ext_mgr=None, gp_plugin=None):
+        if not sc_plugin:
+            sc_plugin = DB_GP_PLUGIN_KLASS
+        if not service_plugins:
+            service_plugins = {
+                'l3_plugin_name': 'router',
+                'gp_plugin_name': gp_plugin or GP_PLUGIN_KLASS,
+                'sc_plugin_name': sc_plugin}
 
         super(ServiceChainDbTestCase, self).setUp(
-            gp_plugin=gp_plugin or GP_PLUGIN_KLASS, core_plugin=core_plugin,
-            sc_plugin=sc_plugin, service_plugins=service_plugins,
-            ext_mgr=ext_mgr)
-        self.plugin = self._sc_plugin
+            plugin=core_plugin, ext_mgr=ext_mgr,
+            service_plugins=service_plugins
+        )
+        self.plugin = importutils.import_object(sc_plugin)
+        if not ext_mgr:
+            ext_mgr = extensions.PluginAwareExtensionManager.get_instance()
+            self.ext_api = test_extensions.setup_extensions_middleware(ext_mgr)
+        engine = db_api.get_engine()
+        model_base.BASEV2.metadata.create_all(engine)
 
 
 class TestServiceChainResources(ServiceChainDbTestCase):
@@ -99,7 +173,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
         res = self.deserialize(self.fmt,
                                req.get_response(self.ext_api))
 
-        for k, v in six.iteritems(attrs):
+        for k, v in attrs.iteritems():
             self.assertEqual(v, res[resource][k])
 
     def test_create_servicechain_specs_same_node(self):
@@ -142,7 +216,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
             service_profile_id=profile['service_profile']['id'],
             config="config1")
 
-        for k, v in six.iteritems(attrs):
+        for k, v in attrs.iteritems():
             self.assertEqual(v, scn['servicechain_node'][k])
 
         self._test_show_resource('servicechain_node',
@@ -180,7 +254,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
                                       scn['servicechain_node']['id'])
         res = self.deserialize(self.fmt, req.get_response(self.ext_api))
 
-        for k, v in six.iteritems(attrs):
+        for k, v in attrs.iteritems():
             self.assertEqual(v, res['servicechain_node'][k])
 
         self._test_show_resource('servicechain_node',
@@ -202,7 +276,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
 
         req = self.new_delete_request('servicechain_specs', scs_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
 
         # After deleting the Service Chain Spec, node delete should succeed
         req = self.new_delete_request('servicechain_nodes', scn_id)
@@ -222,7 +296,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
 
         scs = self.create_servicechain_spec(name=name, nodes=[scn_id])
 
-        for k, v in six.iteritems(attrs):
+        for k, v in attrs.iteritems():
             self.assertEqual(v, scs['servicechain_spec'][k])
 
         self._test_show_resource('servicechain_spec',
@@ -239,7 +313,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
             name=name, nodes=[scn1_id, scn2_id])
         scs = self.create_servicechain_spec(
             name=name, nodes=[scn1_id, scn2_id])
-        for k, v in six.iteritems(attrs):
+        for k, v in attrs.iteritems():
             self.assertEqual(v, scs['servicechain_spec'][k])
 
     def test_list_servicechain_specs(self):
@@ -291,7 +365,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
                                       scs['servicechain_spec']['id'])
         res = self.deserialize(self.fmt, req.get_response(self.ext_api))
 
-        for k, v in six.iteritems(attrs):
+        for k, v in attrs.iteritems():
             self.assertEqual(v, res['servicechain_spec'][k])
 
         self._test_show_resource('servicechain_spec',
@@ -337,7 +411,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
 
         req = self.new_delete_request('servicechain_instances', sci_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         self.assertRaises(service_chain.ServiceChainInstanceNotFound,
                           self.plugin.get_servicechain_instance,
                           ctx, sci_id)
@@ -345,7 +419,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
         # Deleting the spec should succeed after the instance is deleted
         req = self.new_delete_request('servicechain_specs', scs_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         self.assertRaises(service_chain.ServiceChainSpecNotFound,
                           self.plugin.get_servicechain_spec, ctx, scs_id)
 
@@ -369,7 +443,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
             management_ptg_id=policy_target_group_id,
             classifier_id=classifier_id,
             config_param_values=config_param_values)
-        for k, v in six.iteritems(attrs):
+        for k, v in attrs.iteritems():
             self.assertEqual(v, sci['servicechain_instance'][k])
 
         self._test_show_resource('servicechain_instance',
@@ -451,7 +525,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
         req = self.new_update_request('servicechain_instances', data,
                                       sci['servicechain_instance']['id'])
         res = self.deserialize(self.fmt, req.get_response(self.ext_api))
-        for k, v in six.iteritems(attrs):
+        for k, v in attrs.iteritems():
             self.assertEqual(v, res['servicechain_instance'][k])
 
         self._test_show_resource('servicechain_instance',
@@ -480,7 +554,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
         scn = self.create_service_profile(
             service_type=constants.FIREWALL, vendor="vendor1")
 
-        for k, v in six.iteritems(attrs):
+        for k, v in attrs.iteritems():
             self.assertEqual(scn['service_profile'][k], v)
 
         self._test_show_resource('service_profile',
@@ -488,11 +562,11 @@ class TestServiceChainResources(ServiceChainDbTestCase):
 
     def test_list_service_profile(self):
         scns = [self.create_service_profile(name='sp1', description='sp',
-                                            service_type='LOADBALANCERV2'),
+                                            service_type='LOADBALANCER'),
                 self.create_service_profile(name='sp2', description='sp',
-                                            service_type='LOADBALANCERV2'),
+                                            service_type='LOADBALANCER'),
                 self.create_service_profile(name='sp3', description='sp',
-                                            service_type='LOADBALANCERV2')]
+                                            service_type='LOADBALANCER')]
         self._test_list_resources('service_profile', scns,
                                   query_params='description=sp')
 
@@ -511,7 +585,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
                                       scn['service_profile']['id'])
         res = self.deserialize(self.fmt, req.get_response(self.ext_api))
 
-        for k, v in six.iteritems(attrs):
+        for k, v in attrs.iteritems():
             self.assertEqual(res['service_profile'][k], v)
 
         self._test_show_resource('service_profile',
@@ -520,7 +594,7 @@ class TestServiceChainResources(ServiceChainDbTestCase):
     def test_delete_service_profile(self):
         ctx = context.get_admin_context()
 
-        sp = self.create_service_profile(service_type='LOADBALANCERV2')
+        sp = self.create_service_profile(service_type='LOADBALANCER')
         sp_id = sp['service_profile']['id']
 
         scn = self.create_servicechain_node(service_profile_id=sp_id)
@@ -532,26 +606,17 @@ class TestServiceChainResources(ServiceChainDbTestCase):
 
         req = self.new_delete_request('servicechain_nodes', scn_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
 
         # After deleting the Service Chain Spec, node delete should succeed
         req = self.new_delete_request('service_profiles', sp_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         self.assertRaises(service_chain.ServiceProfileNotFound,
                           self.plugin.get_service_profile,
                           ctx, sp_id)
 
 
-class TestServiceChainStatusAttributesForResources(
-    test_group_policy_db.TestStatusAttributesForResources):
-
-    def test_set_status_attrs(self):
-        for resource_name in service_chain.RESOURCE_ATTRIBUTE_MAP:
-            self._test_set_status_attrs(self._get_resource_singular(
-                resource_name), self._sc_plugin)
-
-
 class TestQuotasForServiceChain(ServiceChainDbTestCase):
 
     def setUp(self, core_plugin=None, sc_plugin=None,
