@@ -12,24 +12,19 @@
 # limitations under the License.
 
 import mock
+from neutron import context
+from neutron import manager
 from neutron.tests.unit.plugins.ml2 import test_plugin
-from neutron_lib import context
 from oslo_config import cfg
-from oslo_log import log as logging
 import webob.exc
 
-from gbpservice.neutron.db.grouppolicy import group_policy_mapping_db as gpmdb
 from gbpservice.neutron.extensions import group_policy as gpolicy
-from gbpservice.neutron.services.grouppolicy import config
-from gbpservice.neutron.services.grouppolicy.drivers import dummy_driver
-from gbpservice.neutron.services.grouppolicy import plugin as gplugin
 from gbpservice.neutron.tests.unit.db.grouppolicy import (
     test_group_policy_db as tgpdb)
 from gbpservice.neutron.tests.unit.db.grouppolicy import (
     test_group_policy_mapping_db as tgpmdb)
 
 
-LOG = logging.getLogger(__name__)
 cfg.CONF.import_opt('policy_drivers',
                     'gbpservice.neutron.services.grouppolicy.config',
                     group='group_policy')
@@ -38,6 +33,9 @@ GP_PLUGIN_KLASS = (
 )
 SERVICECHAIN_SPECS = 'servicechain/servicechain_specs'
 SERVICECHAIN_NODES = 'servicechain/servicechain_nodes'
+AGENT_TYPE = 'Open vSwitch agent'
+AGENT_CONF = {'alive': True, 'binary': 'somebinary',
+              'topic': 'sometopic', 'agent_type': AGENT_TYPE}
 
 
 class FakeDriver(object):
@@ -49,35 +47,38 @@ class FakeDriver(object):
         return self._fill_order
 
 
-NEW_STATUS = 'new_status'
-NEW_STATUS_DETAILS = 'new_status_details'
+class GroupPolicyPluginTestCase(tgpmdb.GroupPolicyMappingDbTestCase):
 
-
-def get_status_for_test(self, context):
-    resource_name = [item for item in context.__dict__.keys()
-     if item.startswith('_original')][0][len('_original'):]
-    getattr(context, resource_name)['status'] = NEW_STATUS
-    getattr(context, resource_name)['status_details'] = NEW_STATUS_DETAILS
-
-
-class GroupPolicyPluginTestBase(tgpmdb.GroupPolicyMappingDbTestCase):
-
-    def setUp(self, core_plugin=None, l3_plugin=None, gp_plugin=None,
-              ml2_options=None, sc_plugin=None, qos_plugin=None,
-              trunk_plugin=None):
+    def setUp(self, core_plugin=None, gp_plugin=None, ml2_options=None):
         if not gp_plugin:
             gp_plugin = GP_PLUGIN_KLASS
-        ml2_opts = ml2_options or {'mechanism_drivers': ['openvswitch'],
-                                   'extension_drivers': ['port_security']}
+        ml2_opts = ml2_options or {'mechanism_drivers': ['openvswitch']}
         for opt, val in ml2_opts.items():
             cfg.CONF.set_override(opt, val, 'ml2')
         core_plugin = core_plugin or test_plugin.PLUGIN_NAME
-        super(GroupPolicyPluginTestBase, self).setUp(core_plugin=core_plugin,
-                                                     l3_plugin=l3_plugin,
-                                                     gp_plugin=gp_plugin,
-                                                     sc_plugin=sc_plugin,
-                                                     qos_plugin=qos_plugin,
-                                                     trunk_plugin=trunk_plugin)
+        super(GroupPolicyPluginTestCase, self).setUp(core_plugin=core_plugin,
+                                                     gp_plugin=gp_plugin)
+        cfg.CONF.set_override('servicechain_drivers', ['dummy'],
+                              group='servicechain')
+
+    def test_reverse_on_delete(self):
+        manager = self.plugin.policy_driver_manager
+        ctx = context.get_admin_context()
+        drivers = manager.ordered_policy_drivers
+        first, second = mock.Mock(), mock.Mock()
+        first.obj, second.obj = FakeDriver(), FakeDriver()
+        try:
+            manager.ordered_policy_drivers = [first, second]
+            manager.reverse_ordered_policy_drivers = [second, first]
+            ordered_obj = [first.obj, second.obj]
+            ctx.call_order = []
+            manager._call_on_drivers('nodelete', ctx)
+            self.assertEqual(ordered_obj, ctx.call_order)
+            ctx.call_order = []
+            manager._call_on_drivers('delete', ctx)
+            self.assertEqual(ordered_obj[::-1], ctx.call_order)
+        finally:
+            manager.ordered_policy_drivers = drivers
 
     def _create_l2_policy_on_shared(self, **kwargs):
         l3p = self.create_l3_policy(shared=True)['l3_policy']
@@ -120,7 +121,7 @@ class GroupPolicyPluginTestBase(tgpmdb.GroupPolicyMappingDbTestCase):
     def _create_servicechain_spec(self, node_types=None, shared=False):
         node_types = node_types or []
         if not node_types:
-            node_types = ['LOADBALANCERV2']
+            node_types = ['LOADBALANCER']
         node_ids = []
         for node_type in node_types:
             node_ids.append(self._create_servicechain_node(node_type,
@@ -136,7 +137,7 @@ class GroupPolicyPluginTestBase(tgpmdb.GroupPolicyMappingDbTestCase):
         scs_id = spec['servicechain_spec']['id']
         return scs_id
 
-    def _create_servicechain_node(self, node_type="LOADBALANCERV2",
+    def _create_servicechain_node(self, node_type="LOADBALANCER",
                                   shared=False):
         config = "{}"
         data = {'servicechain_node': {'service_type': node_type,
@@ -149,6 +150,25 @@ class GroupPolicyPluginTestBase(tgpmdb.GroupPolicyMappingDbTestCase):
         scn_id = node['servicechain_node']['id']
         return scn_id
 
+    def _bind_port_to_host(self, port_id, host):
+        plugin = manager.NeutronManager.get_plugin()
+        ctx = context.get_admin_context()
+        agent = {'host': host}
+        agent.update(AGENT_CONF)
+        plugin.create_or_update_agent(ctx, agent)
+        data = {'port': {'binding:host_id': host, 'device_owner': 'a',
+                         'device_id': 'b'}}
+        # Create EP with bound port
+        req = self.new_update_request('ports', data, port_id,
+                                      self.fmt)
+        return self.deserialize(self.fmt, req.get_response(self.api))
+
+    def _unbind_port(self, port_id):
+        data = {'port': {'binding:host_id': ''}}
+        req = self.new_update_request('ports', data, port_id,
+                                      self.fmt)
+        return self.deserialize(self.fmt, req.get_response(self.api))
+
     def _get_object(self, type, id, api, expected_res_status=None):
         req = self.new_show_request(type, id, self.fmt)
         res = req.get_response(api)
@@ -161,44 +181,6 @@ class GroupPolicyPluginTestBase(tgpmdb.GroupPolicyMappingDbTestCase):
         return self.deserialize(self.fmt, res)
 
 
-class GroupPolicyPluginTestCase(GroupPolicyPluginTestBase):
-
-    def tearDown(self):
-        policy_drivers = cfg.CONF.group_policy.policy_drivers
-        LOG.warning("PDs used in this test: %s",
-                    policy_drivers)
-        # Always reset configuration to dummy driver. Any
-        # test which requires to configure a different
-        # policy driver would have done so in it's setup
-        # (and should have ideally reset it too).
-        config.cfg.CONF.set_override('policy_drivers',
-                                     ['dummy'],
-                                     group='group_policy')
-        super(GroupPolicyPluginTestCase, self).tearDown()
-
-
-class TestGroupPolicyPluginDrivers(GroupPolicyPluginTestBase):
-
-    def test_reverse_on_delete(self):
-        manager = self.plugin.policy_driver_manager
-        ctx = context.get_admin_context()
-        drivers = manager.ordered_policy_drivers
-        first, second = mock.Mock(), mock.Mock()
-        first.obj, second.obj = FakeDriver(), FakeDriver()
-        try:
-            manager.ordered_policy_drivers = [first, second]
-            manager.reverse_ordered_policy_drivers = [second, first]
-            ordered_obj = [first.obj, second.obj]
-            ctx.call_order = []
-            manager._call_on_drivers('nodelete', ctx)
-            self.assertEqual(ordered_obj, ctx.call_order)
-            ctx.call_order = []
-            manager._call_on_drivers('delete', ctx)
-            self.assertEqual(ordered_obj[::-1], ctx.call_order)
-        finally:
-            manager.ordered_policy_drivers = drivers
-
-
 class TestL3Policy(GroupPolicyPluginTestCase):
 
     def _get_es_dict(self, es, addr=None):
@@ -207,10 +189,10 @@ class TestL3Policy(GroupPolicyPluginTestCase):
     def test_shared_l3_policy_create(self):
         # Verify default is False
         l3p = self.create_l3_policy()
-        self.assertFalse(l3p['l3_policy']['shared'])
+        self.assertEqual(False, l3p['l3_policy']['shared'])
         # Verify shared True created without errors
         l3p = self.create_l3_policy(shared=True)
-        self.assertTrue(l3p['l3_policy']['shared'])
+        self.assertEqual(True, l3p['l3_policy']['shared'])
 
     def test_shared_l3p_create_with_es(self):
         def combination(l3p, es):
@@ -345,28 +327,6 @@ class TestL3Policy(GroupPolicyPluginTestCase):
         self.assertEqual('InvalidL3PExternalIPAddress',
                          res['NeutronError']['type'])
 
-    def test_multiple_es_negative(self):
-        attrs = {'external_routes': [{'destination': '0.0.0.0/0',
-                                      'nexthop': '172.1.1.1'}],
-                 'cidr': '172.1.1.0/24'}
-        es = self.create_external_segment(**attrs)['external_segment']
-        another_es = self.create_external_segment(**attrs)['external_segment']
-
-        res = self.create_l3_policy(
-            external_segments={es['id']: [], another_es['id']: []},
-            expected_res_status=400)
-        self.assertEqual('IdenticalExternalRoute',
-                         res['NeutronError']['type'])
-
-        l3p = self.create_l3_policy(
-            external_segments={es['id']: []},
-            expected_res_status=201)['l3_policy']
-        res = self.update_l3_policy(l3p['id'],
-            external_segments={es['id']: [], another_es['id']: []},
-            expected_res_status=400)
-        self.assertEqual('IdenticalExternalRoute',
-                         res['NeutronError']['type'])
-
 
 class TestL2Policy(GroupPolicyPluginTestCase):
 
@@ -374,10 +334,10 @@ class TestL2Policy(GroupPolicyPluginTestCase):
         l3p = self.create_l3_policy(shared=True)['l3_policy']
         # Verify Default False
         l2p = self.create_l2_policy(l3_policy_id=l3p['id'])
-        self.assertFalse(l2p['l2_policy']['shared'])
+        self.assertEqual(False, l2p['l2_policy']['shared'])
         # Verify shared True created without errors
         l2p = self.create_l2_policy(l3_policy_id=l3p['id'], shared=True)
-        self.assertTrue(l2p['l2_policy']['shared'])
+        self.assertEqual(True, l2p['l2_policy']['shared'])
 
     def test_shared_l2_policy_update(self):
         l2p = self._create_l2_policy_on_shared()
@@ -427,11 +387,11 @@ class TestPolicyRuleSet(GroupPolicyPluginTestCase):
         # Verify shared policy_rule_set created with shared rules
         prs = self._create_policy_rule_set_on_shared(
             shared=True, expected_res_status=201)
-        self.assertTrue(prs['shared'])
+        self.assertEqual(True, prs['shared'])
 
         # Verify non shared policy_rule_set created with shared rules
         prs = self._create_policy_rule_set_on_shared(expected_res_status=201)
-        self.assertFalse(prs['shared'])
+        self.assertEqual(False, prs['shared'])
 
     def test_shared_policy_rule_set_update(self):
         prs = self._create_policy_rule_set_on_shared()
@@ -480,11 +440,11 @@ class TestPolicyRule(GroupPolicyPluginTestCase):
         # Verify shared rule created with shared actions and classifier
         pr = self._create_rule_on_shared(shared=True,
                                          expected_res_status=201)
-        self.assertTrue(pr['shared'])
+        self.assertEqual(True, pr['shared'])
 
         # Verify non shared rule create with shared actions and classifier
         pr = self._create_rule_on_shared(expected_res_status=201)
-        self.assertFalse(pr['shared'])
+        self.assertEqual(False, pr['shared'])
 
     def test_shared_rule_update(self):
         pr = self._create_rule_on_shared()
@@ -571,7 +531,7 @@ class TestPolicyTargetGroup(GroupPolicyPluginTestCase):
         req = self.new_delete_request('policy_target_groups', ptg['id'],
                                       self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         self.assertRaises(gpolicy.PolicyTargetGroupNotFound,
                           self.plugin.get_policy_target_group, ctx, ptg['id'])
 
@@ -605,33 +565,33 @@ class TestPolicyTargetGroup(GroupPolicyPluginTestCase):
         # policy_rule_sets
         ptg = self.create_policy_target_group(
             l2_policy_id=l2p['id'], expected_res_status=201)
-        self.assertFalse(ptg['policy_target_group']['shared'])
+        self.assertEqual(False, ptg['policy_target_group']['shared'])
         ptg = self.create_policy_target_group(
             l2_policy_id=l2p['id'],
             provided_policy_rule_sets={prs['id']: '', ctns['id']: ''},
             consumed_policy_rule_sets={prs['id']: '', ctns['id']: ''},
             expected_res_status=201)
-        self.assertFalse(ptg['policy_target_group']['shared'])
+        self.assertEqual(False, ptg['policy_target_group']['shared'])
         ptg = self.create_policy_target_group(
             l2_policy_id=l2p['id'], network_service_policy_id=nsp['id'],
             expected_res_status=201)
-        self.assertFalse(ptg['policy_target_group']['shared'])
+        self.assertEqual(False, ptg['policy_target_group']['shared'])
         ptg = self.create_policy_target_group(
             l2_policy_id=l2p['id'], network_service_policy_id=nspns['id'],
             expected_res_status=201)
-        self.assertFalse(ptg['policy_target_group']['shared'])
+        self.assertEqual(False, ptg['policy_target_group']['shared'])
 
         # Verify shared True created without errors by providing/consuming
         # shared policy_rule_sets
         ptg = self.create_policy_target_group(
             l2_policy_id=l2p['id'], shared=True,
             expected_res_status=201)
-        self.assertTrue(ptg['policy_target_group']['shared'])
+        self.assertEqual(True, ptg['policy_target_group']['shared'])
         ptg = self.create_policy_target_group(
             l2_policy_id=l2p['id'], provided_policy_rule_sets={prs['id']: ''},
             consumed_policy_rule_sets={prs['id']: ''}, shared=True,
             expected_res_status=201)
-        self.assertTrue(ptg['policy_target_group']['shared'])
+        self.assertEqual(True, ptg['policy_target_group']['shared'])
 
         # Verify not shared created without error on not shared l2p
         self.create_policy_target_group(l2_policy_id=l2pns['id'],
@@ -717,27 +677,16 @@ class TestPolicyTargetGroup(GroupPolicyPluginTestCase):
         self.assertEqual('ManagementPolicyTargetGroupExists',
                          res['NeutronError']['type'])
 
-    def test_update_l2p_rejectet(self):
-        l2p_1 = self.create_l2_policy()['l2_policy']
-        l2p_2 = self.create_l2_policy()['l2_policy']
-        ptg = self.create_policy_target_group(
-            l2_policy_id=l2p_1['id'],
-            expected_res_status=201)['policy_target_group']
-        res = self.update_policy_target_group(
-            ptg['id'], l2_policy_id=l2p_2['id'], expected_res_status=400)
-        self.assertEqual('L2PolicyUpdateOfPolicyTargetGroupNotSupported',
-                         res['NeutronError']['type'])
-
 
 class TestExternalSegment(GroupPolicyPluginTestCase):
 
     def test_shared_es_create(self):
         # Verify default is False
         es = self.create_external_segment()
-        self.assertFalse(es['external_segment']['shared'])
+        self.assertEqual(False, es['external_segment']['shared'])
         # Verify shared True created without errors
         es = self.create_external_segment(shared=True)
-        self.assertTrue(es['external_segment']['shared'])
+        self.assertEqual(True, es['external_segment']['shared'])
 
     def test_shared_es_update(self):
         es = self.create_external_segment()['external_segment']
@@ -774,23 +723,6 @@ class TestExternalSegment(GroupPolicyPluginTestCase):
         self.assertEqual('ExternalRouteOverlapsWithL3PIpPool',
                          res['NeutronError']['type'])
 
-    def test_identical_routes_negative(self):
-        attrs = {'external_routes': [{'destination': '0.0.0.0/0',
-                                      'nexthop': '172.1.0.1'}],
-                 'cidr': '172.1.0.0/24'}
-        es1 = self.create_external_segment(**attrs)['external_segment']
-        es2 = self.create_external_segment()['external_segment']
-
-        self.create_l3_policy(
-            external_segments={es1['id']: [], es2['id']: []},
-            expected_res_status=201)
-
-        del attrs['cidr']
-        res = self.update_external_segment(
-            es2['id'], expected_res_status=400, **attrs)
-        self.assertEqual('IdenticalExternalRoute',
-                         res['NeutronError']['type'])
-
 
 class TestExternalPolicy(GroupPolicyPluginTestCase):
 
@@ -807,26 +739,26 @@ class TestExternalPolicy(GroupPolicyPluginTestCase):
         # policy_rule_sets
         ep = self.create_external_policy(
             external_segments=[es['id']], expected_res_status=201)
-        self.assertFalse(ep['external_policy']['shared'])
+        self.assertEqual(False, ep['external_policy']['shared'])
         ep = self.create_external_policy(
             external_segments=[es['id']],
             provided_policy_rule_sets={prs['id']: '', prsns['id']: ''},
             consumed_policy_rule_sets={prs['id']: '', prsns['id']: ''},
             expected_res_status=201)
-        self.assertFalse(ep['external_policy']['shared'])
+        self.assertEqual(False, ep['external_policy']['shared'])
 
         # Verify shared True created without errors by providing/consuming
         # shared policy_rule_sets
         ep = self.create_external_policy(
             external_segments=[es['id']], shared=True,
             expected_res_status=201)
-        self.assertTrue(ep['external_policy']['shared'])
+        self.assertEqual(True, ep['external_policy']['shared'])
         ep = self.create_external_policy(
             external_segments=[es['id']],
             provided_policy_rule_sets={prs['id']: ''},
             consumed_policy_rule_sets={prs['id']: ''}, shared=True,
             expected_res_status=201)
-        self.assertTrue(ep['external_policy']['shared'])
+        self.assertEqual(True, ep['external_policy']['shared'])
 
         # Verify not shared created without error on not shared es
         self.create_external_policy(
@@ -921,8 +853,8 @@ class TestNatPool(GroupPolicyPluginTestCase):
 
 class TestPolicyTarget(GroupPolicyPluginTestCase):
 
-    def _test_cross_tenant(self, is_admin=False):
-        status = {False: 404, True: 201}
+    def _test_cross_tenant_fails(self, is_admin=False):
+        status = {False: 404, True: 400}
         ptg = self.create_policy_target_group(
             expected_res_status=201, tenant_id='tenant',
             is_admin_context=is_admin)['policy_target_group']
@@ -933,6 +865,9 @@ class TestPolicyTarget(GroupPolicyPluginTestCase):
         if not is_admin:
             self.assertEqual(
                 'GbpResourceNotFound', res['NeutronError']['type'])
+        else:
+            self.assertEqual(
+                'InvalidCrossTenantReference', res['NeutronError']['type'])
 
         # Create EP without PTG
         pt = self.create_policy_target(
@@ -940,132 +875,28 @@ class TestPolicyTarget(GroupPolicyPluginTestCase):
             is_admin_context=is_admin)['policy_target']
 
         # Update PT fails
-        res = self.update_policy_target(
-            pt['id'], tenant_id='another', policy_target_group_id=ptg['id'],
-            expected_res_status=status[is_admin] if not is_admin else 200,
-            is_admin_context=is_admin)
-
+        res = self.update_policy_target(pt['id'],
+                                        expected_res_status=status[is_admin],
+                                        tenant_id='another',
+                                        policy_target_group_id=ptg['id'],
+                                        is_admin_context=is_admin)
         if not is_admin:
             self.assertEqual(
                 'GbpResourceNotFound', res['NeutronError']['type'])
+        else:
+            self.assertEqual(
+                'InvalidCrossTenantReference', res['NeutronError']['type'])
 
     def test_cross_tenant_fails(self):
-        self._test_cross_tenant()
+        self._test_cross_tenant_fails()
 
-    def test_cross_tenant_admin(self):
-        self._test_cross_tenant(True)
-
-
-class TestResourceStatusChange(GroupPolicyPluginTestCase):
-
-    def setUp(self, core_plugin=None, gp_plugin=None, ml2_options=None,
-              sc_plugin=None):
-        for resource_name in gpolicy.RESOURCE_ATTRIBUTE_MAP:
-            method_name = "get_" + self._get_resource_singular(
-                resource_name) + "_status"
-            setattr(dummy_driver.NoopDriver, method_name, get_status_for_test)
-        super(TestResourceStatusChange, self).setUp(
-            core_plugin=core_plugin, gp_plugin=gp_plugin, sc_plugin=sc_plugin)
-
-    def _test_status_change_on_get(self, resource_name, fields=None):
-        resource_singular = self._get_resource_singular(resource_name)
-        if resource_name == 'policy_rules':
-            pc_id = self.create_policy_classifier()['policy_classifier']['id']
-            resource = self.create_policy_rule(policy_classifier_id=pc_id)
-        else:
-            resource = getattr(self, "create_" + resource_singular)()
-        self.assertEqual(NEW_STATUS, resource[resource_singular]['status'])
-        self.assertEqual(NEW_STATUS_DETAILS,
-                         resource[resource_singular]['status_details'])
-
-        # Reset status directly in the DB to test that GET works
-        reset_status = {resource_singular: {'status': None,
-                                            'status_details': None}}
-        neutron_context = context.Context('', self._tenant_id)
-        getattr(gpmdb.GroupPolicyMappingDbPlugin,
-                "update_" + resource_singular)(
-                    self._gbp_plugin, neutron_context,
-                    resource[resource_singular]['id'], reset_status)
-
-        req = self.new_show_request(
-            resource_name, resource[resource_singular]['id'], fmt=self.fmt,
-            fields=fields)
-        res = self.deserialize(self.fmt, req.get_response(self.ext_api))
-
-        if not fields:
-            self.assertEqual(NEW_STATUS, res[resource_singular]['status'])
-            self.assertEqual(NEW_STATUS_DETAILS,
-                             res[resource_singular]['status_details'])
-        elif not gplugin.STATUS_SET.intersection(set(fields)):
-            db_obj = getattr(
-                gpmdb.GroupPolicyMappingDbPlugin, "get_" + resource_singular)(
-                self._gbp_plugin, neutron_context,
-                resource[resource_singular]['id'])
-            self.assertIsNone(db_obj['status'])
-            self.assertIsNone(db_obj['status_details'])
-
-    def test_status_change_on_get(self):
-        for resource_name in gpolicy.RESOURCE_ATTRIBUTE_MAP:
-            self._test_status_change_on_get(resource_name)
-
-    def test_no_status_change_on_get(self):
-        # We explicitly populate the fields list with no status attributes
-        for resource_name in gpolicy.RESOURCE_ATTRIBUTE_MAP:
-            self._test_status_change_on_get(resource_name,
-                                            fields=['name'])
-
-    def _test_status_change_on_list(self, resource_name, fields=None):
-        resource_singular = self._get_resource_singular(resource_name)
-        if resource_name == 'policy_rules':
-            pc_id = self.create_policy_classifier()['policy_classifier']['id']
-            objs = [self.create_policy_rule(policy_classifier_id=pc_id),
-                    self.create_policy_rule(policy_classifier_id=pc_id),
-                    self.create_policy_rule(policy_classifier_id=pc_id)]
-        else:
-            create_method = "create_" + resource_singular
-            objs = [getattr(self, create_method)(),
-                    getattr(self, create_method)(),
-                    getattr(self, create_method)()]
-
-        neutron_context = context.Context('', self._tenant_id)
-        reset_status = {resource_singular: {'status': None,
-                                            'status_details': None}}
-        for obj in objs:
-            # Reset status directly in the DB to test that GET works
-            getattr(gpmdb.GroupPolicyMappingDbPlugin,
-                    "update_" + resource_singular)(
-                        self._gbp_plugin, neutron_context,
-                        obj[resource_singular]['id'], reset_status)
-            req = self.new_show_request(
-                resource_name, obj[resource_singular]['id'], fmt=self.fmt,
-                fields=fields)
-            res = self.deserialize(self.fmt, req.get_response(self.ext_api))
-
-            if not fields:
-                self.assertEqual(NEW_STATUS, res[resource_singular]['status'])
-                self.assertEqual(NEW_STATUS_DETAILS,
-                                 res[resource_singular]['status_details'])
-            elif not gplugin.STATUS_SET.intersection(set(fields)):
-                db_obj = getattr(
-                    gpmdb.GroupPolicyMappingDbPlugin,
-                    "get_" + resource_singular)(
-                    self._gbp_plugin, neutron_context,
-                    obj[resource_singular]['id'])
-                self.assertIsNone(db_obj['status'])
-                self.assertIsNone(db_obj['status_details'])
-
-    def test_status_change_on_list(self):
-        for resource_name in gpolicy.RESOURCE_ATTRIBUTE_MAP:
-            self._test_status_change_on_list(resource_name)
-
-    def test_no_status_change_on_list(self):
-        for resource_name in gpolicy.RESOURCE_ATTRIBUTE_MAP:
-            self._test_status_change_on_list(resource_name, fields=['name'])
+    def test_cross_tenant_fails_admin(self):
+        self._test_cross_tenant_fails(True)
 
 
 class TestPolicyAction(GroupPolicyPluginTestCase):
 
-    def test_redirect_value(self):
+    def test_redirect_value_fails(self):
         scs_id = self._create_servicechain_spec(
             node_types=['FIREWALL_TRANSPARENT'])
         res = self.create_policy_action(action_type='redirect',
@@ -1082,7 +913,9 @@ class TestPolicyAction(GroupPolicyPluginTestCase):
 
         res = self.create_policy_action(
             action_type='redirect', action_value=scs_id, tenant_id='different',
-            expected_res_status=201, is_admin_context=True)
+            expected_res_status=400, is_admin_context=True)
+        self.assertEqual(
+            'InvalidCrossTenantReference', res['NeutronError']['type'])
 
         res = self.create_policy_action(
             action_type='redirect', action_value=scs_id,
@@ -1096,7 +929,7 @@ class TestPolicyAction(GroupPolicyPluginTestCase):
             node_types=['FIREWALL_TRANSPARENT'], shared=True)
         self.create_policy_action(
             action_type='redirect', action_value=scs_id, shared=True,
-            expected_res_status=201)
+            expected_res_status=201)['policy_action']
         data = {'servicechain_spec': {'shared': False}}
         scs_req = self.new_update_request(
             SERVICECHAIN_SPECS, data, scs_id, self.fmt)
@@ -1120,93 +953,4 @@ class TestGroupPolicyPluginGroupResources(
 
 class TestGroupPolicyPluginMappedGroupResourceAttrs(
         GroupPolicyPluginTestCase, tgpmdb.TestMappedGroupResourceAttrs):
-
     pass
-
-
-class TestQuotasForGBP(GroupPolicyPluginTestCase):
-
-    def setUp(self, core_plugin=None, gp_plugin=None):
-        cfg.CONF.set_override('quota_l3_policy', 1, group='QUOTAS')
-        cfg.CONF.set_override('quota_l2_policy', 1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_target_group', 1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_target', 1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_action', 1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_classifier', 1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_rule', 1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_rule_set', 1, group='QUOTAS')
-        cfg.CONF.set_override('quota_network_service_policy', 1,
-                              group='QUOTAS')
-        cfg.CONF.set_override('quota_external_policy', 1, group='QUOTAS')
-        cfg.CONF.set_override('quota_external_segment', 1, group='QUOTAS')
-        cfg.CONF.set_override('quota_nat_pool', 1, group='QUOTAS')
-        super(TestQuotasForGBP, self).setUp(
-            core_plugin=core_plugin, gp_plugin=gp_plugin)
-
-    def tearDown(self):
-        cfg.CONF.set_override('quota_l3_policy', -1, group='QUOTAS')
-        cfg.CONF.set_override('quota_l2_policy', -1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_target_group', -1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_target', -1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_action', -1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_classifier', -1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_rule', -1, group='QUOTAS')
-        cfg.CONF.set_override('quota_policy_rule_set', -1, group='QUOTAS')
-        cfg.CONF.set_override('quota_network_service_policy', -1,
-                              group='QUOTAS')
-        cfg.CONF.set_override('quota_external_policy', -1, group='QUOTAS')
-        cfg.CONF.set_override('quota_external_segment', -1, group='QUOTAS')
-        cfg.CONF.set_override('quota_nat_pool', -1, group='QUOTAS')
-        super(TestQuotasForGBP, self).tearDown()
-
-    def test_group_resources_quota(self):
-        ptg_id = self.create_policy_target_group()['policy_target_group']['id']
-        self.create_policy_target(policy_target_group_id=ptg_id)
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_policy_target,
-                          policy_target_group_id=ptg_id)
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_policy_target_group)
-
-    def test_l3policy_quota(self):
-        self.create_l3_policy()
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_l3_policy)
-
-    def test_l2policy_quota(self):
-        self.create_l2_policy()
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_l2_policy)
-
-    def test_policy_resources_quota(self):
-        pa_id = self.create_policy_action()['policy_action']['id']
-        pc_id = self.create_policy_classifier()['policy_classifier']['id']
-        pr_id = self.create_policy_rule(
-            policy_classifier_id=pc_id,
-            policy_actions=[pa_id])['policy_rule']['id']
-        self.create_policy_rule_set(policy_rules=[pr_id])
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_policy_action)
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_policy_classifier)
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_policy_rule,
-                          policy_classifier_id=pc_id,
-                          policy_actions=[pa_id])
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_policy_rule_set,
-                          policy_rules=[pr_id])
-        self.create_network_service_policy()
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_network_service_policy)
-
-    def test_external_connectivity_resources_quota(self):
-        self.create_external_policy()
-        self.create_external_segment()
-        self.create_nat_pool()
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_external_policy)
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_external_segment)
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_nat_pool)
