@@ -11,9 +11,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from neutron.db import api as db_api
+from neutron.db import model_base
 from oslo_config import cfg
 import webob.exc
 
+from gbpservice.neutron.db import servicechain_db  # noqa
+from gbpservice.neutron.services.servicechain.plugins.ncp import (  # noqa
+    model)  # noqa
 from gbpservice.neutron.tests.unit.services.grouppolicy import (
     test_grouppolicy_plugin as test_plugin)
 
@@ -25,6 +30,8 @@ class ImplicitPolicyTestCase(
         cfg.CONF.set_override('policy_drivers', ['implicit_policy'],
                               group='group_policy')
         super(ImplicitPolicyTestCase, self).setUp()
+        engine = db_api.get_engine()
+        model_base.BASEV2.metadata.create_all(engine)
 
 
 class TestImplicitL2Policy(ImplicitPolicyTestCase):
@@ -57,19 +64,19 @@ class TestImplicitL2Policy(ImplicitPolicyTestCase):
         # policy.
         req = self.new_delete_request('policy_target_groups', ptg1_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         req = self.new_show_request('l2_policies', l2p1_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNotFound.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNotFound.code)
 
         # Verify deleting 2nd policy_target group does cleanup its L2
         # policy.
         req = self.new_delete_request('policy_target_groups', ptg2_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         req = self.new_show_request('l2_policies', l2p2_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNotFound.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNotFound.code)
 
     def test_implicit_shared_lifecycle_negative(self):
         # Create PTG non shared
@@ -96,12 +103,44 @@ class TestImplicitL2Policy(ImplicitPolicyTestCase):
         # Verify deleting policy_target group does not cleanup L2 policy.
         req = self.new_delete_request('policy_target_groups', ptg_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         req = self.new_show_request('l2_policies', l2p_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPOk.code)
 
-    def test_delete_from_implicit(self):
+    def test_update_from_implicit(self):
+        # Create policy_target group with implicit L2 policy.
+        ptg = self.create_policy_target_group()
+        ptg_id = ptg['policy_target_group']['id']
+        l2p1_id = ptg['policy_target_group']['l2_policy_id']
+        req = self.new_show_request('l2_policies', l2p1_id, fmt=self.fmt)
+        l2p1 = self.deserialize(self.fmt, req.get_response(self.ext_api))
+        self.assertEqual(ptg['policy_target_group']['name'],
+                         l2p1['l2_policy']['name'])
+
+        # Update policy_target group to explicit L2 policy.
+        l2p2 = self.create_l2_policy()
+        l2p2_id = l2p2['l2_policy']['id']
+        data = {'policy_target_group': {'l2_policy_id': l2p2_id}}
+        req = self.new_update_request('policy_target_groups', data, ptg_id)
+        ptg = self.deserialize(self.fmt, req.get_response(self.ext_api))
+        self.assertEqual(l2p2_id, ptg['policy_target_group']['l2_policy_id'])
+
+        # Verify old L2 policy was cleaned up.
+        req = self.new_show_request('l2_policies', l2p1_id, fmt=self.fmt)
+        res = req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, webob.exc.HTTPNotFound.code)
+
+        # Verify deleting policy_target group does not cleanup new L2
+        # policy.
+        req = self.new_delete_request('policy_target_groups', ptg_id)
+        res = req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
+        req = self.new_show_request('l2_policies', l2p2_id, fmt=self.fmt)
+        res = req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, webob.exc.HTTPOk.code)
+
+    def test_update_to_implicit(self):
         # Create policy_target group with explicit L2 policy.
         l2p1 = self.create_l2_policy()
         l2p1_id = l2p1['l2_policy']['id']
@@ -109,13 +148,30 @@ class TestImplicitL2Policy(ImplicitPolicyTestCase):
         ptg_id = ptg['policy_target_group']['id']
         self.assertEqual(l2p1_id, ptg['policy_target_group']['l2_policy_id'])
 
-        # Delete PTG
-        self.delete_policy_target_group(ptg_id, expected_res_status=204)
+        # Update policy_target group to implicit L2 policy.
+        data = {'policy_target_group': {'l2_policy_id': None}}
+        req = self.new_update_request('policy_target_groups', data, ptg_id)
+        ptg = self.deserialize(self.fmt, req.get_response(self.ext_api))
+        l2p2_id = ptg['policy_target_group']['l2_policy_id']
+        self.assertNotEqual(l2p1_id, l2p2_id)
+        self.assertIsNotNone(l2p2_id)
+        req = self.new_show_request('l2_policies', l2p2_id, fmt=self.fmt)
+        l2p2 = self.deserialize(self.fmt, req.get_response(self.ext_api))
+        self.assertEqual(ptg['policy_target_group']['name'],
+                         l2p2['l2_policy']['name'])
 
         # Verify old L2 policy was not cleaned up.
         req = self.new_show_request('l2_policies', l2p1_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPOk.code)
+
+        # Verify deleting policy_target group does cleanup new L2 policy.
+        req = self.new_delete_request('policy_target_groups', ptg_id)
+        res = req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
+        req = self.new_show_request('l2_policies', l2p2_id, fmt=self.fmt)
+        res = req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, webob.exc.HTTPNotFound.code)
 
 
 class TestImplicitL3Policy(ImplicitPolicyTestCase):
@@ -144,18 +200,18 @@ class TestImplicitL3Policy(ImplicitPolicyTestCase):
         # Verify deleting 1st L2 policy does not cleanup L3 policy.
         req = self.new_delete_request('l2_policies', l2p1_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         req = self.new_show_request('l3_policies', l3p_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPOk.code)
 
         # Verify deleting last L2 policy does cleanup L3 policy.
         req = self.new_delete_request('l2_policies', l2p2_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         req = self.new_show_request('l3_policies', l3p_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNotFound.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNotFound.code)
 
     def test_impicit_lifecycle(self):
         self._test_implicit_lifecycle()
@@ -174,10 +230,10 @@ class TestImplicitL3Policy(ImplicitPolicyTestCase):
         # Verify deleting L2 policy does not cleanup L3 policy.
         req = self.new_delete_request('l2_policies', l2p_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         req = self.new_show_request('l3_policies', l3p_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPOk.code)
 
     def test_unowned_default_lifecycle(self):
         # Create L2 policy with unowned default L3 policy.
@@ -190,10 +246,10 @@ class TestImplicitL3Policy(ImplicitPolicyTestCase):
         # Verify deleting L2 policy does not cleanup L3 policy.
         req = self.new_delete_request('l2_policies', l2p_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         req = self.new_show_request('l3_policies', l3p_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPOk.code)
 
     def test_single_default_policy(self):
         # Verify only one default L3 policy can be created per tenant.
@@ -223,15 +279,15 @@ class TestImplicitL3Policy(ImplicitPolicyTestCase):
         # Verify old L3 policy was cleaned up
         req = self.new_show_request('l3_policies', l3p1_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNotFound.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNotFound.code)
 
         # Verify deleting L2 policy does not cleanup new L3 policy.
         req = self.new_delete_request('l2_policies', l2p_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         req = self.new_show_request('l3_policies', l3p2_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPOk.code)
 
     def test_update_to_implicit(self):
         # Create L2 policy with explicit L3 policy.
@@ -255,15 +311,15 @@ class TestImplicitL3Policy(ImplicitPolicyTestCase):
         # Verify old L3 policy was not cleaned up.
         req = self.new_show_request('l3_policies', l3p1_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPOk.code)
 
         # Verify deleting L2 policy does cleanup new L3 policy.
         req = self.new_delete_request('l2_policies', l2p_id)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNoContent.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNoContent.code)
         req = self.new_show_request('l3_policies', l3p2_id, fmt=self.fmt)
         res = req.get_response(self.ext_api)
-        self.assertEqual(webob.exc.HTTPNotFound.code, res.status_int)
+        self.assertEqual(res.status_int, webob.exc.HTTPNotFound.code)
 
 
 class TestImplicitExternalSegment(ImplicitPolicyTestCase):
