@@ -16,11 +16,11 @@ import itertools
 import copy
 import heatclient
 import mock
-from neutron_lib.api.definitions import external_net
-from neutron_lib import context as neutron_context
-from neutron_lib.plugins import constants
+from neutron import context as neutron_context
+from neutron.extensions import external_net as external_net
+from neutron.openstack.common import uuidutils
+from neutron.plugins.common import constants
 from oslo_serialization import jsonutils
-from oslo_utils import uuidutils
 import webob
 
 from gbpservice.neutron.services.servicechain.plugins.ncp import config
@@ -63,9 +63,6 @@ class MockHeatClientFunctions(object):
     def get(self, stack_id):
         return MockStackObject('DELETE_COMPLETE')
 
-    def update(self, *args, **fields):
-        return {'stack': {'id': uuidutils.generate_uuid()}}
-
 
 class MockHeatClientDeleteNotFound(object):
     def __init__(self, api_version, endpoint, **kwargs):
@@ -75,7 +72,6 @@ class MockHeatClientDeleteNotFound(object):
 class MockHeatClient(object):
     def __init__(self, api_version, endpoint, **kwargs):
         self.stacks = MockHeatClientFunctions()
-        self.resources = mock.MagicMock()
 
 
 class HeatNodeDriverTestCase(
@@ -85,27 +81,31 @@ class HeatNodeDriverTestCase(
             "AWSTemplateFormatVersion": "2010-09-09",
             "Resources": {
                 "test_pool": {
-                    "Type": "OS::Neutron::LBaaS::Pool",
+                    "Type": "OS::Neutron::Pool",
                     "Properties": {
-                        "description": "Haproxy pool from template",
-                        "lb_algorithm": "ROUND_ROBIN",
+                        "admin_state_up": True,
+                        "description": "Haproxy pool from teplate",
+                        "lb_method": "ROUND_ROBIN",
+                        "monitors": [{"Ref": "HttpHM"}],
+                        "name": "Haproxy pool",
                         "protocol": "HTTP",
-                        'listener': {u'get_resource': u'listener'},
-                    }
-                },
-                "test_listener": {
-                    "Type": "OS::Neutron::LBaaS::Listener",
-                    "Properties": {
-                        "protocol": "HTTP",
-                        "protocol_port": 80,
+                        "subnet_id": {"Ref": "Subnet"},
+                        "vip": {
+                            "subnet": {"Ref": "Subnet"},
+                            "address": {"Ref": "vip_ip"},
+                            "name": "Haproxy vip",
+                            "protocol_port": 80,
+                            "connection_limit": -1,
+                            "admin_state_up": True,
+                            "description": "Haproxy vip from template"
+                        }
                     }
                 },
                 "test_lb": {
-                    "Type": "OS::Neutron::LBaaS::LoadBalancer",
+                    "Type": "OS::Neutron::LoadBalancer",
                     "Properties": {
-                        "provider": 'haproxy',
-                        'vip_address': '1.1.1.1',
-                        'vip_subnet': '1.1.1.0/24',
+                        "pool_id": {"Ref": "HaproxyPool"},
+                        "protocol_port": 80
                     }
                 }
             }
@@ -148,7 +148,7 @@ class HeatNodeDriverTestCase(
                    new=MockHeatClient).start()
         super(HeatNodeDriverTestCase, self).setUp(
             node_drivers=['heat_node_driver'],
-            node_plumber='stitching_plumber',
+            node_plumber='agnostic_plumber',
             core_plugin=test_gp_driver.CORE_PLUGIN)
 
     def _create_network(self, fmt, name, admin_state_up, **kwargs):
@@ -170,7 +170,7 @@ class HeatNodeDriverTestCase(
             self.assertTrue(driver.obj.initialized)
 
     def _create_profiled_servicechain_node(
-            self, service_type=constants.LOADBALANCERV2, shared_profile=False,
+            self, service_type=constants.LOADBALANCER, shared_profile=False,
             profile_tenant_id=None, profile_id=None, **kwargs):
         if not profile_id:
             prof = self.create_service_profile(
@@ -234,14 +234,13 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
         member_ip = port['fixed_ips'][0]['ip_address']
         member_name = 'mem-' + member_ip
         member = {member_name: {
-                        'Type': 'OS::Neutron::LBaaS::PoolMember',
+                        'Type': 'OS::Neutron::PoolMember',
                         'Properties': {
-                            'subnet': {'get_param': 'Subnet'},
-                            'weight': 1,
+                            'protocol_port': '80',
                             'admin_state_up': True,
-                            'address': member_ip,
-                            'protocol_port': {'get_param': 'app_port'},
-                            'pool': {'Ref': u'test_pool'}
+                            'pool_id': {'Ref': u'test_pool'},
+                            'weight': 1,
+                            'address': member_ip
                         }
                     }
                   }
@@ -264,7 +263,7 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
                     shared=True,
                     name="default",
                     external_routes=routes,
-                    subnet_id=sub['subnet']['id'])
+                    subnet_id=sub['subnet']['id'])['external_segment']
                 return self.create_external_policy(
                     consumed_policy_rule_sets={consumed_prs: ''})
 
@@ -275,14 +274,15 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
                                         'id': uuidutils.generate_uuid()}}
 
             node_id = self._create_profiled_servicechain_node(
-                service_type=constants.LOADBALANCERV2)[
-                        'servicechain_node']['id']
+                service_type=constants.LOADBALANCER)['servicechain_node']['id']
             spec = self.create_servicechain_spec(
                 nodes=[node_id],
                 expected_res_status=201)['servicechain_spec']
 
             prs = self._create_redirect_prs(spec['id'])['policy_rule_set']
-            provider = self.create_policy_target_group()['policy_target_group']
+            provider = self.create_policy_target_group(
+                provided_policy_rule_sets={prs['id']: ''})[
+                                                'policy_target_group']
 
             _, port1 = self._create_policy_target_port(provider['id'])
             _, port2 = self._create_policy_target_port(provider['id'])
@@ -293,8 +293,6 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
                 self.create_policy_target_group(
                     consumed_policy_rule_sets={prs['id']: ''})
 
-            self.update_policy_target_group(
-                provider['id'], provided_policy_rule_sets={prs['id']: ''})
             created_stacks_map = self._get_node_instance_stacks(node_id)
             self.assertEqual(1, len(created_stacks_map))
 
@@ -410,20 +408,18 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
 
     def _get_firewall_rule_dict(self, rule_name, protocol, port, provider_cidr,
                                 consumer_cidr):
-        if provider_cidr and consumer_cidr:
-            fw_rule = {rule_name: {'type': "OS::Neutron::FirewallRule",
-                                   'properties': {
-                                       "protocol": protocol,
-                                       "enabled": True,
-                                       "destination_port": port,
-                                       "action": "allow",
-                                       "destination_ip_address": provider_cidr,
-                                       "source_ip_address": consumer_cidr
-                                   }
-                                   }
-                       }
-            return fw_rule
-        return {}
+        fw_rule = {rule_name: {'type': "OS::Neutron::FirewallRule",
+                               'properties': {
+                                    "protocol": protocol,
+                                    "enabled": True,
+                                    "destination_port": port,
+                                    "action": "allow",
+                                    "destination_ip_address": provider_cidr,
+                                    "source_ip_address": consumer_cidr
+                               }
+                               }
+                   }
+        return fw_rule
 
     def test_fw_node_east_west(self):
         classifier_port = '66'
@@ -437,26 +433,22 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
             provider = self.create_policy_target_group(
                 provided_policy_rule_sets={prs['id']: ''})[
                                                 'policy_target_group']
-            self.create_policy_target_group(
-                consumed_policy_rule_sets={prs['id']: ''})
+            consumer = self.create_policy_target_group(
+                consumed_policy_rule_sets={prs['id']: ''})[
+                                                'policy_target_group']
 
             created_stacks_map = self._get_node_instance_stacks(node_id)
             self.assertEqual(1, len(created_stacks_map))
             stack_id = created_stacks_map[0].stack_id
 
             provider_cidr = self._get_ptg_cidr(provider)
-            # TODO(ivar): This has to be removed once support to consumer list
-            # is implemented
-            #consumer_cidr = self._get_ptg_cidr(consumer)
-            consumer_cidr = []
+            consumer_cidr = self._get_ptg_cidr(consumer)
             fw_rule = self._get_firewall_rule_dict(
                 'Rule_1', classifier_protocol, classifier_port,
                 provider_cidr, consumer_cidr)
 
             expected_stack_template = copy.deepcopy(
                                         self.DEFAULT_FW_CONFIG_DICT)
-            expected_stack_template['resources'][
-                'test_fw_policy']['properties']['firewall_rules'] = []
             expected_stack_template['resources'].update(fw_rule)
             expected_stack_name = mock.ANY
             expected_stack_params = {}
@@ -465,7 +457,7 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
                     expected_stack_template,
                     expected_stack_params)
 
-            self._test_node_cleanup(provider, stack_id)
+            self._test_node_cleanup(consumer, stack_id)
 
     def _test_fw_node_north_south(self, consumer_cidrs):
         classifier_port = '66'
@@ -483,11 +475,8 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
             routes = []
             for consumer_cidr in consumer_cidrs:
                 routes.append({'destination': consumer_cidr, 'nexthop': None})
-            self._create_external_policy(prs['id'], routes=routes)
-
-            # TODO(ivar): This has to be removed once support to consumer list
-            # is implemented
-            consumer_cidrs = []
+            self._create_external_policy(
+                        prs['id'], routes=routes)['external_policy']
 
             created_stacks_map = self._get_node_instance_stacks(node_id)
             self.assertEqual(1, len(created_stacks_map))
@@ -532,7 +521,7 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
             stack_create.return_value = {'stack': {
                                         'id': uuidutils.generate_uuid()}}
             prof = self.create_service_profile(
-                        service_type=constants.LOADBALANCERV2,
+                        service_type=constants.LOADBALANCER,
                         vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
 
             node = self.create_servicechain_node(
@@ -591,6 +580,8 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
                     self.delete_policy_target_group(provider['id'],
                                                 expected_res_status=204)
                     stack_delete.assert_called_once_with(mock.ANY)
+                    self.assertEqual(STACK_ACTION_WAIT_TIME / 5,
+                                     stack_get.call_count)
 
             # Create and delete another service chain instance and verify that
             # we call get method for heat stack only once if the stack state
@@ -610,6 +601,7 @@ class TestServiceChainInstance(HeatNodeDriverTestCase):
                     self.delete_policy_target_group(provider['id'],
                                                 expected_res_status=204)
                     stack_delete.assert_called_once_with(mock.ANY)
+                    self.assertEqual(1, stack_get.call_count)
 
     def test_stack_not_found_ignored(self):
         mock.patch(heatclient.__name__ + ".client.Client",
