@@ -14,55 +14,70 @@
 import webob.exc
 
 import mock
-from neutron.common import config
-from neutron_lib import context as n_context
-from neutron_lib import exceptions as n_exc
-from neutron_lib.plugins import constants as pconst
-from neutron_lib.plugins import directory
+from neutron.common import config  # noqa
+from neutron.common import exceptions as n_exc
+from neutron import context as n_context
+from neutron.db import api as db_api
+from neutron.db import model_base
+from neutron import manager
+from neutron.plugins.common import constants as pconst
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 
+from gbpservice.neutron.db.grouppolicy import group_policy_mapping_db  # noqa
 from gbpservice.neutron.services.grouppolicy import config as gpconfig  # noqa
 from gbpservice.neutron.services.servicechain.plugins.ncp import (
     context as ncp_context)
 from gbpservice.neutron.services.servicechain.plugins.ncp import (
     exceptions as exc)
-from gbpservice.neutron.services.servicechain.plugins.ncp import (
-    plugin as ncp_plugin)
 import gbpservice.neutron.services.servicechain.plugins.ncp.config  # noqa
+from gbpservice.neutron.services.servicechain.plugins.ncp import model
 from gbpservice.neutron.services.servicechain.plugins.ncp.node_drivers import (
     dummy_driver as dummy_driver)
-from gbpservice.neutron.tests.unit.db.grouppolicy import test_group_policy_db
 from gbpservice.neutron.tests.unit.services.grouppolicy import (
     test_resource_mapping as test_gp_driver)
 from gbpservice.neutron.tests.unit.services.servicechain import (
-    base_test_servicechain_plugin as test_base)
+    test_servicechain_plugin as test_base)
 
-
-class ServiceChainNCPTestPlugin(ncp_plugin.NodeCompositionPlugin):
-
-    supported_extension_aliases = ['servicechain'] + (
-        test_group_policy_db.UNSUPPORTED_REQUIRED_EXTS)
-    path_prefix = "/servicechain"
-
-
-SC_PLUGIN_KLASS = (ServiceChainNCPTestPlugin.__module__ + '.' +
-                   ServiceChainNCPTestPlugin.__name__)
-CORE_PLUGIN = test_gp_driver.CORE_PLUGIN
+SC_PLUGIN_KLASS = (
+    "gbpservice.neutron.services.servicechain.plugins.ncp.plugin."
+    "NodeCompositionPlugin")
+CORE_PLUGIN = ('gbpservice.neutron.tests.unit.services.grouppolicy.'
+               'test_resource_mapping.NoL3NatSGTestPlugin')
 GP_PLUGIN_KLASS = (
     "gbpservice.neutron.services.grouppolicy.plugin.GroupPolicyPlugin"
 )
-CHAIN_TENANT_ID = 'sci_owner'
 
 
-class NodeCompositionPluginTestMixin(object):
+class NodeCompositionPluginTestCase(
+        test_base.TestGroupPolicyPluginGroupResources):
 
     DEFAULT_LB_CONFIG = '{}'
     SERVICE_PROFILE_VENDOR = 'dummy'
 
+    def setUp(self, core_plugin=None, gp_plugin=None, node_drivers=None,
+              node_plumber=None):
+        if node_drivers:
+            cfg.CONF.set_override('node_drivers', node_drivers,
+                                  group='node_composition_plugin')
+        cfg.CONF.set_override('node_plumber', node_plumber or 'dummy_plumber',
+                              group='node_composition_plugin')
+        config.cfg.CONF.set_override('policy_drivers',
+                                     ['implicit_policy', 'resource_mapping'],
+                                     group='group_policy')
+        super(NodeCompositionPluginTestCase, self).setUp(
+            core_plugin=core_plugin or CORE_PLUGIN,
+            gp_plugin=gp_plugin or GP_PLUGIN_KLASS,
+            sc_plugin=SC_PLUGIN_KLASS)
+        engine = db_api.get_engine()
+        model_base.BASEV2.metadata.create_all(engine)
+        self.driver = self.sc_plugin.driver_manager.ordered_drivers[0].obj
+
     @property
     def sc_plugin(self):
-        return directory.get_plugin(pconst.SERVICECHAIN)
+        plugins = manager.NeutronManager.get_service_plugins()
+        servicechain_plugin = plugins.get(pconst.SERVICECHAIN)
+        return servicechain_plugin
 
     def _create_service_profile(self, **kwargs):
         """Create service profile wrapper that can be used by drivers."""
@@ -83,14 +98,13 @@ class NodeCompositionPluginTestMixin(object):
         prs = self.create_policy_rule_set(policy_rules=[rule['id']])
         return prs
 
-    def _create_simple_service_chain(self, number_of_nodes=1,
-                                     service_type='LOADBALANCERV2'):
-        prof = self.create_service_profile(
-            service_type=service_type,
+    def _create_simple_service_chain(self, number_of_nodes=1):
+        prof = self._create_service_profile(
+            service_type='LOADBALANCER',
             vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
 
         node_ids = []
-        for x in range(number_of_nodes):
+        for x in xrange(number_of_nodes):
             node_ids.append(self.create_servicechain_node(
                 service_profile_id=prof['id'],
                 config=self.DEFAULT_LB_CONFIG,
@@ -118,54 +132,11 @@ class NodeCompositionPluginTestMixin(object):
         self.sc_plugin.driver_manager.ordered_drivers.append(ext)
         self.sc_plugin.driver_manager.drivers[name] = ext
 
-
-class NodeCompositionPluginTestCase(
-        test_base.BaseTestGroupPolicyPluginGroupResources,
-        NodeCompositionPluginTestMixin):
-
-    def setUp(self, core_plugin=None, gp_plugin=None, node_drivers=None,
-              node_plumber=None):
-        cfg.CONF.set_override(
-            'extension_drivers', ['proxy_group'], group='group_policy')
-        if node_drivers:
-            cfg.CONF.set_override('node_drivers', node_drivers,
-                                  group='node_composition_plugin')
-        cfg.CONF.set_override('node_plumber', node_plumber or 'dummy_plumber',
-                              group='node_composition_plugin')
-        config.cfg.CONF.set_override('policy_drivers',
-                                     ['implicit_policy', 'resource_mapping',
-                                      'chain_mapping'],
-                                     group='group_policy')
-        super(NodeCompositionPluginTestCase, self).setUp(
-            core_plugin=core_plugin or CORE_PLUGIN,
-            gp_plugin=gp_plugin or GP_PLUGIN_KLASS,
-            sc_plugin=SC_PLUGIN_KLASS)
-        self.driver = self.sc_plugin.driver_manager.ordered_drivers[0].obj
-
-    def _create_simple_chain(self):
-        node = self._create_profiled_servicechain_node(
-            service_type="LOADBALANCERV2",
-            config=self.DEFAULT_LB_CONFIG)['servicechain_node']
-        spec = self.create_servicechain_spec(
-            nodes=[node['id']])['servicechain_spec']
-
-        action = self.create_policy_action(
-            action_type='REDIRECT', action_value=spec['id'])['policy_action']
-        classifier = self.create_policy_classifier(
-            direction='bi', port_range=80, protocol='tcp')['policy_classifier']
-        rule = self.create_policy_rule(
-            policy_classifier_id=classifier['id'],
-            policy_actions=[action['id']])['policy_rule']
-
-        prs = self.create_policy_rule_set(
-            policy_rules=[rule['id']])['policy_rule_set']
-
-        provider = self.create_policy_target_group(
-            provided_policy_rule_sets={prs['id']: ''})['policy_target_group']
-        consumer = self.create_policy_target_group(
-            consumed_policy_rule_sets={prs['id']: ''})['policy_target_group']
-
-        return provider, consumer, node
+    @property
+    def sc_plugin(self):
+        plugins = manager.NeutronManager.get_service_plugins()
+        servicechain_plugin = plugins.get(pconst.SERVICECHAIN)
+        return servicechain_plugin
 
     def test_spec_ordering_list_servicechain_instances(self):
         pass
@@ -174,7 +145,7 @@ class NodeCompositionPluginTestCase(
         # Verify Context attributes for simple config
         plugin_context = n_context.get_admin_context()
         profile = self._create_service_profile(
-            service_type="LOADBALANCERV2",
+            service_type="LOADBALANCER",
             vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
         node = self.create_servicechain_node(
             service_profile_id=profile['id'],
@@ -182,13 +153,13 @@ class NodeCompositionPluginTestCase(
         spec = self.create_servicechain_spec(
             nodes=[node['id']])['servicechain_spec']
         provider = self.create_policy_target_group()['policy_target_group']
-        self.create_policy_target_group()
+        consumer = self.create_policy_target_group()['policy_target_group']
         management = self.create_policy_target_group(
-            service_management=True,
-            is_admin_context=True)['policy_target_group']
+            service_management=True)['policy_target_group']
         classifier = self.create_policy_classifier()['policy_classifier']
+
         instance = self.create_servicechain_instance(
-            provider_ptg_id=provider['id'], consumer_ptg_id='N/A',
+            provider_ptg_id=provider['id'], consumer_ptg_id=consumer['id'],
             servicechain_specs=[spec['id']], classifier_id=classifier['id'])[
                                                     'servicechain_instance']
 
@@ -205,25 +176,19 @@ class NodeCompositionPluginTestCase(
         self.assertIsNotNone(ctx.admin_context)
         self.assertIsNotNone(ctx.admin_session)
         del ctx.current_profile['nodes']
-        self.assertEqual(ctx.current_profile['id'], profile['id'])
+        self.assertEqual(ctx.current_profile, profile)
         self.assertEqual(instance['id'], ctx.instance['id'])
         self.assertEqual(provider['id'], ctx.provider['id'])
-        self.assertIsNone(ctx.consumer)
+        self.assertEqual(consumer['id'], ctx.consumer['id'])
         self.assertEqual(management['id'], ctx.management['id'])
         self.assertEqual([spec['id']], [x['id'] for x in ctx.relevant_specs])
         self.assertIsNone(ctx.original_node)
         self.assertEqual(0, len(ctx.get_service_targets()))
 
-        instance['provider_ptg_id'] = 'dummy-id'
-        ctx = ncp_context.get_node_driver_context(
-            self.plugin, plugin_context, instance, node)
-        self.assertIsNone(ctx.provider)
-        self.assertIsNone(ctx.consumer)
-
     def test_context_relevant_specs(self):
         plugin_context = n_context.get_admin_context()
         node_used = self._create_profiled_servicechain_node(
-            service_type="LOADBALANCERV2",
+            service_type="LOADBALANCER",
             config=self.DEFAULT_LB_CONFIG)['servicechain_node']
         spec_used = self.create_servicechain_spec(
             nodes=[node_used['id']])['servicechain_spec']
@@ -262,7 +227,7 @@ class NodeCompositionPluginTestCase(
                                     dict((x, {}) for x in params)})
 
         prof = self._create_service_profile(
-            service_type='LOADBALANCERV2', shared=True,
+            service_type='LOADBALANCER', shared=True,
             vendor=self.SERVICE_PROFILE_VENDOR,
             tenant_id='admin')['service_profile']
 
@@ -356,9 +321,11 @@ class NodeCompositionPluginTestCase(
 
     def test_update_node_fails(self):
         validate_update = self.driver.validate_update = mock.Mock()
+        validate_update.side_effect = exc.NodeCompositionPluginBadRequest(
+            resource='node', msg='reason')
 
         prof = self._create_service_profile(
-            service_type='LOADBALANCERV2',
+            service_type='LOADBALANCER',
             vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
 
         node_id = self.create_servicechain_node(
@@ -375,9 +342,6 @@ class NodeCompositionPluginTestCase(
         self.create_policy_target_group(
             consumed_policy_rule_sets={prs['id']: ''})
 
-        validate_update.side_effect = exc.NodeCompositionPluginBadRequest(
-            resource='node', msg='reason')
-
         res = self.update_servicechain_node(node_id,
                                             description='somethingelse',
                                             expected_res_status=400)
@@ -386,7 +350,7 @@ class NodeCompositionPluginTestCase(
 
     def test_update_instantiated_profile_fails(self):
         prof = self._create_service_profile(
-            service_type='LOADBALANCERV2',
+            service_type='LOADBALANCER',
             vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
 
         node_id = self.create_servicechain_node(
@@ -425,9 +389,10 @@ class NodeCompositionPluginTestCase(
         spec = self.create_servicechain_spec(
             nodes=[node['id']])['servicechain_spec']
         provider = self.create_policy_target_group()['policy_target_group']
+        consumer = self.create_policy_target_group()['policy_target_group']
         classifier = self.create_policy_classifier()['policy_classifier']
         self.create_servicechain_instance(
-            provider_ptg_id=provider['id'], consumer_ptg_id='N/A',
+            provider_ptg_id=provider['id'], consumer_ptg_id=consumer['id'],
             servicechain_specs=[spec['id']], classifier_id=classifier['id'],
             expected_res_status=201)
 
@@ -449,16 +414,17 @@ class NodeCompositionPluginTestCase(
         spec = self.create_servicechain_spec(
             nodes=[node['id']])['servicechain_spec']
         provider = self.create_policy_target_group()['policy_target_group']
+        consumer = self.create_policy_target_group()['policy_target_group']
         classifier = self.create_policy_classifier()['policy_classifier']
         self.create_servicechain_instance(
-            provider_ptg_id=provider['id'], consumer_ptg_id='N/A',
+            provider_ptg_id=provider['id'], consumer_ptg_id=consumer['id'],
             servicechain_specs=[spec['id']], classifier_id=classifier['id'],
             expected_res_status=400)
 
     def test_multiple_nodes_update(self):
         update = self.driver.update = mock.Mock()
         prof = self._create_service_profile(
-            service_type='LOADBALANCERV2',
+            service_type='LOADBALANCER',
             vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
         node = self.create_servicechain_node(
             service_profile_id=prof['id'],
@@ -474,9 +440,9 @@ class NodeCompositionPluginTestCase(
         self.update_servicechain_node(node['id'], name='somethingelse')
         self.assertEqual(3, update.call_count)
 
-    def test_inuse_spec_node_update_rejected(self):
+    def test_update_spec(self):
         prof = self.create_service_profile(
-            service_type='LOADBALANCERV2',
+            service_type='LOADBALANCER',
             vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
 
         node1 = self.create_servicechain_node(
@@ -497,13 +463,12 @@ class NodeCompositionPluginTestCase(
 
         res = self.update_servicechain_spec(spec['id'],
                                             nodes=[node1['id']],
-                                            expected_res_status=400)
-        self.assertEqual('InuseSpecNodeUpdateNotAllowed',
-                         res['NeutronError']['type'])
+                                            expected_res_status=200)
+        self.assertEqual([node1['id']], res['servicechain_spec']['nodes'])
 
     def test_instance_update(self):
         prof = self.create_service_profile(
-            service_type='LOADBALANCERV2',
+            service_type='LOADBALANCER',
             vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
 
         node1 = self.create_servicechain_node(
@@ -539,7 +504,7 @@ class NodeCompositionPluginTestCase(
         rem = self.driver.update_policy_target_removed = mock.Mock()
 
         prof = self._create_service_profile(
-            service_type='LOADBALANCERV2',
+            service_type='LOADBALANCER',
             vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
         node = self.create_servicechain_node(
             service_profile_id=prof['id'],
@@ -552,28 +517,37 @@ class NodeCompositionPluginTestCase(
         prs = self._create_redirect_prs(spec['id'])['policy_rule_set']
         provider = self.create_policy_target_group(
             provided_policy_rule_sets={prs['id']: ''})['policy_target_group']
-        self.create_policy_target_group(
-            consumed_policy_rule_sets={prs['id']: ''})
+        consumer = self.create_policy_target_group(
+            consumed_policy_rule_sets={prs['id']: ''})['policy_target_group']
 
         # Verify notification issued for created PT in the provider
         pt = self.create_policy_target(
             policy_target_group_id=provider['id'])['policy_target']
-        pt['port_attributes'] = {}
         self.assertEqual(1, add.call_count)
         add.assert_called_with(mock.ANY, pt)
-        del pt['port_attributes']
 
         # Verify notification issued for deleted PT in the provider
         self.delete_policy_target(pt['id'])
         self.assertEqual(1, rem.call_count)
         rem.assert_called_with(mock.ANY, pt)
 
+        # Verify notification issued for created PT in the consumer
+        pt = self.create_policy_target(
+            policy_target_group_id=consumer['id'])['policy_target']
+        self.assertEqual(2, add.call_count)
+        add.assert_called_with(mock.ANY, pt)
+
+        # Verify notification issued for deleted PT in the consumer
+        self.delete_policy_target(pt['id'])
+        self.assertEqual(2, rem.call_count)
+        rem.assert_called_with(mock.ANY, pt)
+
     def test_irrelevant_ptg_update(self):
         add = self.driver.update_policy_target_added = mock.Mock()
         rem = self.driver.update_policy_target_removed = mock.Mock()
 
         prof = self._create_service_profile(
-            service_type='LOADBALANCERV2',
+            service_type='LOADBALANCER',
             vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
         node = self.create_servicechain_node(
             service_profile_id=prof['id'],
@@ -603,7 +577,7 @@ class NodeCompositionPluginTestCase(
         update_hook = self.driver.notify_chain_parameters_updated = mock.Mock()
 
         prof = self.create_service_profile(
-            service_type='LOADBALANCERV2',
+            service_type='LOADBALANCER',
             vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
         node = self.create_servicechain_node(
             service_profile_id=prof['id'],
@@ -626,7 +600,7 @@ class NodeCompositionPluginTestCase(
         self.create_policy_target_group(
             provided_policy_rule_sets={prs['id']: ''})
         self.create_policy_target_group(
-            consumed_policy_rule_sets={prs['id']: ''})
+            consumed_policy_rule_sets={prs['id']: ''})['policy_target_group']
         instances = self._list('servicechain_instances')[
             'servicechain_instances']
         self.assertEqual(1, len(instances))
@@ -644,6 +618,7 @@ class NodeCompositionPluginTestCase(
         spec = self.create_servicechain_spec(
             nodes=[node['id']])['servicechain_spec']
         provider = self.create_policy_target_group()['policy_target_group']
+        consumer = self.create_policy_target_group()['policy_target_group']
 
         # Verify admin created SM is None
         management = self.create_policy_target_group(
@@ -651,7 +626,7 @@ class NodeCompositionPluginTestCase(
             is_admin_context=True)['policy_target_group']
         pc = self.create_policy_classifier()['policy_classifier']
         instance = self.create_servicechain_instance(
-            provider_ptg_id=provider['id'], consumer_ptg_id='N/A',
+            provider_ptg_id=provider['id'], consumer_ptg_id=consumer['id'],
             servicechain_specs=[spec['id']],
             classifier_id=pc['id'])['servicechain_instance']
         ctx = ncp_context.get_node_driver_context(
@@ -665,7 +640,7 @@ class NodeCompositionPluginTestCase(
             service_management=True, tenant_id='admin',
             is_admin_context=True, shared=True)['policy_target_group']
         instance = self.create_servicechain_instance(
-            provider_ptg_id=provider['id'], consumer_ptg_id='N/A',
+            provider_ptg_id=provider['id'], consumer_ptg_id=consumer['id'],
             servicechain_specs=[spec['id']],
             classifier_id=pc['id'])['servicechain_instance']
         # Now admin Service Management PTG is visible
@@ -678,135 +653,123 @@ class NodeCompositionPluginTestCase(
             service_management=True,
             is_admin_context=True)['policy_target_group']
         instance = self.create_servicechain_instance(
-            provider_ptg_id=provider['id'], consumer_ptg_id='N/A',
+            provider_ptg_id=provider['id'], consumer_ptg_id=consumer['id'],
             servicechain_specs=[spec['id']],
             classifier_id=pc['id'])['servicechain_instance']
         ctx = ncp_context.get_node_driver_context(
             self.plugin, plugin_context, instance, node)
         self.assertEqual(private_management['id'], ctx.management['id'])
 
-    def test_node_drivers_notified_consumer_event(self):
-        add = self.driver.update_node_consumer_ptg_added = mock.Mock()
-        rem = self.driver.update_node_consumer_ptg_removed = mock.Mock()
 
-        prof = self._create_service_profile(
-            service_type='LOADBALANCERV2',
-            vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
-        node = self.create_servicechain_node(
-            service_profile_id=prof['id'],
-            config=self.DEFAULT_LB_CONFIG,
-            expected_res_status=201)['servicechain_node']
+class AgnosticChainPlumberTestCase(NodeCompositionPluginTestCase):
 
+    def setUp(self):
+        cfg.CONF.set_override('policy_drivers', ['implicit_policy',
+                                                 'resource_mapping'],
+                              group='group_policy')
+        cfg.CONF.set_override('allow_overlapping_ips', True)
+
+        super(AgnosticChainPlumberTestCase, self).setUp(
+            node_drivers=['node_dummy'], node_plumber='agnostic_plumber',
+            core_plugin=test_gp_driver.CORE_PLUGIN)
+        res = mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.'
+                         '_check_router_needs_rescheduling').start()
+        res.return_value = None
+        self.driver = self.sc_plugin.driver_manager.ordered_drivers[0].obj
+        self.driver.get_plumbing_info = mock.Mock()
+        self.driver.get_plumbing_info.return_value = {}
+
+    def _create_simple_chain(self):
+        node = self._create_profiled_servicechain_node(
+            service_type="LOADBALANCER",
+            config=self.DEFAULT_LB_CONFIG)['servicechain_node']
         spec = self.create_servicechain_spec(
-            nodes=[node['id']],
-            expected_res_status=201)['servicechain_spec']
-        prs = self._create_redirect_prs(spec['id'])['policy_rule_set']
-        self.create_policy_target_group(
-            provided_policy_rule_sets={prs['id']: ''})
+            nodes=[node['id']])['servicechain_spec']
+
+        action = self.create_policy_action(
+            action_type='REDIRECT', action_value=spec['id'])['policy_action']
+        classifier = self.create_policy_classifier(
+            direction='bi', port_range=80, protocol='tcp')['policy_classifier']
+        rule = self.create_policy_rule(
+            policy_classifier_id=classifier['id'],
+            policy_actions=[action['id']])['policy_rule']
+
+        prs = self.create_policy_rule_set(
+            policy_rules=[rule['id']])['policy_rule_set']
+
+        provider = self.create_policy_target_group(
+            provided_policy_rule_sets={prs['id']: ''})['policy_target_group']
         consumer = self.create_policy_target_group(
             consumed_policy_rule_sets={prs['id']: ''})['policy_target_group']
 
-        # Verify notification issued for PTG consuming
-        add.assert_called_with(mock.ANY, consumer)
+        return provider, consumer, node
 
-        # Verify notification issued for PTG unconsuming
-        consumer = self.update_policy_target_group(
-            consumer['id'],
-            consumed_policy_rule_sets={})['policy_target_group']
-        rem.assert_called_with(mock.ANY, consumer)
+    def test_one_pt_prov_cons(self):
+        context = n_context.get_admin_context()
+        self.driver.get_plumbing_info.return_value = {'provider': [{}],
+                                                      'consumer': [{}]}
+        provider, consumer, node = self._create_simple_chain()
 
-        provider, consumer, prs = self._create_simple_service_chain(3)
-        with mock.patch.object(ncp_plugin.NodeCompositionPlugin,
-                "update_chains_consumer_removed") as ptg_removed:
-                plugin_context = n_context.get_admin_context()
-                self._gbp_plugin.delete_policy_target_group(plugin_context,
-                        consumer['id'])
+        # Verify Service PT created and correctly placed
+        prov_cons = {'provider': provider, 'consumer': consumer}
+        targets = model.get_service_targets(context.session)
+        self.assertEqual(2, len(targets))
+        old_relationship = None
+        for target in targets:
+            self.assertEqual(node['id'], target.servicechain_node_id)
+            pt = self.show_policy_target(
+                target.policy_target_id)['policy_target']
+            self.assertEqual(prov_cons[target.relationship]['id'],
+                             pt['policy_target_group_id'])
+            self.assertNotEqual(old_relationship, target.relationship)
+            old_relationship = target.relationship
 
-                self.assertEqual(ptg_removed.call_count, 1)
-                consumer['consumed_policy_rule_sets'] = []
-                ptg_removed.assert_called_once_with(
-                        mock.ANY, consumer, mock.ANY)
+        self.update_policy_target_group(
+            provider['id'], provided_policy_rule_sets={})
+        # With chain deletion, also the Service PTs are deleted
+        new_targets = model.get_service_targets(context.session)
+        self.assertEqual(0, len(new_targets))
+        for target in targets:
+            self.show_policy_target(
+                target.policy_target_id, expected_res_status=404)
 
-        add.reset_mock()
-        rem.reset_mock()
+    def test_pt_override(self):
+        context = n_context.get_admin_context()
+        test_name = 'test_name'
+        self.driver.get_plumbing_info.return_value = {
+            'provider': [{'name': test_name}]}
+        self._create_simple_chain()
+        targets = model.get_service_targets(context.session)
+        self.assertEqual(1, len(targets))
+        pt = self.show_policy_target(
+            targets[0].policy_target_id)['policy_target']
+        self.assertEqual(test_name, pt['name'])
 
-    def test_no_unrelated_chains_notified(self):
-        add = self.driver.update_node_consumer_ptg_added = mock.Mock()
-        rem = self.driver.update_node_consumer_ptg_removed = mock.Mock()
+    def test_ptg_delete(self):
+        self.driver.get_plumbing_info.return_value = {'provider': [{}],
+                                                      'consumer': [{}]}
+        provider, _, _ = self._create_simple_service_chain()
+        # Deleting a PTG will fail because of existing PTs
+        res = self.delete_policy_target_group(provider['id'],
+                                              expected_res_status=400)
+        self.assertEqual('PolicyTargetGroupInUse',
+                         res['NeutronError']['type'])
 
-        prof = self._create_service_profile(
-            service_type='LOADBALANCERV2',
-            vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
-        node = self.create_servicechain_node(
-            service_profile_id=prof['id'],
-            config=self.DEFAULT_LB_CONFIG,
-            expected_res_status=201)['servicechain_node']
-
-        spec = self.create_servicechain_spec(
-            nodes=[node['id']],
-            expected_res_status=201)['servicechain_spec']
-        prs = self._create_redirect_prs(spec['id'])['policy_rule_set']
-        # This creates a chain
-        self.create_policy_target_group(
-            provided_policy_rule_sets={prs['id']: ''})
-
-        # Create a PRS and assign a consumer with no provider (hence, no chain)
-        prs = self._create_redirect_prs(spec['id'])['policy_rule_set']
-        ptg = self.create_policy_target_group(
-            consumed_policy_rule_sets={prs['id']: ''})['policy_target_group']
-
-        # No notification should be issued
-        self.assertFalse(add.called)
-        self.assertFalse(rem.called)
-
-        # Remove the consumer
-        self.update_policy_target_group(ptg['id'],
-                                        consumed_policy_rule_sets={},
+        # Removing the PRSs will make the PTG deletable again
+        self.update_policy_target_group(provider['id'],
+                                        provided_policy_rule_sets={},
                                         expected_res_status=200)
-        # No notification should be issued
-        self.assertFalse(add.called)
-        self.assertFalse(rem.called)
-
-    def test_node_drivers_notified_provider_updated(self):
-        upd = self.driver.policy_target_group_updated = mock.Mock()
-
-        prof = self._create_service_profile(
-            service_type='LOADBALANCERV2',
-            vendor=self.SERVICE_PROFILE_VENDOR)['service_profile']
-        node = self.create_servicechain_node(
-            service_profile_id=prof['id'],
-            config=self.DEFAULT_LB_CONFIG,
-            expected_res_status=201)['servicechain_node']
-
-        spec = self.create_servicechain_spec(
-            nodes=[node['id']],
-            expected_res_status=201)['servicechain_spec']
-        prs = self._create_redirect_prs(spec['id'])['policy_rule_set']
-        provider = self.create_policy_target_group(
-            provided_policy_rule_sets={prs['id']: ''})['policy_target_group']
-
-        # TODO(Sumit): Remove the following mocks
-        # once Heat node driver supports reporting status
-        provider['status'] = mock.ANY
-        provider['status_details'] = mock.ANY
-        # Verify notification issued for PTG consuming
-        upd.assert_called_with(mock.ANY, None, provider)
-        upd.reset_mock()
-
-        # Verify notification issued for PTG consuming
-        new_provider = self.update_policy_target_group(
-            provider['id'],
-            consumed_policy_rule_sets={prs['id']: ''})['policy_target_group']
-        upd.assert_called_with(mock.ANY, provider, new_provider)
-
-        upd.reset_mock()
+        self.delete_policy_target_group(provider['id'],
+                                        expected_res_status=204)
 
 
 class TestQuotasForServiceChain(test_base.ServiceChainPluginTestCase):
 
     @property
     def sc_plugin(self):
-        return directory.get_plugin(pconst.SERVICECHAIN)
+        plugins = manager.NeutronManager.get_service_plugins()
+        servicechain_plugin = plugins.get(pconst.SERVICECHAIN)
+        return servicechain_plugin
 
     def setUp(self, core_plugin=None, gp_plugin=None, node_drivers=None,
               node_plumber=None):
@@ -816,13 +779,14 @@ class TestQuotasForServiceChain(test_base.ServiceChainPluginTestCase):
         cfg.CONF.set_override('node_plumber', node_plumber or 'dummy_plumber',
                               group='node_composition_plugin')
         config.cfg.CONF.set_override('policy_drivers',
-                                     ['implicit_policy', 'resource_mapping',
-                                      'chain_mapping'],
+                                     ['implicit_policy', 'resource_mapping'],
                                      group='group_policy')
         super(TestQuotasForServiceChain, self).setUp(
             core_plugin=core_plugin or CORE_PLUGIN,
             gp_plugin=gp_plugin or GP_PLUGIN_KLASS,
             sc_plugin=SC_PLUGIN_KLASS)
+        engine = db_api.get_engine()
+        model_base.BASEV2.metadata.create_all(engine)
         self.driver = self.sc_plugin.driver_manager.ordered_drivers[0].obj
         cfg.CONF.set_override('quota_servicechain_node', 1,
                               group='QUOTAS')
@@ -844,30 +808,9 @@ class TestQuotasForServiceChain(test_base.ServiceChainPluginTestCase):
                               group='QUOTAS')
         super(TestQuotasForServiceChain, self).tearDown()
 
-    def test_servicechain_node_quota(self):
-        self.create_servicechain_node()
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_servicechain_node)
-
-    def test_servicechain_spec_quota(self):
-        self.create_servicechain_spec()
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_servicechain_spec)
-
-    def test_servicechain_instance_quota(self):
-        self.create_servicechain_instance()
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_servicechain_instance)
-
-    def test_service_profile(self):
-        self.create_service_profile(service_type=pconst.FIREWALL)
-        self.assertRaises(webob.exc.HTTPClientError,
-                          self.create_service_profile,
-                          service_type=pconst.FIREWALL)
-
     def test_quota_implicit_service_instance(self):
         prof = self.create_service_profile(
-            service_type='LOADBALANCERV2',
+            service_type='LOADBALANCER',
             vendor="vendor")['service_profile']
 
         node1_id = self.create_servicechain_node(
@@ -895,4 +838,4 @@ class TestQuotasForServiceChain(test_base.ServiceChainPluginTestCase):
         # creation error
         self.assertRaises(webob.exc.HTTPClientError,
                           self.create_policy_target_group,
-                          provided_policy_rule_sets={prs['id']: ''})
+                          consumed_policy_rule_sets={prs['id']: ''})
