@@ -14,12 +14,12 @@ import copy
 import re
 
 import mock
+from neutron.common import constants as n_consts
+from neutron.openstack.common import uuidutils
+from neutron.plugins.common import constants
 from neutron.tests import base
 from neutron.tests.unit.api.v2 import test_base
 from neutron.tests.unit.extensions import base as test_extensions_base
-from neutron_lib import constants as n_consts
-from neutron_lib.plugins import constants
-from oslo_utils import uuidutils
 from webob import exc
 
 from gbpservice.neutron.extensions import group_policy as gp
@@ -33,8 +33,6 @@ GP_PLUGIN_BASE_NAME = (
 GROUPPOLICY_URI = 'grouppolicy'
 POLICY_TARGETS_URI = GROUPPOLICY_URI + '/' + 'policy_targets'
 POLICY_TARGET_GROUPS_URI = GROUPPOLICY_URI + '/' + 'policy_target_groups'
-APPLICATION_POLICY_GROUPS_URI = (
-    GROUPPOLICY_URI + '/' + 'application_policy_groups')
 L2_POLICIES_URI = GROUPPOLICY_URI + '/' + 'l2_policies'
 L3_POLICIES_URI = GROUPPOLICY_URI + '/' + 'l3_policies'
 POLICY_RULES_URI = GROUPPOLICY_URI + '/' + 'policy_rules'
@@ -71,9 +69,9 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
         def _is_gbp_resource(plural):
             return plural in gp.RESOURCE_ATTRIBUTE_MAP
         # Update Method
-        if re.match("^get_(create|update).+(default|)_attrs.*$", item):
+        if re.match("^get_(create|update).+(default|)_attrs$", item):
             resource = re.sub("^get_(create|update)_", "", item)
-            resource = re.sub("(_default|)_attrs.*$", "", resource)
+            resource = re.sub("(_default|)_attrs$", "", resource)
             if _is_gbp_resource(cm.get_resource_plural(resource)):
                 return getattr(cm, item)
         raise AttributeError
@@ -86,26 +84,18 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
         res = self.api.post(_get_path(POLICY_TARGETS_URI, fmt=self.fmt),
                             self.serialize(data),
                             content_type='application/%s' % self.fmt)
+        self.instance.create_policy_target.assert_called_once_with(
+            mock.ANY, policy_target=default_data)
         self.assertEqual(exc.HTTPCreated.code, res.status_int)
         res = self.deserialize(res)
         self.assertIn('policy_target', res)
-        if 'fixed_ips' in expected_value and (
-                str(res['policy_target']['fixed_ips']) ==
-                str(n_consts.ATTR_NOT_SPECIFIED)):
-            default_data['policy_target']['fixed_ips'] = (
-                    n_consts.ATTR_NOT_SPECIFIED)
-            expected_value['fixed_ips'] = res['policy_target']['fixed_ips']
-
         self.assertEqual(expected_value, res['policy_target'])
-        self.instance.create_policy_target.assert_called_once_with(
-            mock.ANY, policy_target=default_data)
 
     def test_create_policy_target_with_defaults(self):
         policy_target_id = _uuid()
         data = {'policy_target': {'policy_target_group_id': _uuid(),
                                   'tenant_id': _uuid()}}
-        default_attrs = (
-            self.get_create_policy_target_default_attrs_and_prj_id())
+        default_attrs = self.get_create_policy_target_default_attrs()
         default_data = copy.copy(data)
         default_data['policy_target'].update(default_attrs)
         expected_value = dict(default_data['policy_target'])
@@ -192,8 +182,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
     def test_create_policy_target_group_with_defaults(self):
         policy_target_group_id = _uuid()
         data = {'policy_target_group': {'tenant_id': _uuid()}}
-        default_attrs = (
-            self.get_create_policy_target_group_default_attrs_and_prj_id())
+        default_attrs = self.get_create_policy_target_group_default_attrs()
         default_data = copy.copy(data)
         default_data['policy_target_group'].update(default_attrs)
         expected_value = copy.deepcopy(default_data['policy_target_group'])
@@ -265,111 +254,6 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
     def test_delete_policy_target_group(self):
         self._test_entity_delete('policy_target_group')
 
-    def _test_create_application_policy_group(self, data, expected_value,
-                                              default_data=None):
-        if not default_data:
-            default_data = data
-
-        self.instance.create_application_policy_group.return_value = (
-            expected_value)
-        res = self.api.post(_get_path(APPLICATION_POLICY_GROUPS_URI,
-                                      fmt=self.fmt),
-                            self.serialize(data),
-                            content_type='application/%s' % self.fmt)
-        self.instance.create_application_policy_group.assert_called_once_with(
-            mock.ANY, application_policy_group=default_data)
-        self.assertEqual(exc.HTTPCreated.code, res.status_int)
-        res = self.deserialize(res)
-        self.assertIn('application_policy_group', res)
-        self.assertEqual(expected_value, res['application_policy_group'])
-
-    def test_create_application_policy_group_with_defaults(self):
-        application_policy_group_id = _uuid()
-        data = {'application_policy_group': {'tenant_id': _uuid()}}
-        default_attrs = (
-            self.get_create_application_policy_group_default_attrs_and_prj_id(
-            ))
-        default_data = copy.copy(data)
-        default_data['application_policy_group'].update(default_attrs)
-        expected_value = copy.deepcopy(
-            default_data['application_policy_group'])
-        expected_value['id'] = application_policy_group_id
-
-        self._test_create_application_policy_group(data, expected_value,
-                                                   default_data)
-
-    def test_create_application_policy_group(self):
-        application_policy_group_id = _uuid()
-        data = {'application_policy_group':
-                self.get_create_application_policy_group_attrs()}
-        expected_value = copy.deepcopy(data['application_policy_group'])
-        expected_value['id'] = application_policy_group_id
-
-        self._test_create_application_policy_group(data, expected_value)
-
-    def test_list_application_policy_groups(self):
-        application_policy_group_id = _uuid()
-        expected_value = [{'tenant_id': _uuid(),
-                           'id': application_policy_group_id}]
-
-        self.instance.get_application_policy_groups.return_value = (
-            expected_value)
-
-        res = self.api.get(_get_path(APPLICATION_POLICY_GROUPS_URI,
-                                     fmt=self.fmt))
-
-        self.instance.get_application_policy_groups.assert_called_once_with(
-            mock.ANY, fields=mock.ANY, filters=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
-        res = self.deserialize(res)
-        self.assertIn('application_policy_groups', res)
-        self.assertEqual(expected_value, res['application_policy_groups'])
-
-    def test_get_application_policy_group(self):
-        application_policy_group_id = _uuid()
-        expected_value = {'tenant_id': _uuid(),
-                          'id': application_policy_group_id}
-
-        self.instance.get_application_policy_group.return_value = (
-            expected_value)
-
-        res = self.api.get(_get_path(APPLICATION_POLICY_GROUPS_URI,
-                                     id=application_policy_group_id,
-                                     fmt=self.fmt))
-
-        self.instance.get_application_policy_group.assert_called_once_with(
-            mock.ANY, application_policy_group_id, fields=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
-        res = self.deserialize(res)
-        self.assertIn('application_policy_group', res)
-        self.assertEqual(expected_value, res['application_policy_group'])
-
-    def test_update_application_policy_group(self):
-        application_policy_group_id = _uuid()
-        update_data = {'application_policy_group':
-                       self.get_update_application_policy_group_attrs()}
-        expected_value = {'tenant_id': _uuid(),
-                          'id': application_policy_group_id}
-
-        self.instance.update_application_policy_group.return_value = (
-            expected_value)
-
-        res = self.api.put(_get_path(APPLICATION_POLICY_GROUPS_URI,
-                                     id=application_policy_group_id,
-                                     fmt=self.fmt),
-                           self.serialize(update_data))
-
-        self.instance.update_application_policy_group.assert_called_once_with(
-            mock.ANY, application_policy_group_id,
-            application_policy_group=update_data)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
-        res = self.deserialize(res)
-        self.assertIn('application_policy_group', res)
-        self.assertEqual(expected_value, res['application_policy_group'])
-
-    def test_delete_application_policy_group(self):
-        self._test_entity_delete('application_policy_group')
-
     def _test_create_l2_policy(self, data, expected_value, default_data=None):
         if not default_data:
             default_data = data
@@ -387,7 +271,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
     def test_create_l2_policy_with_defaults(self):
         l2_policy_id = _uuid()
         data = {'l2_policy': {'tenant_id': _uuid(), 'l3_policy_id': _uuid()}}
-        default_attrs = self.get_create_l2_policy_default_attrs_and_prj_id()
+        default_attrs = self.get_create_l2_policy_default_attrs()
         default_data = copy.copy(data)
         default_data['l2_policy'].update(default_attrs)
         expected_value = dict(default_data['l2_policy'])
@@ -472,7 +356,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
     def test_create_l3_policy_with_defaults(self):
         l3_policy_id = _uuid()
         data = {'l3_policy': {'tenant_id': _uuid()}}
-        default_attrs = self.get_create_l3_policy_default_attrs_and_prj_id()
+        default_attrs = self.get_create_l3_policy_default_attrs()
         default_data = copy.copy(data)
         default_data['l3_policy'].update(default_attrs)
         expected_value = dict(default_data['l3_policy'])
@@ -559,8 +443,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
     def test_create_policy_action_with_defaults(self):
         policy_action_id = _uuid()
         data = {'policy_action': {'tenant_id': _uuid()}}
-        default_attrs = (
-            self.get_create_policy_action_default_attrs_and_prj_id())
+        default_attrs = self.get_create_policy_action_default_attrs()
         default_data = copy.copy(data)
         default_data['policy_action'].update(default_attrs)
         expected_value = dict(default_data['policy_action'])
@@ -589,7 +472,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
         instance.get_policy_actions.assert_called_once_with(mock.ANY,
                                                             fields=mock.ANY,
                                                             filters=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
 
     def test_get_policy_action(self):
         policy_action_id = _uuid()
@@ -605,7 +488,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
         instance.get_policy_action.assert_called_once_with(mock.ANY,
                                                            policy_action_id,
                                                            fields=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('policy_action', res)
         self.assertEqual(expected_value, res['policy_action'])
@@ -627,7 +510,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         instance.update_policy_action.assert_called_once_with(
             mock.ANY, policy_action_id, policy_action=update_data)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('policy_action', res)
         self.assertEqual(expected_value, res['policy_action'])
@@ -654,8 +537,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
     def test_create_policy_classifier_with_defaults(self):
         policy_classifier_id = _uuid()
         data = {'policy_classifier': {'tenant_id': _uuid()}}
-        default_attrs = (
-            self.get_create_policy_classifier_default_attrs_and_prj_id())
+        default_attrs = self.get_create_policy_classifier_default_attrs()
         default_data = copy.copy(data)
         default_data['policy_classifier'].update(default_attrs)
         expected_value = dict(default_data['policy_classifier'])
@@ -684,7 +566,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         instance.get_policy_classifiers.assert_called_once_with(
             mock.ANY, fields=mock.ANY, filters=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
 
     def test_get_policy_classifier(self):
         policy_classifier_id = _uuid()
@@ -699,7 +581,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         instance.get_policy_classifier.assert_called_once_with(
             mock.ANY, policy_classifier_id, fields=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('policy_classifier', res)
         self.assertEqual(expected_value, res['policy_classifier'])
@@ -721,13 +603,13 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         instance.update_policy_classifier.assert_called_once_with(
             mock.ANY, policy_classifier_id, policy_classifier=update_data)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('policy_classifier', res)
         self.assertEqual(expected_value, res['policy_classifier'])
 
     def test_delete_policy_classifier(self):
-        self._test_entity_delete('policy_classifier')
+        self._test_entity_delete('policy_action')
 
     def _test_create_policy_rule(self, data, expected_value,
                                  default_data=None):
@@ -749,8 +631,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
         policy_rule_id = _uuid()
         data = {'policy_rule': {'tenant_id': _uuid(), 'policy_classifier_id':
                                 _uuid()}}
-        default_attrs = (
-            self.get_create_policy_rule_default_attrs_and_prj_id())
+        default_attrs = self.get_create_policy_rule_default_attrs()
         default_data = copy.copy(data)
         default_data['policy_rule'].update(default_attrs)
         expected_value = dict(default_data['policy_rule'])
@@ -780,7 +661,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
         instance.get_policy_rules.assert_called_once_with(mock.ANY,
                                                           fields=mock.ANY,
                                                           filters=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
 
     def test_get_policy_rule(self):
         policy_rule_id = _uuid()
@@ -795,7 +676,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         instance.get_policy_rule.assert_called_once_with(
             mock.ANY, policy_rule_id, fields=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('policy_rule', res)
         self.assertEqual(expected_value, res['policy_rule'])
@@ -817,13 +698,13 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         instance.update_policy_rule.assert_called_once_with(
             mock.ANY, policy_rule_id, policy_rule=update_data)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('policy_rule', res)
         self.assertEqual(expected_value, res['policy_rule'])
 
     def test_delete_policy_rule(self):
-        self._test_entity_delete('policy_rule')
+        self._test_entity_delete('policy_action')
 
     def _test_create_policy_rule_set(self, data, expected_value,
                                      default_data=None):
@@ -836,7 +717,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
                             content_type='application/%s' % self.fmt)
         self.instance.create_policy_rule_set.assert_called_once_with(
             mock.ANY, policy_rule_set=default_data)
-        self.assertEqual(exc.HTTPCreated.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPCreated.code)
         res = self.deserialize(res)
         self.assertIn('policy_rule_set', res)
         self.assertEqual(expected_value, res['policy_rule_set'])
@@ -844,8 +725,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
     def test_create_policy_rule_set_with_defaults(self):
         policy_rule_set_id = _uuid()
         data = {'policy_rule_set': {'tenant_id': _uuid()}}
-        default_attrs = (
-            self.get_create_policy_rule_set_default_attrs_and_prj_id())
+        default_attrs = self.get_create_policy_rule_set_default_attrs()
         default_data = copy.copy(data)
         default_data['policy_rule_set'].update(default_attrs)
         expected_value = dict(default_data['policy_rule_set'])
@@ -874,7 +754,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         instance.get_policy_rule_sets.assert_called_once_with(
             mock.ANY, fields=mock.ANY, filters=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
 
     def test_get_policy_rule_set(self):
         policy_rule_set_id = _uuid()
@@ -889,7 +769,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         instance.get_policy_rule_set.assert_called_once_with(
             mock.ANY, policy_rule_set_id, fields=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('policy_rule_set', res)
         self.assertEqual(expected_value, res['policy_rule_set'])
@@ -911,7 +791,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         instance.update_policy_rule_set.assert_called_once_with(
             mock.ANY, policy_rule_set_id, policy_rule_set=update_data)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('policy_rule_set', res)
         self.assertEqual(expected_value, res['policy_rule_set'])
@@ -938,8 +818,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
     def test_create_network_service_policy_with_defaults(self):
         network_service_policy_id = _uuid()
         data = {'network_service_policy': {'tenant_id': _uuid()}}
-        default_attrs = (
-            self.get_create_network_service_policy_default_attrs_and_prj_id())
+        default_attrs = self.get_create_network_service_policy_default_attrs()
         default_data = copy.copy(data)
         default_data['network_service_policy'].update(default_attrs)
         expected_value = dict(default_data['network_service_policy'])
@@ -1099,8 +978,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
         self.assertEqual(expected_value, res[entity])
 
     def test_create_external_policy_with_defaults(self):
-        default_attrs = (
-            self.get_create_external_policy_default_attrs_and_prj_id())
+        default_attrs = self.get_create_external_policy_default_attrs()
         self._test_create_entity_with_defaults('external_policy',
                                                default_attrs)
 
@@ -1123,7 +1001,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
     def test_create_external_segment_with_defaults(self):
         default_attrs = (
-            self.get_create_external_segment_default_attrs_and_prj_id())
+            self.get_create_external_segment_default_attrs())
         self._test_create_entity_with_defaults('external_segment',
                                                default_attrs)
 
@@ -1146,7 +1024,7 @@ class GroupPolicyExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
     def test_create_nat_pool_with_defaults(self):
         default_attrs = (
-            self.get_create_nat_pool_default_attrs_and_prj_id())
+            self.get_create_nat_pool_default_attrs())
         self._test_create_entity_with_defaults('nat_pool',
                                                default_attrs)
 
@@ -1175,12 +1053,12 @@ class TestGroupPolicyAttributeConverters(base.BaseTestCase):
             gp.convert_action_to_case_insensitive('ALLOW'), 'allow')
         self.assertEqual(gp.convert_action_to_case_insensitive('In'), 'in')
         self.assertEqual(gp.convert_action_to_case_insensitive('bi'), 'bi')
-        self.assertEqual('', gp.convert_action_to_case_insensitive(''))
+        self.assertEqual(gp.convert_action_to_case_insensitive(''), '')
 
     def test_convert_port_to_string(self):
         self.assertEqual(gp.convert_port_to_string(100), '100')
         self.assertEqual(gp.convert_port_to_string('200'), '200')
-        self.assertEqual('', gp.convert_port_to_string(''))
+        self.assertEqual(gp.convert_port_to_string(''), '')
 
     def test_convert_protocol_check_valid_protocols(self):
         self.assertEqual(gp.convert_protocol('tcp'), n_consts.PROTO_NAME_TCP)
@@ -1307,21 +1185,3 @@ class TestGroupPolicyAttributeValidators(base.BaseTestCase):
         incorrect = {uuid: not_a_list}
         self.assertEqual(gp._validate_external_dict(incorrect),
                          "'%s' is not a list" % not_a_list)
-
-    def test_validate_gbp_resource_name(self):
-        long_name = 'x' * 128
-        self.assertIsNone(gp._validate_gbp_resource_name(long_name))
-        invalid_name = 'x' * 129
-        msg = "'" + invalid_name + "' exceeds maximum length of 128"
-        self.assertEqual(gp._validate_gbp_resource_name(invalid_name), msg)
-
-    def test_validate_gbp_uuid(self):
-        self.assertIsNone(gp._validate_gbp_uuid_or_none(_uuid()))
-        auto_uuid = 'auto' + '9' * 32
-        self.assertIsNone(gp._validate_gbp_uuid_or_none(auto_uuid))
-        bad_uuid = 'autt' + '9' * 32
-        self.assertEqual(gp._validate_gbp_uuid_or_none(bad_uuid),
-                         "'%s' is not a valid UUID" % bad_uuid)
-        bad_uuid = 'auto' + 'z' * 32
-        self.assertEqual(gp._validate_gbp_uuid_or_none(bad_uuid),
-                         "'%s' is not a valid UUID" % bad_uuid)
