@@ -13,10 +13,10 @@
 import copy
 
 import mock
+from neutron.openstack.common import uuidutils
+from neutron.plugins.common import constants
 from neutron.tests.unit.api.v2 import test_base
 from neutron.tests.unit.extensions import base as test_extensions_base
-from neutron_lib.plugins import constants
-from oslo_utils import uuidutils
 from webob import exc
 
 from gbpservice.neutron.extensions import servicechain
@@ -59,7 +59,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
         self.instance.create_servicechain_node.assert_called_once_with(
             mock.ANY, servicechain_node=default_data)
 
-        self.assertEqual(exc.HTTPCreated.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPCreated.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_node', res)
         self.assertEqual(expected_value, res['servicechain_node'])
@@ -74,8 +74,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
                 'service_type': None,
             }
         }
-        default_attrs = (
-            cm.get_create_servicechain_node_default_attrs_and_prj_id())
+        default_attrs = cm.get_create_servicechain_node_default_attrs()
         default_data = copy.copy(data)
         default_data['servicechain_node'].update(default_attrs)
         expected_value = dict(default_data['servicechain_node'])
@@ -102,7 +101,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         self.instance.get_servicechain_nodes.assert_called_once_with(
             mock.ANY, fields=mock.ANY, filters=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_nodes', res)
         self.assertEqual(expected_value, res['servicechain_nodes'])
@@ -119,7 +118,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         self.instance.get_servicechain_node.assert_called_once_with(
             mock.ANY, servicechain_node_id, fields=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_node', res)
         self.assertEqual(expected_value, res['servicechain_node'])
@@ -139,7 +138,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         self.instance.update_servicechain_node.assert_called_once_with(
             mock.ANY, servicechain_node_id, servicechain_node=update_data)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_node', res)
         self.assertEqual(expected_value, res['servicechain_node'])
@@ -159,7 +158,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         self.instance.create_servicechain_spec.assert_called_once_with(
             mock.ANY, servicechain_spec=default_data)
-        self.assertEqual(exc.HTTPCreated.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPCreated.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_spec', res)
         self.assertEqual(expected_value, res['servicechain_spec'])
@@ -171,8 +170,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
                 'nodes': [_uuid(), _uuid()], 'tenant_id': _uuid()
             }
         }
-        default_attrs = (
-            cm.get_create_servicechain_spec_default_attrs_and_prj_id())
+        default_attrs = cm.get_create_servicechain_spec_default_attrs()
         default_data = copy.copy(data)
         default_data['servicechain_spec'].update(default_attrs)
         expected_value = dict(default_data['servicechain_spec'])
@@ -199,7 +197,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         self.instance.get_servicechain_specs.assert_called_once_with(
             mock.ANY, fields=mock.ANY, filters=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_specs', res)
         self.assertEqual(expected_value, res['servicechain_specs'])
@@ -215,7 +213,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         self.instance.get_servicechain_spec.assert_called_once_with(
             mock.ANY, servicechain_spec_id, fields=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_spec', res)
         self.assertEqual(expected_value, res['servicechain_spec'])
@@ -235,7 +233,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         self.instance.update_servicechain_spec.assert_called_once_with(
             mock.ANY, servicechain_spec_id, servicechain_spec=update_data)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_spec', res)
         self.assertEqual(expected_value, res['servicechain_spec'])
@@ -256,7 +254,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         self.instance.create_servicechain_instance.assert_called_once_with(
             mock.ANY, servicechain_instance=default_data)
-        self.assertEqual(exc.HTTPCreated.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPCreated.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_instance', res)
         self.assertEqual(expected_value, res['servicechain_instance'])
@@ -273,8 +271,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
                 'classifier_id': _uuid(),
             }
         }
-        default_attrs = (
-            cm.get_create_servicechain_instance_default_attrs_and_prj_id())
+        default_attrs = cm.get_create_servicechain_instance_default_attrs()
         default_data = copy.copy(data)
         default_data['servicechain_instance'].update(default_attrs)
         expected_value = dict(default_data['servicechain_instance'])
@@ -302,7 +299,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         self.instance.get_servicechain_instances.assert_called_once_with(
             mock.ANY, fields=mock.ANY, filters=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_instances', res)
         self.assertEqual(expected_value, res['servicechain_instances'])
@@ -318,7 +315,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
 
         self.instance.get_servicechain_instance.assert_called_once_with(
             mock.ANY, servicechain_instance_id, fields=mock.ANY)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_instance', res)
         self.assertEqual(expected_value, res['servicechain_instance'])
@@ -339,7 +336,7 @@ class ServiceChainExtensionTestCase(test_extensions_base.ExtensionTestCase):
         self.instance.update_servicechain_instance.assert_called_once_with(
             mock.ANY, servicechain_instance_id,
             servicechain_instance=update_data)
-        self.assertEqual(exc.HTTPOk.code, res.status_int)
+        self.assertEqual(res.status_int, exc.HTTPOk.code)
         res = self.deserialize(res)
         self.assertIn('servicechain_instance', res)
         self.assertEqual(expected_value, res['servicechain_instance'])
