@@ -154,9 +154,9 @@ function run_tests {
 }
 
 function copy_subunit_log {
-  LOGNAME=`cat .stestr/next-stream`
+  LOGNAME=`cat .testrepository/next-stream`
   LOGNAME=$(($LOGNAME - 1))
-  LOGNAME=".stestr/${LOGNAME}"
+  LOGNAME=".testrepository/${LOGNAME}"
   cp $LOGNAME subunit.log
 }
 
